/*
 * (c) Copyright 1992 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: Video.h,v $
 * Revision 1.12  1993/11/19  04:48:54  drapeau
 * Added support for SVR4.  Changes too numerous to mention here.
 *
 * Revision 1.3  91/08/19  12:40:24  eric
 * added rcs header
 * 
 *
 */

#ifdef H_ID
#ifndef lint
static char *rcsid_video_h = "@(#)$Id: Video.h,v 1.12 1993/11/19 04:48:54 drapeau Exp $ Parallax Graphics Inc";
#endif
#endif

/*
 *
 *	video.h - Include file for XView interface to XLIB video extensions
 *
 *	Written by John Malleo-Roach
 *
 */

#ifndef VIDEO_INCLUDE
#define VIDEO_INCLUDE

/* INCLUDE FILES */

#include <sys/ipc.h>
#include <sys/shm.h>
#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/notify.h>
#include <X11/extensions/XShm.h>

/* CONSTANTS */

#define VIDEO_IN  &video_in_pkg
#define VIDEO_OUT &video_out_pkg

#define ATTR_PKG_VIDEO_IN				ATTR_PKG_UNUSED_FIRST
#define ATTR_PKG_VIDEO_OUT				ATTR_PKG_UNUSED_FIRST+1

#define VIDEO_IN_ATTR(type, ordinal)	ATTR(ATTR_PKG_VIDEO_IN,  type, ordinal)
#define VIDEO_OUT_ATTR(type, ordinal)	ATTR(ATTR_PKG_VIDEO_OUT, type, ordinal)

/* TYPEDEFS */

typedef Xv_opaque VideoIn;
typedef Xv_opaque VideoOut;

enum {

/* Error codes placed into global variable Xv_VideoStatus */

	VIDEO_OKAY,					/* Operation completed successfully.		*/
	VIDEO_UNKNOWN,				/* An unknown error was detected.			*/
	VIDEO_NO_HARDWARE,			/* No video hardware could be located.		*/
	VIDEO_NO_MORE_CHANNELS,		/* No more video channels available. 		*/
	VIDEO_CHANNEL_IN_USE,		/* Requested channel is already in use.		*/
	VIDEO_BAD_FORMAT,			/* Bad format for specified channel.		*/
	VIDEO_BAD_STANDARD,			/* Bad standard for specified channel.		*/
	VIDEO_BAD_INPUT_TYPE,		/* Bad input type for specified channel.	*/
	VIDEO_BAD_OUTPUT_TYPE,		/* Bad output type for specified channel.	*/
	VIDEO_NO_SYNC_DETECTED,		/* No sync was detected on requested channel*/
	VIDEO_INVALID_OPERATION,	/* Invalid operation attempted on object	*/

/* Valid values for attribute VIDEO_GRAHICS */

	VIDEO_GRAPHICS_1,			/* One bit graphics							*/
	VIDEO_GRAPHICS_8,			/* Eight bit graphics						*/
	VIDEO_GRAPHICS_24,			/* Twenty four bit graphics					*/
	VIDEO_PIXELS,				/* Video pixels								*/
	VIDEO_OVERLAY,				/* Graphics overlay							*/

/* Valid values for attribute VIDEO_INPUT_TYPE */

	VIDEO_YUV7,					/* Seven bit YUV format for 1280 for Viper	*/
	VIDEO_RGB24,				/* Twenty four bit RGB (True Color)			*/
	VIDEO_RGB8,					/* Eight bit Pseudo color					*/
	VIDEO_GRAY7,				/* Seven bit grayscale (bit <0> = 0)		*/
	VIDEO_GRAY8,				/* Eight bit grayscale						*/

/* Valid values for attribute VIDEO_FORMAT */

	VIDEO_COMPOSITE,			/* Composite NTSC/PAL video signal			*/
	VIDEO_SUPER_VHS,			/* SuperVHS (1 luma, 2 chroma, and sync)	*/
	VIDEO_YUV,					/* YUV Component							*/
	VIDEO_RGB,					/* RGB Component							*/

/* Valid values for attribute VIDEO_CHANNEL */

	VIDEO_IO_OFF,				/* Disable all channels						*/
	VIDEO_IN_ONE,				/* Video input channel 1					*/
	VIDEO_IN_TWO,				/* Video input channel 2					*/
	VIDEO_OUT_ONE,				/* Video output channel 1					*/

/* Valid values for attribute VIDEO_STANDARD */

	VIDEO_NTSC,					/* American and Japanese					*/
	VIDEO_PAL,					/* European									*/
	VIDEO_SECAM,				/* Variation on PAL							*/

/* Valid values for attribute VIDEO_HARDWARE */

	VIDEO_1280,					/* Old time Parallax 1280 series			*/
	VIDEO_VIPER,				/* Viper Series								*/
	VIDEO_VIDEOVIEW,			/* Sun VideoView							*/
	VIDEO_XVIDEO_24,			/* 24 bit XVideo board						*/
	VIDEO_XVIDEO_8				/*  8 bit XVideo board						*/
};

/* Valid attributes for VIDEO_IN */

enum {
	VIDEO_LIVE				= VIDEO_IN_ATTR(ATTR_BOOLEAN,		1),
	VIDEO_SCALE				= VIDEO_IN_ATTR(ATTR_BOOLEAN,		2),
	VIDEO_SQUEEZE			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		3),
	VIDEO_JPEG_ZOOM			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		4),
	VIDEO_NEW_FRAME			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		5),
	VIDEO_FACTORY_RESET		= VIDEO_IN_ATTR(ATTR_BOOLEAN,		6),
	VIDEO_USER0				= VIDEO_IN_ATTR(ATTR_BOOLEAN,		7),
	VIDEO_USER1				= VIDEO_IN_ATTR(ATTR_BOOLEAN,		8),
	VIDEO_SYNC				= VIDEO_IN_ATTR(ATTR_BOOLEAN,		9),
	VIDEO_INTERLACED		= VIDEO_IN_ATTR(ATTR_BOOLEAN,		10),
	VIDEO_NORMAL_ASPECT		= VIDEO_IN_ATTR(ATTR_BOOLEAN,		11),
	VIDEO_TRACK_WIN			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		12),
	VIDEO_EXPOSED			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		13),
	VIDEO_USE_SHM			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		14),
	VIDEO_SHM_DONE			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		15),
	VIDEO_JPEG_ONLY			= VIDEO_IN_ATTR(ATTR_BOOLEAN,		16),
	VIDEO_X					= VIDEO_IN_ATTR(ATTR_INT,			17),
	VIDEO_Y					= VIDEO_IN_ATTR(ATTR_INT,			18),
	VIDEO_WIDTH				= VIDEO_IN_ATTR(ATTR_INT,			19),
	VIDEO_SIGNAL_WIDTH		= VIDEO_IN_ATTR(ATTR_INT,			20),
	VIDEO_HEIGHT			= VIDEO_IN_ATTR(ATTR_INT,			21),
	VIDEO_SIGNAL_HEIGHT		= VIDEO_IN_ATTR(ATTR_INT,			22),
	VIDEO_INTERVAL			= VIDEO_IN_ATTR(ATTR_INT,			23),
	VIDEO_STANDARD			= VIDEO_IN_ATTR(ATTR_ENUM,			24),
	VIDEO_FORMAT			= VIDEO_IN_ATTR(ATTR_ENUM,			25),
	VIDEO_INPUT_TYPE		= VIDEO_IN_ATTR(ATTR_ENUM,			26),
	VIDEO_CHANNEL			= VIDEO_IN_ATTR(ATTR_ENUM,			27),
	VIDEO_SATURATION		= VIDEO_IN_ATTR(ATTR_INT,			28),
	VIDEO_HUE				= VIDEO_IN_ATTR(ATTR_INT,			29),
	VIDEO_CONTRAST			= VIDEO_IN_ATTR(ATTR_INT,			30),
	VIDEO_BRIGHTNESS		= VIDEO_IN_ATTR(ATTR_INT,			31),
	VIDEO_HARDWARE			= VIDEO_IN_ATTR(ATTR_ENUM,			32),
	VIDEO_GRAPHICS			= VIDEO_IN_ATTR(ATTR_ENUM,			33),
	VIDEO_PIXRECT			= VIDEO_IN_ATTR(ATTR_PIXRECT_PTR,	34),
	VIDEO_XIMAGE			= VIDEO_IN_ATTR(ATTR_OPAQUE,		35),
	VIDEO_EVENT_PROC		= VIDEO_IN_ATTR(ATTR_FUNCTION_PTR,	36),
	VIDEO_Q_FACTOR			= VIDEO_IN_ATTR(ATTR_SHORT,			37),
	VIDEO_JPEG_IMAGE		= VIDEO_IN_ATTR(ATTR_OPAQUE,		38),
	VIDEO_SAVE_STILL		= VIDEO_IN_ATTR(ATTR_BOOLEAN,		39),
	VIDEO_GC				= VIDEO_IN_ATTR(ATTR_OPAQUE,		40),
/* Private defines */
	VIDEO_SHM_PTR			= VIDEO_IN_ATTR(ATTR_OPAQUE,		41),
	VIDEO_COLOR             = VIDEO_IN_ATTR(ATTR_BOOLEAN,       42),
	VIDEO_CONF_SIZE			= VIDEO_IN_ATTR(ATTR_INT,			43),
	VIDEO_QTABLE			= VIDEO_IN_ATTR(ATTR_OPAQUE,		44),
	VIDEO_FAST_FRAME        = VIDEO_IN_ATTR(ATTR_OPAQUE,        45)
};

/* Definition of a JPEG compressed image */

typedef struct {
	int   width;
	int   height;
	int   size;
	char *data;
	XShmSegmentInfo *info;
} JPEGImage;

typedef struct {
	Xv_canvas	parent_data;
	Xv_opaque	private_data;
} VideoInPublic;

/* Valid attributes for VIDEO_OUT */

enum {
	VIDEO_OUTPUT			= VIDEO_OUT_ATTR(ATTR_BOOLEAN,		 1),
	VIDEO_OUT_SCALE			= VIDEO_OUT_ATTR(ATTR_BOOLEAN,		 2),
	VIDEO_OUT_ASPECT		= VIDEO_OUT_ATTR(ATTR_BOOLEAN,		 3),
	VIDEO_OUT_X				= VIDEO_OUT_ATTR(ATTR_INT,			 4),
	VIDEO_OUT_Y				= VIDEO_OUT_ATTR(ATTR_INT,			 5),
	VIDEO_OUT_WIDTH			= VIDEO_OUT_ATTR(ATTR_INT,			 6),
	VIDEO_OUT_HEIGHT		= VIDEO_OUT_ATTR(ATTR_INT,			 7),
	VIDEO_OUT_STANDARD		= VIDEO_OUT_ATTR(ATTR_INT,			 8),
	VIDEO_OUT_FORMAT		= VIDEO_OUT_ATTR(ATTR_INT,			 9),
	VIDEO_OUTPUT_TYPE		= VIDEO_OUT_ATTR(ATTR_INT,			10),
	VIDEO_OUT_CHANNEL		= VIDEO_OUT_ATTR(ATTR_INT,			11),
	VIDEO_OUT_HARDWARE		= VIDEO_OUT_ATTR(ATTR_INT,			12),
};

typedef struct {
	Xv_generic_struct	parent_data;
	Xv_opaque			private_data;
} VideoOutPublic;

/* EXTERNAL VARIABLES */

extern int    Xv_VideoStatus;

extern Xv_pkg video_in_pkg;
extern Xv_pkg video_out_pkg;

#endif

/* end of video.h */
