/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: VideoImpl.h,v $
 * Revision 1.11  1993/11/19  04:48:54  drapeau
 * Added support for SVR4.  Changes too numerous to mention here.
 *
 * Revision 1.2  91/08/19  12:40:38  eric
 * added rcs header
 * 
 *
 */

#ifdef H_ID
#ifndef lint
static char *rcsid_VideoImpl_h = "@(#)$Id: VideoImpl.h,v 1.11 1993/11/19 04:48:54 drapeau Exp $ Parallax Graphics Inc";
#endif
#endif

/*
 *
 *	video_impl.h - Private implementation for XView video support
 *
 *	Written by John Malleo-Roach
 *
 */

/* INCLUDE FILES */

#include "Video.h"

/* CONSTANTS */

#define VideoLiveFlag		0x0000001
#define VideoScaleFlag		0x0000002
#define VideoSqueezeFlag	0x0000004
#define VideoJPEGFlag		0x0000008

#define VideoRepaintFlag	0x0000010
#define VideoCreateFlag		0x0000020
#define VideoValueFlag		0x0000040
#define VideoIOFlag			0x0000080

#define VideoSyncFlag		0x0000100
#define VideoLacedFlag		0x0000200
#define VideoAspectFlag		0x0000400
#define VideoCalledFlag		0x0000800

#define VideoResizeFlag		0x0001000
#define VideoAdjustFlag		0x0002000
#define VideoOutFlag		0x0004000
#define VideoFirstComp		0x0008000

#define VideoFirstDeComp	0x0010000
#define VideoGCIsOut		0x0020000
#define VideoChannelFlag	0x0040000
#define VideoFormatFlag		0x0080000

#define VideoStandardFlag	0x0100000
#define VideoInputFlag		0x0200000
#define VideoSizeFlag		0x0400000
#define VideoTrackWinFlag	0x0800000

#define VideoUseShmFlag		0x1000000
#define VideoJPEGZoomFlag	0x2000000
#define VideoSaveStillFlag  0x4000000

#define Boolean				int

/* TYPEDEFS */

typedef struct {
	Xv_object	public_self;		/* Pointer back to self				*/
	int			flags;				/* Store different video flags		*/
	int 		vx;					/* Source video x location			*/
	int 		dx;					/* Destination video x location		*/
	int			vy;					/* Source video y location			*/
	int			dy;					/* Destination video y location		*/
	int			vw;					/* Source video active width		*/
	int			dw;					/* Destination video active width	*/
	int			vh;					/* Source video active height		*/
	int			dh;					/* Destination video active height	*/
	int			width;				/* Video signal width				*/
	int			height;				/* Video signal height				*/
	int			interval;			/* Video signal interval			*/
	int			standard;			/* NTSC, PAL, or SECAM				*/
	int			p_standard;			/* Previous input standard			*/
	int			format;				/* Composite, SVHS, YUV, RGB		*/
	int			p_format;			/* Previous video format			*/
	int			input_type;			/* RGB, GRAY, YUV and their depths	*/
	int			p_input_type;		/* Previous input type				*/
	int			channel;			/* I/O channel being used			*/
	int			p_channel;			/* Previous I/O channel				*/
	int			saturation;			/* Visual parameters for the video	*/
	int			hue;
	int			contrast;
	int			brightness;
	Display		*dpy;				/* Current X windows display		*/
	Xv_Window	pw;					/* Canvas paint window				*/
	Drawable	pw_xid;				/* Paint window X id				*/
	GC			pw_gc;				/* Xid graphics context             */
	int			tagbits;			/* Video plane tag bits for input   */
	Rect		exposure;			/* Last exposure rectangle			*/
	short		QFactor;			/* Quantization factor for JPEG		*/
	u_char      *QTable;			/* Quantization table for JPEG		*/
	short		QSize;				/* Size of quantization table		*/
	JPEGImage	*ImageCache;		/* Pointer to shared memory			*/
	XImage      *SavedStill;        /* Pointer to saved video still     */
	int         conf_size;          /* Number of conference regions     */
} VideoInPrivate;

typedef struct {
	Xv_object	public_self;		/* Pointer back to self				*/
	int			flags;				/* Store different video flags		*/
	int 		vx;					/* Destination video x location		*/
	int 		sx;					/* Source video x location			*/
	int 		lx;					/* Source video x location			*/
	int			vy;					/* Destination video y location		*/
	int			sy;					/* Source video y location			*/
	int			ly;					/* Source video y location			*/
	int			vw;					/* Destination video active width	*/
	int			sw;					/* Source video active width		*/
	int			lw;					/* Source video active width		*/
	int			vh;					/* Destination video active height	*/
	int			sh;					/* Source video active height		*/
	int			lh;					/* Source video active height		*/
	int			width;				/* Video signal width				*/
	int			height;				/* Video signal height				*/
	int			interval;			/* Video signal interval			*/
	int			standard;			/* NTSC, PAL, or SECAM				*/
	int			p_standard;			/* Previous input standard			*/
	int			format;				/* Composite, SVHS, YUV, RGB		*/
	int			p_format;			/* Previous video format			*/
	int			output_type;		/* RGB, GRAY, YUV and their depths	*/
	int			p_output_type;		/* Previous input type				*/
	int			channel;			/* I/O channel being used			*/
	int			p_channel;			/* Previous I/O channel				*/
	Display		*dpy;				/* Current X windows display		*/
	Drawable	root_xid;			/* Root window X id					*/
	GC			root_gc;			/* Root window graphics context		*/
} VideoOutPrivate;

#define VIDEO_PUBLIC(item)  XV_PUBLIC(item)

#define VIDEO_IN_PRIVATE(item)  XV_PRIVATE(VideoInPrivate, VideoInPublic, item)
#define VIDEO_OUT_PRIVATE(item) XV_PRIVATE(VideoOutPrivate, VideoOutPublic, item)

/* end of VideoImpl.h */


