/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/XVideoIn.h,v 1.15 1994/10/26 01:52:12 drapeau Exp $ */
/* $Log: XVideoIn.h,v $
 * Revision 1.15  1994/10/26  01:52:12  drapeau
 * Added a new resource, "drawMethod", to the widget, allowing
 * application programmers to draw graphics over the video without the
 * graphics being overwritten by video.
 *
 * Revision 1.14  1994/08/27  06:13:43  drapeau
 * Added a new resource, autoSearch, that allows the widget to
 * automatically search for a valid video input on all channels if the
 * programmer's first choice has no valid video.  By default, this is set
 * to False.
 *
 * Revision 1.13  1994/07/20  21:52:15  drapeau
 * Added a resource "instance" to reflect the new model of the hardware.
 * For example, the PowerVideo/700 product has at least two composite
 * input cables available on either digitizer; the "instance" field is
 * used to specify which is to be used.
 *
 * Also made the "hardwareParams" field public so that applications have
 * better information about the hardware being used.
 *
 * Revision 1.12  1994/05/03  17:57:07  drapeau
 * Added a resource called "diagnostics"; when set, the widget will print
 * diagnostic messages about itself to aid in debugging.
 *
 * Revision 1.11  1993/10/26  17:58:20  drapeau
 * Small modification: moved RCS id variable inside the ifndef statement
 * to avoid multiple inclusions.
 *
 * Revision 1.10  1993/09/10  12:15:00  drapeau
 * This is the first public release of the VideoIn widget header files.
 * */
#ifndef _XVideoIn_h
#define _XVideoIn_h
static char xVideoInHeaderRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/XVideoIn.h,v 1.15 1994/10/26 01:52:12 drapeau Exp $";


/****************************************************************
 *
 * XVideoIn widget
 *
 ****************************************************************/

/* Resources specific to this widget:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 brightness	     Brightness		Int		DefaultBrightness
 contrast	     Contrast		Int		DefaultContrast
 hue		     Hue		Int		DefaultHue
 saturation	     Saturation		Int		DefaultSaturation
 cropped	     Cropped		Boolean		True
 normalAspectRatio   NormalAspectRatio	Boolean		True
 sourceRect	     SourceRect		XRectangle*	NULL
 displayRect	     DisplayRect	XRectangle*	NULL
 live		     Live		Boolean		True
 newFrame	     NewFrame		Boolean		False
 saveColorSettings   saveColorSettings	Int		FACTORY
 restoreColorSettings restoreColorSettings Int		FACTORY
 signalStandard	     SignalStandard	Int		PLX_NTSC
 signalFormat	     SignalFormat	Int		PLX_COMP
 hardwareType	     HardwareType	Int		PLX_XVIDEO_24
 interlaced	     Interlaced		Boolean		True
 sync		     Sync		Boolean		True
 signalHeight	     SignalHeight	Dimension	480
 signalWidth	     SignalWidth	Dimension	640
 blankIntervalHeight blankIntervalHeight	Dimension	45
 showBlankInterval   ShowBlankInterval	Boolean		False
 channel	     Channel		Int		PLX_INPUT_0
 videoParameterString VideoParameterString String	"COLOR"
 resizeParent	     ResizeParent	Boolean		True
 diagnostics	     Diagnostics	Boolean		False
 hardwareParams	     HardwareParams	plx_IO*		NULL
 instance	     Instance		int		0
 autoSearch	     AutoSearch		Boolean		False
 drawMethod	     DrawMethod		DrawMethod	NULL
*/

#define XtNbrightness		"brightness"
#define XtNcontrast		"contrast"
#define XtNhue			"hue"
#define XtNsaturation		"saturation"
#define XtNcropped		"cropped"
#define XtNnormalAspectRatio	"normalAspectRatio"
#define XtNsourceRect		"sourceRect"
#define XtNdisplayRect		"displayRect"
#define XtNlive			"live"
#define XtNnewFrame		"newFrame"
#define XtNsaveColorSettings	"saveColorSettings"
#define XtNrestoreColorSettings	"restoreColorSettings"
#define XtNsignalStandard	"signalStandard"
#define XtNsignalFormat		"signalFormat"
#define XtNhardwareType		"hardwareType"
#define XtNinterlaced		"interlaced"
#define XtNsync			"sync"
#define XtNsignalHeight		"signalHeight"
#define XtNsignalWidth		"signalWidth"
#define XtNblankIntervalHeight	"blankIntervalHeight"
#define XtNshowBlankInterval	"showBlankInterval"
#define XtNchannel		"channel"
#define XtNvideoParameterString	"videoParameterString"
#define XtNresizeParent		"resizeParent"
#define XtNdiagnostics		"diagnostics"
#define XtNhardwareParams	"hardwareParams"
#define XtNinstance		"instance"
#define XtNautoSearch		"autoSearch"
#define XtNdrawMethod		"drawMethod"

#define XtCBrightness		"Brightness"
#define XtCContrast		"Contrast"
#define XtCHue			"Hue"
#define XtCSaturation		"Saturation"
#define XtCCropped		"Cropped"
#define XtCNormalAspectRatio	"NormalAspectRatio"
#define XtCSourceRect		"SourceRect"
#define XtCDisplayRect		"DisplayRect"
#define XtCLive			"Live"
#define XtCNewFrame		"NewFrame"
#define XtCSaveColorSettings	"SaveColorSettings"
#define XtCRestoreColorSettings	"RestoreColorSettings"
#define XtCSignalStandard	"SignalStandard"
#define XtCSignalFormat		"SignalFormat"
#define XtCHardwareType		"HardwareType"
#define XtCInterlaced		"Interlaced"
#define XtCSync			"Sync"
#define XtCSignalHeight		"SignalHeight"
#define XtCSignalWidth		"SignalWidth"
#define XtCBlankIntervalHeight	"BlankIntervalHeight"
#define XtCShowBlankInterval	"ShowBlankInterval"
#define XtCChannel		"Channel"
#define XtCVideoParameterString	"VideoParameterString"
#define XtCResizeParent		"ResizeParent"
#define XtCDiagnostics		"Diagnostics"
#define XtCHardwareParams	"HardwareParams"
#define XtCInstance		"Instance"
#define XtCAutoSearch		"AutoSearch"
#define XtCDrawMethod		"DrawMethod"

/* declare specific XVideoInWidget class and instance datatypes */

typedef struct _XVideoInClassRec*	XVideoInWidgetClass;
typedef struct _XVideoInRec*		XVideoInWidget;

/* declare the class constant */

extern WidgetClass xVideoInWidgetClass;

#endif /* _XVideoIn_h */
