/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/XVideoInP.h,v 1.16 1994/10/26 01:52:04 drapeau Exp $ */
/* $Log: XVideoInP.h,v $
 * Revision 1.16  1994/10/26  01:52:04  drapeau
 * Added a new resource, "drawMethod", to the widget, allowing
 * application programmers to draw graphics over the video without the
 * graphics being overwritten by video.
 *
 * Revision 1.15  1994/10/14  21:15:09  drapeau
 * Added structure fields to hold information event handlers that may be
 * installed on the widget.
 *
 * Revision 1.14  1994/08/27  06:14:07  drapeau
 * Added a new resource, autoSearch, that allows the widget to
 * automatically search for a valid video input on all channels if the
 * programmer's first choice has no valid video.  By default, this is set
 * to False.
 *
 * Revision 1.13  1994/07/20  21:52:48  drapeau
 * Added a resource "instance" to reflect the new model of the hardware.
 * For example, the PowerVideo/700 product has at least two composite
 * input cables available on either digitizer; the "instance" field is
 * used to specify which is to be used.
 *
 * Also made the "hardwareParams" field public so that applications have
 * better information about the hardware being used.
 *
 * Revision 1.12  1994/06/09  22:14:30  drapeau
 * Changed the implementation of still images in the VideoIn widget to
 * use XImages instead of Pixmaps.  This was done to support the HP
 * platform.
 *
 * Revision 1.11  1994/05/03  17:57:15  drapeau
 * Added a resource called "diagnostics"; when set, the widget will print
 * diagnostic messages about itself to aid in debugging.
 *
 * Revision 1.10  1993/09/10  12:14:37  drapeau
 * This is the first public release of the VideoIn widget header files.
 * */
static char xVideoInPHeaderRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/XVideoInP.h,v 1.16 1994/10/26 01:52:04 drapeau Exp $";

#ifndef _XVideoInP_h
#define _XVideoInP_h

#include <XVideoIn.h>
#include <X11/CoreP.h>						    /* Include superclass private header file */
#include <XPlxExt.h>						    /* Include file for X extensions for Parallax video */


/* define unique representation types not found in <X11/StringDefs.h> */

#define XtRXVideoInResource "XVideoInResource"


/* Defined constants used internally by this widget */

#define DefaultBrightness	128
#define DefaultContrast		128
#define DefaultHue		128
#define DefaultSaturation	128
#define NoNVRAM			-1

typedef struct
{
  int empty;
} XVideoInClassPart;

typedef struct _XVideoInClassRec
{
  CoreClassPart		core_class;
  XVideoInClassPart	xVideoIn_class;
} XVideoInClassRec;

extern XVideoInClassRec xVideoInClassRec;

typedef struct
{
  int		brightness;					    /* Public resources */
  int		contrast;
  int		hue;
  int		saturation;
  Boolean	cropped;					    /* Indicates whether video be cropped or squeezed */
  Boolean	normalAspectRatio;				    /* Whether aspect ratio should be 4:3 (normal) or not */
  XRectangle*	sourceRect;					    /* Rectangle for the incoming video signal */
  XRectangle*	displayRect;					    /* Rectangle in which digitized video signal is displayed */
  Boolean	live;						    /* Whether video should be shown live or still-frame */
  Boolean	newFrame;					    /* Whether to grab a new frame of video for digitization */
  int		saveColorSettings;				    /* Specifies where to save current color settings */
  int		restoreColorSettings;				    /* Where to load color settings: from factory, user0, user1 */
  int		signalStandard;					    /* Which video standard to be used: NTSC, PAL, SECAM */
  int		signalFormat;					    /* Format of the video input: RGB, SVHS, etc. */
  int		hardwareType;					    /* Indicates board being used by the computer */
  Boolean	interlaced;					    /* Read-only: whether current signal is an interlaced signal */
  Boolean	sync;						    /* Read-only: whether valid sync signal is detected */
  Dimension	signalWidth;					    /* Number of rows and columns provided by the video... */
  Dimension	signalHeight;					    /* ...standard currently selected as input */
  Dimension	blankIntervalHeight;				    /* Number of rows for blanking interval for current input */
  Boolean	showBlankInterval;				    /* Whether to digitize blanking interval with rest of signal */
  int		channel;					    /* Indicates channel on which to look for video signal */
  String	videoParameterString;				    /* String specifies simple special effects for the board */
  Boolean	resizeParent;					    /* Whether to try and resize the widget's parent when... */
								    /* ...this widget is resized */
  int		instance;					    /* Which instance of an input to use on a digitizer */
  Boolean	diagnostics;					    /* Whether to print widget diagnostics */
  plx_IO*	hardwareParams;					    /* Holds all relevant info about the board being used. */
  XtExposeProc	drawMethod;					    /* Hook for app pgmr to draw his/her own graphics in widget */
  
  GC		gc;						    /* Private variables */
  Dimension	startingY;					    /* Y coordinate, where to begin digitizing video after... */
								    /* ....correcting for the blanking interval height. */
  XImage*	stillImagePixmap;				    /* Used to hold a still image when "live" resource is False */
  Boolean	pixmapExists;					    /* Is there a Pixmap to be used for displaying still video? */
  XRectangle	defaultSourceRect;
  XRectangle	defaultDisplayRect;
  Window	videoWindow;					    /* Points to window used for XPlx extension calls */
  Boolean	autoSearch;					    /* Whether to auto-search for a valid input signal */
  Bool		eventHandlerInstalled;				    /* Keeps track of whether the widget looks for window... */
  Widget	eventHandlerWidget;				    /* ...move events, and on which widget. */
} XVideoInPart;

typedef struct _XVideoInRec
{
  CorePart	core;
  XVideoInPart	xVideoIn;
} XVideoInRec;

#endif /* _XVideoInP_h */
