/* $Header: /plx/S/source/Xclients/solaris2.4/3.4a/src/client/motif/MovieTool/RCS/MovieTool.c,v 1.28.1.1 1995/04/05 18:35:25 lovvik Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: MovieTool.c,v $
 * Revision 1.28.1.1  1995/04/05  18:35:25  lovvik
 * 2 changes:
 *   1)  Modified the autoPlay option to call the Play() function
 *       after entering XtAppMainLoop.  This fixed some synchronization
 * 	 problems when using the -autoPlay option.
 *
 *   2)  Added some code to catch window manager delete calls.
 *
 * Revision 1.28  1994/08/23  01:28:48  drapeau
 * Changed the default sampling rate to 48K.  Also added extra interface
 * code for an alert dialog box.
 *
 * Revision 1.27  1994/08/11  03:12:59  drapeau
 * Added proper X resource management, mostly adding new data structures
 * and variables to hold app-specific resources, but also enhancing the
 * CheckOptions() function to check for these resources.
 *
 * Also, simplified the include file structure, now only including a main
 * program header file instead of a messy series of header files.
 *
 * Revision 1.26  1994/06/24  01:13:21  drapeau
 * Fixed the Usage() function so that messages are printed more nicely on
 * the terminal.
 *
 * Revision 1.25  1994/06/10  01:20:27  drapeau
 * Added the Usage() function to explain the command-line options
 * available from the application.
 *
 * Also, added the option "silentMovie" to force MovieTool to avoid using
 * the audio device.
 *
 * Revision 1.24  1994/05/31  19:53:08  drapeau
 * Added command-line options to tell MovieTool to begin playback of a
 * movie immediately upon launch (if a movie filename is specified on the
 * command line, of course), and to quit immediately when finished
 * playing the movie.
 *
 * Also made diagnostic reporting more robust.  Diagnostic messages
 * appear on screen only if the "-diagnostics" flag is set on the command
 * line.
 *
 * Revision 1.23  1994/01/20  18:00:59  drapeau
 * Minor modification to support the new PortableAudio interface
 * developed to make MovieTool more portable across platforms.
 *
 * Revision 1.22  1994/01/10  20:03:46  drapeau
 * Added support for the MAEstro Multimedia Authoring Environment;
 * specifically, added the CheckOptions() function to allow MovieTool to
 * connect with Port Managers on other machines, also to allow turning on
 * and off of diagnostics, and to allow variable audio drift.
 *
 * Revision 1.21  1993/12/03  02:02:57  drapeau
 * No major changes; this was done simply to revise the RCS version
 * numbering scheme for this file.
 *
 * Revision 1.2  1993/12/03  01:26:31  drapeau
 * This version of MovieTool contains only the main() function, so that
 * code can be better segmented.  This scheme works better with the
 * XDesigner, allowing interface changes to be made without having to
 * constantly merge the new code with this code (interface code is now in
 * the module "interface.c").
 * */
static char movieToolRcsid[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4a/src/client/motif/MovieTool/RCS/MovieTool.c,v 1.28.1.1 1995/04/05 18:35:25 lovvik Exp $";
#include <Xm/Protocols.h>
#include "MovieTool.h"

XtAppContext	app_context;
Display*	display;					    /*  Display             */
AppResources	mtResources;

XtResource	movieToolResources[] =				    /* MovieTool X Resources */
{
{
  XtNaudioDelay,
  XtCAudioDelay,
  XtRInt,
  sizeof(int),
  XtOffsetOf(AppResources, audioDelay),
  XtRImmediate,
  (XtPointer) 4,						    /* By default, wait 3 frames before recording audio for... */
},								    /* ....it (this was tuned by hand) */
{
  XtNchannels,
  XtCChannels,
  XtRInt,
  sizeof(int),
  XtOffsetOf(AppResources, channels),
  XtRImmediate,
  (XtPointer) 2,
},
{
  XtNsampleRate,
  XtCSampleRate,
  XtRInt,
  sizeof(int),
  XtOffsetOf(AppResources, sampleRate),
  XtRImmediate,
  (XtPointer) 48000,
},
{
  XtNdiagnostics,
  XtCDiagnostics,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, diagnostics),
  XtRImmediate,
  (XtPointer) False,						    /* By default, don't print record & playback stats */
},
{
  XtNautoPlay,
  XtCAutoPlay,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, autoPlay),
  XtRImmediate,
  (XtPointer) False,						    /* By default, don't try to automatically play movies... */
},
{
  XtNautoQuit,							    /* ...and quit after playing a movie. */
  XtCAutoQuit,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, autoQuit),
  XtRImmediate,
  (XtPointer) False,
},
{
  XtNsilent,
  XtCSilent,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, silent),
  XtRImmediate,
  (XtPointer) False,
},
{
  XtNautoRepeat,
  XtCAutoRepeat,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, autoRepeat),
  XtRImmediate,
  (XtPointer) False,						    /* By default, play a movie once only; don't loop back */
},
{
  XtNhelp,
  XtCHelp,
  XtRBoolean,
  sizeof(Boolean),
  XtOffsetOf(AppResources, help),
  XtRImmediate,
  (XtPointer) False,
},
};

XrmOptionDescRec	mtOptions[] =				    /* MovieTool command-line options */
{
{"-audioDelay",	"*audioDelay",	XrmoptionSepArg,	NULL},
{"-channels",	"*channels",	XrmoptionSepArg,	NULL},
{"-sampleRate",	"*sampleRate",	XrmoptionSepArg,	NULL},
{"-diagnostics",".diagnostics",	XrmoptionNoArg,		"True"},
{"-autoPlay",	"*autoPlay",	XrmoptionNoArg,		"True"},
{"-autoQuit",	"*autoQuit",	XrmoptionNoArg,		"True"},
{"-silent",	"*silent",	XrmoptionNoArg,		"True"},
{"-autoRepeat",	"*autoRepeat",	XrmoptionNoArg,		"True"},
{"-help",	"*help",	XrmoptionNoArg,		"True"},
};


void WMDeleteWindowCallback(Widget, XtPointer, XtPointer);


void Usage(void)
{
  printf("MovieTool may be launched like this:\n");
  printf("MovieTool <options> <movieFileName>\n");
  printf("Note: The following options are also X resources; for example,\n");
  printf("the \"-sampleRate\" option is equivalent to the \"MovieTool.sampleRate\" X resource.\n");
  printf("\nMovieTool Options:\n");
  printf("\n");
  printf("movieFileName specifies the name of a JPEG Movie file to open when\n");
  printf("MovieTool launches.\n");
  printf("\n");
  printf("-diagnostics:\tprint copious diagnostic messages during\n");
  printf("\t\t\tapplication launch, record, and playback.\n");
  printf("\n");
  printf("-audioDelay:\tset the number of frames audio will lag behind video.\n");
  printf("\n");
  printf("-channels:\tset the number of audio channels to be recorded\n");
  printf("\t\t\t(1 for mono, 2 for stereo).\n");
  printf("\n");
  printf("-sampleRate:\tset the number of audio samples per second.\n");
  printf("\t\t\tto be recorded (e.g., -sampleRate 8000 to record 8000\n");
  printf("\t\t\tsamples per second).\n");
  printf("\n");
  printf("-silent:\tForces MovieTool to avoid playing and recording audio,\n");
  printf("\t\t\teven if it is available.\n");
  printf("\n");
  printf("-autoPlay:\t when a movieFileName is specified, this option\n");
  printf("\t\t\ttells MovieTool to begin playback of the movie\n");
  printf("\t\t\timmediately after launching.\n");
  printf("\n");
  printf("-autoQuit:\tOnly useful when the -autoPlay option is also set,\n");
  printf("\t\t\tthis option tells MovieTool to quit immediately after\n");
  printf("\t\t\tplaying the movie specified on the command line.\n");
  printf("\t\t\tThis option is especially good for shell scripts that\n");
  printf("\t\t\tneed to play movies and quit MovieTool without human\n");
  printf("\t\t\tintervention (e.g., in kiosk applications).\n");
  printf("-----------\n\n");
  return;
} /* end function Usage */



void CheckOptions(int argc, char** argv)
{
  int	counter;
  
  startupMovieName = (char*)NULL;
  XtVaGetApplicationResources(mainWindow,			    /* Get user-specified X resources or command-line options */
			      &mtResources,
			      movieToolResources,
			      XtNumber(movieToolResources),
			      NULL);
  startupMovie = False;						    /* By default, no movie was specified on the command line */
  audioDelayInFrames = mtResources.audioDelay;
  numAudioChannels = mtResources.channels;
  sampleRate = mtResources.sampleRate;
  printDiagnostics = mtResources.diagnostics;
  autoPlay = mtResources.autoPlay;
  autoQuit = mtResources.autoQuit;
  silentMovie = mtResources.silent;
  autoRepeat = mtResources.autoRepeat;
  if (mtResources.diagnostics == True)
  {
    printf("MovieTool diagnostics are enabled.\n");
  }
  if (mtResources.help == True)					    /* Was help requested? */
  {
    Usage();							    /* Yes, print the help and exit immediately */
    exit(-1);
  }
  if (argc > 1)							    /* Was a movie filename specified on the command line? */
  {
    startupMovie = True;					    /* Yes, indicate so, and... */
    startupMovieName = argv[1];					    /* ...point to the filename specified. */
  }
  else								    /* No, no movie was specified, so the autoPlay and... */
  {								    /* ...autoQuit resources are moot; set them to False. */
    autoPlay = False;
    autoQuit = False;
  }
  return;
} /* end function CheckOptions */


int main (int argc, char **argv)
{
Atom		deleteWindowAtom;

  XtSetLanguageProc ( (XtAppContext) NULL, (XtLanguageProc) NULL, (XtPointer) NULL );
  XtToolkitInitialize ();
  app_context = XtCreateApplicationContext ();
  display = XtOpenDisplay (app_context,
			   NULL,
			   argv[0],
			   "MovieTool",
			   (XrmOptionDescRec*)&mtOptions,
			   XtNumber(mtOptions),
			   &argc, argv);
  if (!display)
  {
    printf("%s: can't open display, exiting...\n", argv[0]);
    exit (-1);
  }

  create_mainWindow ( display, argv[0], argc, argv );
  deleteWindowAtom = XmInternAtom(display,			    /* Create an Atom for communicating with the window mgr... */
				  "WM_DELETE_WINDOW",		    /* ...that the app. needs to be informed about shells... */
				  False);			    /* ...being deleted. */
  XmAddWMProtocolCallback(mainWindow, deleteWindowAtom, 	    /* Intercept window manager Delete Window calls in order... */
			  (XtCallbackProc)WMDeleteWindowCallback,   /* ...to update the interface components correctly. */
			  (XtPointer)NULL);

  create_audioShell ( mainWindow );
  create_input1Shell ( mainWindow );
  create_fileDialog ( mainWindow );
  
/*
  fileSelectionBox = mpCreateFileSelectionDialog(mainWindow, "fileDialog", NULL, 0);
  fileDialog = XtParent(fileSelectionBox);
*/
  create_smartMovieShell ( mainWindow );
  create_movieShell ( mainWindow );
  create_alertShell (mainWindow);
  errorMessage = malloc(1024);					    /* Allocate some memory to store error messages */
  monitorStatus = True;						    /* Monitor is turned on by default */
  userTurnedOffMonitor = False;					    /* Indicate whether the app turned off monitor or user did */
  CheckOptions(argc, argv);
  InitializeXVideo();
  XtRealizeWidget (mainWindow);
  audioFD = InitializeAudio();
  fileFormat = strdup("JPEGMovie");				    /* By default, save movies as JPEG movie files */
  timecodeString = malloc(4);					    /* Hold enough characters for two-digit display */
  PrintAudioInfo(audioFD, &audioInfo);
  InitializeInterface();
  InitMAEstroMessaging();
/*
  printf("Debugging Message: Fix defined constants for encoding in PortableAudio.h!!!!!\n");
*/
  if (startupMovie == True)					    /* Was a movie specified on the command line? */
  {								    /* Yes, open the movie right away */
    OpenMovie((Widget)NULL, startupMovieName, (XtPointer)NULL);	    /* Yes, open the movie for playback */
    if ((movieFile != (FILE*)NULL) && (autoPlay == True))	    /* Did the file open successfully?  And, should the movie... */
    {								    /* ...be played immediately on playback? */
      XtPopdown(mainWindow);					    /* Yes, hide the main panel and play the movie... */
      AutoPlay();
    }
  }
  XtAppMainLoop (app_context);
  exit (0);
}


