/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/MovieTool.h,v 1.32 1994/09/13 08:05:50 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: MovieTool.h,v $
 * Revision 1.32  1994/09/13  08:05:50  drapeau
 * Declared a new global variable to handle the size of the imageQuality
 * widget's source rectangle, which tracks the size of the video input
 * widget being viewed.
 *
 * Revision 1.31  1994/08/27  07:11:53  drapeau
 * Removed an extraneous function prototype since it is now in the
 * include file "XPlxProtos.h".
 *
 * Revision 1.30  1994/08/23  01:29:45  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.29  1994/08/19  05:59:35  drapeau
 * Minor change: added function prototype for Stop() so that it can
 * cleanly be called from other callback functions.
 *
 * Revision 1.28  1994/08/11  03:13:43  drapeau
 * Added a few new variables for better diagnostics as well as
 * app-specific X resources.
 *
 * Revision 1.27  1994/06/10  01:20:48  drapeau
 * Included errno.h for better error reporting.
 *
 * Revision 1.26  1994/05/31  19:54:05  drapeau
 * Added a number of variables to support better diagnostics and
 * automatic playback and quit for movies.  Also, changed types of some
 * variables used in synchronization code (changed from integer data
 * types to floating point data types).
 *
 * Revision 1.25  1994/01/20  22:51:36  drapeau
 * Added two variables that had previously been removed.  They were added
 * to support a slight change in the PortableAudio interface which was
 * made to accommodate the Sun audio device driver implementation, which
 * is flawed, in my opinion.
 *
 * Revision 1.24  1994/01/20  18:05:59  drapeau
 * Modified code to support the PortableAudio interface, increasing
 * portability across platforms.
 *
 * Also added new variables to support new features and removed obsolete
 * variables.
 *
 * Revision 1.23  1994/01/10  20:12:56  drapeau
 * Added support for the MAEstro Multimedia Authoring Environment.  Also
 * added a few new variables for better statistics tracking and
 * synchronization.
 *
 * Revision 1.22  1993/12/04  01:03:29  drapeau
 * Modified the code to make the Pause feature more robust.  Before this
 * version, pausing a movie during playback would throw off the
 * synchronization mechanism; pause now works correctly.
 *
 * Revision 1.21  1993/12/03  02:06:11  drapeau
 * Minor change to clean up RCS id info; revision numbering change made, too.
 *
 * Revision 1.2  1993/12/03  01:30:13  drapeau
 * Added declarations for new functions and variables.  Also added
 * copyright notice to the code.
 * */
static char movieToolHeaderRcsid[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/MovieTool.h,v 1.32 1994/09/13 08:05:50 drapeau Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/LabelG.h>
#include <Xm/PushBG.h>
#include <Xm/SeparatoG.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/time.h>
#include <signal.h>
#include <fcntl.h>
#include <malloc.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>
#include "PortableAudio.h"
#include <movie.h>						    /* For JPEG Movie file definitions */
#include "externs.h"
#include <Sender.h>
#include <Receiver.h>
#include <errno.h>
#include <math.h>

#define	Stopped		0					    /* Defined constants to describe the status of the... */
#define	Playing		1					    /* ...application for recording and playback. */
#define	Recording	2
#define	Paused		3

#define MovieModified	0					    /* Defined constants to indicate whether a movie file has... */
#define MovieNew	1					    /* ...been modified and needs saving, or is empty (not... */
#define MovieOK		2					    /* ...yet been recorded), or has no changes made to it. */

#define MaxMovieDurationInFrames	(30 * 3600)		    /* Enough frames for one hour of NTSC video */
#define HighQualitySamplingRate		44100
#define OneImageSizeInBytes		(1024 * 256)		    /* Define maximum size to allocate for one JPEG image */

#define XtNaudioDelay	"audioDelay"
#define XtNchannels	"channels"
#define XtNsampleRate	"sampleRate"
#define XtNautoPlay	"autoPlay"
#define XtNautoQuit	"autoQuit"
#define XtNsilent	"silent"
#define XtNautoRepeat	"autoRepeat"
#define XtNhelp		"help"


#define XtCAudioDelay	"AudioDelay"
#define XtCChannels	"Channels"
#define XtCSampleRate	"SampleRate"
#define XtCAutoPlay	"AutoPlay"
#define XtCAutoQuit	"AutoQuit"
#define XtCSilent	"Silent"
#define XtCAutoRepeat	"AutoRepeat"
#define XtCHelp		"Help"

typedef struct _AppResources					    /* X resources specific to MovieTool */
{
  int		audioDelay;					    /* Number of frames to make audio lag behind video */
  int		channels;					    /* Number of audio channels to record (1 or 2) */
  int		sampleRate;					    /* Audio sampling rate, in Hertz */
  Boolean	diagnostics;					    /* Whether to print diagnostic info for record & playback */
  Boolean	autoPlay;					    /* Whether to play the movie on the cmd line immediately */
  Boolean	autoQuit;					    /* Whether to quit MovieTool after playing the movie */
  Boolean	silent;						    /* Whether to record audio along with video */
  Boolean	autoRepeat;					    /* Whether to play the movie over and over */
  Boolean	help;						    /* Whether to print a help message */
} AppResources;


extern Widget		imageQualityWidget;			    /* Externs outside of XDesigner variables */
extern Widget		input1Widget;
extern Widget		playbackWidget;
extern jpheader		movieHeader;
extern int*		frameLocation;
extern int*		recordFrameIndex;
extern float*		skippedFrameIndex;
extern float*		recordSkippedFrameIndex;
extern char*		videoFrameIndex;
extern long*		audioReadTime;
extern long*		audioWriteTime;
extern long*		compressTime;
extern long*		diskWriteTime;
extern long*		playInterval;
extern int		movieToolStatus;
extern XPlxCImage*	jpegImage;
extern XPlxCImage*	imageQualityImage;
extern char*		jpegData;
extern FILE*		movieFile;
extern int		currentFrame;
extern XtWorkProcId	playbackWorkProcID;
extern char*		audioData;
extern XtAppContext	app_context;
extern int		signalFrameRate;
extern int		audioControlFD;
extern int		outputFile;
extern int		audioBytesPerFrame;
extern int		recordAudioFlag;
extern int		audioBandwidth;
extern PortableAudio	audioInfo;
extern jpheader		recordHeader;
extern XtWorkProcId	recordWorkProcID;
extern char		outputFileName[MAXPATHLEN];
extern char*		currentMovieFileName;
extern char		titleString[MAXPATHLEN + 64];  
extern int		totalBytesInRecording;
extern XRectangle	recordSourceRect;
extern Boolean		resetPlaybackClock;
extern struct timeval	playbackStartTime;
extern struct timeval	recordStartTime;
extern int		playbackStartFrame;
extern int		playbackEndFrame;
extern int		framesSkippedDuringPlayback;
extern int		framesSkippedDuringRecording;
extern int		recordFrameRate;
extern long		totalTimePaused;
extern Boolean		printDiagnostics;
extern int		audioDelayInFrames;
extern int		firstFrame;
extern int		lastFrame;
extern char		diagString[MAXPATHLEN + 64];
extern Sender*		sender;
extern Receiver*	receiver;
extern XtInputId	maestroFD;
extern int		movieStatus;
extern Audio_hdr	audioPlayHeader;
extern Audio_hdr	audioRecordHeader;
extern Boolean		silentMovie;
extern int		sampleRate;
extern int		numAudioChannels;
extern int		playbackSpeed;
extern Boolean		usingSharedMemory;
extern int		audioFD;
extern int		audioPlayFD;
extern int		audioRecordFD;
extern char*		fileFormat;
extern Boolean		monitorStatus;
extern int		lastMonitorVolume;
extern Boolean		userTurnedOffMonitor;
extern Boolean		autoPlay;
extern Boolean		autoQuit;
extern Boolean		startupMovie;
extern Boolean		autoRepeat;
extern char*		startupMovieName;
extern char*		timecodeString;
extern char*		errorMessage;
extern XRectangle	imageQualitySourceRect;

void		UpdateTimecodeDisplay(int);
void		OpenMovie(Widget, XtPointer, XtPointer);
void		CheckMAEstroMessages(XtPointer, int*, XtInputId*);
void		SetMovieStatus(int);
void		SaveMPEGFile(char*);
void		SaveMovie(Widget, XtPointer, XtPointer);
void		CompactMovie(Widget, XtPointer, XtPointer);
void		PrintDiagnostics(char*);
void		Stop(Widget, XtPointer, XtPointer);
long		ElapsedTime(struct timeval*, struct timeval*);
