/* $Header: /plx/S/source/Xclients/solaris2.4/3.4a/src/client/motif/MovieTool/RCS/callbacks.c,v 1.34.1.1 1995/04/05 20:25:58 lovvik Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: callbacks.c,v $
 * Revision 1.34.1.1  1995/04/05  20:25:58  lovvik
 * 2 changes:
 *   1)  Modified WriteAudio() to flush the buffer in the event that
 *       the write() command failed.  This fixed a locking problem
 *       on the Sparc-5.
 *
 *   2)  Added a ReadAudio() that is basically the same in form as
 *       the WriteAudio().  It is simply a non-blocking write with
 *       checks which verify that all audio data has been written
 *       to the audio device.
 *
 * Revision 1.34  1994/09/13  08:06:59  drapeau
 * Two changes:
 *
 * 1) Assure that the movie playback shell is popped up whenever playback
 *    is requested.
 *
 * 2) Track changes in the size of the live video widget so that the
 *    imageQuality widget grabs the whole window's contents correctly.
 *
 * Revision 1.33  1994/08/27  05:47:37  drapeau
 * Removed support for OS-specific code and placed it in OS-specific
 * modules.
 *
 * Also, slight modifications to code to disable the speed playback
 * slider during playback.
 *
 * Revision 1.32  1994/08/23  01:33:07  drapeau
 * * Removed extraneous messages to the terminal.
 *
 * * Modified ChangeRecordSpeed() to check if audio is really present; if
 *   it is not, there is no use in trying to make the record speed
 *   accommodate the audio device.
 *
 * * Modified the Stop() function for recording: if an error happened
 *   during recording then "currentFrame" would be set to zero, which
 *   could potentially cause a divide-by-zero error later in the
 *   function.  This case is now caught and prevented.
 *
 * Revision 1.31  1994/08/19  06:00:53  drapeau
 * Modified ChangeRecordSpeed() to account for frame rates that would
 * potentially cause problems with the Sun audio device (not yet sure
 * whether these same problems exist with HP audio).
 *
 * Revision 1.30  1994/08/11  03:16:05  drapeau
 * * Removed extraneous variable declarations.
 * * Added code to enhance diagnostic gathering.
 * * Added callback functions to support editable text fields in the
 *   timecode display.
 *
 * Revision 1.29  1994/06/24  01:16:02  drapeau
 * Two changes:
 *
 * 1) The audio device is now closed upon proper exit from the
 *    application.
 *
 * 2) The PlusButtonCallback() and MinusButtonCallback() functions were
 *    fixed; there were errors in the way the frame +/- buttons
 *    calculated which frame should be played next.  Behavior is now
 *    consistent with the author's expectations.
 *
 * Revision 1.28  1994/06/15  03:27:08  drapeau
 * Enhanced the code in the Record() function to maintain the ability to
 * read and write JPEG Movie files with audio that can be used with older
 * programs that also play JPEG Movies.
 *
 * Revision 1.27  1994/06/10  01:24:07  drapeau
 * Fixed errors in resizing video and JPEG shells (i.e., QuarterSize,
 * FullSize, SmallSize).
 *
 * Added code to handle the silentMovie command-line option more
 * completely, now avoiding both recording and playback of audio.
 *
 * Fixed errors in the calculation of timecode when advancing one second.
 *
 * Revision 1.26  1994/05/31  20:09:43  drapeau
 * A number of changes were made, generally falling into one of these
 * categories:
 *
 * 1) Synchronization code was enhanced.  It is now more precise, using
 *    floating point calculations instead of integer math.  Rounding
 *    errors are reduced by several orders of magnitude.
 *
 * 2) Diagnostic reporting is cleaner.
 *
 * 3) Audio monitoring and playback are cleaned up.
 *
 * Revision 1.25  1994/01/20  22:50:48  drapeau
 * Modified code to support minor change in the PortableAudio interface,
 * which was done to accommodate the Sun audio device driver
 * implementation.
 *
 * Revision 1.24  1994/01/20  17:50:16  drapeau
 * Made a number of changes:
 *
 * * Code now handles the case where the JPEG widget is not able to use
 *   shared memory.
 *
 * * Code now supports the new PortableAudio structure, making audio
 *   support more portable across platforms.
 *
 * * Removed extraneous error diagnostic messages.
 *
 * * When quitting, first checks if a valid MAEstro Sender was created;
 *   if not, don't bother trying to disconnect from the PortManager.
 *
 * Revision 1.23  1994/01/10  22:38:13  drapeau
 * Several changes to make the application more robust:
 *
 * 1) Added support for high-quality Sun audio.
 * 2) Added support for playback at speeds other than the
 *    originally-recorded speed.
 * 3) Added support for the MAEstro Multimedia Authoring Environment.
 * 4) Fixed minor errors in the gathering and reporting of diagnostic
 *    information.
 * 5) Fixed errors in synchronization code (previous versions didn't take
 *    into account the starting frame of a movie; they always assumed
 *    playback would always begin at frame 0).
 * 6) Cleaned up diagnostic reporting.
 * 7) Fixed timecode display; bad math problems.
 *
 * Revision 1.22  1993/12/04  01:02:15  drapeau
 * Modified the code to make the Pause feature more robust.  Before this
 * version, pausing a movie during playback would throw off the
 * synchronization mechanism; pause now works correctly.
 *
 * Revision 1.21  1993/12/03  02:10:24  drapeau
 * Minor change made in rcs info to allow me to change the RCS
 * numbering scheme for this file.
 * */
static char callbacksRcsid[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4a/src/client/motif/MovieTool/RCS/callbacks.c,v 1.34.1.1 1995/04/05 20:25:58 lovvik Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/PushBG.h>
#include <stdlib.h>
#include "MovieTool.h"

static int	audioBufferSize;
static void ResetAudioDevice(XtPointer, XtIntervalId *);

void XDmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtManageChild ( *(Widget *)client_data );
}

void XDunmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmanageChild ( *(Widget *)client_data );
}

void XDpopup_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtPopup ( *(Widget *)client_data, XtGrabNone );
}

void XDpopdown_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtPopdown ( *(Widget *)client_data );
}

void XDmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtMapWidget (*(Widget *)client_data);
}

void XDunmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmapWidget (*(Widget *)client_data);
}

void XDenable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtSetSensitive (*(Widget *)client_data, TRUE);
}

void XDdisable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtSetSensitive (*(Widget *)client_data, FALSE);
}

extern void XDhelp_link (Widget w, XtPointer client_data, XtPointer call_data);



void MonitorOn(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  SetAudioMonitorVolume(audioFD, lastMonitorVolume);		    /* Restore monitor volume to last value */
  XmToggleButtonSetState(monitorOnToggle, True, False);		    /* Make the "Monitor On" toggle show and the "Monitor... */
  XmToggleButtonSetState(monitorOffToggle, False, False);	    /* ...Off" toggle not show. */
}



void MonitorOff(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  if (w != (Widget)NULL)					    /* Determine whether this function was called by the... */
    userTurnedOffMonitor = True;				    /* ...app itself or by the person using the app. */
  else
    userTurnedOffMonitor = False;				    /* The application in this case turned off the monitor */
  SetAudioMonitorVolume(audioFD, 0);				    /* Disable monitor output for now */
  XmToggleButtonSetState(monitorOffToggle, True, False);	    /* Make the "Monitor Off" toggle show and the "Monitor... */
  XmToggleButtonSetState(monitorOnToggle, False, False);	    /* ...On" toggle not show. */
}



void ChangePlaybackSpeed(Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  playbackSpeed = callData->value;
  return;
}



void ChangeRecordSpeed(Widget		w,
		       XtPointer	client_data,
		       XtPointer	xt_call_data)
{
  int				audioBytesPerSecond;
  int				audioFrameSize;
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  int				frameSize;
  
  if (audioFD == -1)						    /* No use trying to make the recording speed cooperate... */
    return;							    /* ...with the audio sample rate if audio isn't present. */
  recordFrameRate = callData->value;				    /* Note the change in recording frame rate */
  frameSize = audioRecordHeader.channels *			    /* Determine the "frame size" for sampling in Sun audio */
    audioRecordHeader.bytes_per_unit;				    /* (the Sun audio device only allows data in multiples... */
								    /* ...of the frame size to be written to it; any other... */
								    /* ...number of bytes is illegal and hoses the device). */
  audioBytesPerSecond = audioInfo.play.sampleRate *		    /* Determine how much bandwidth is necessary for one... */
    audioInfo.play.channels *					    /* ...second of audio. */
      audioInfo.play.bitsPerSample / 8;
  audioFrameSize = audioBytesPerSecond / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
  if ((audioFrameSize % frameSize) != 0)
  {
    for (recordFrameRate = callData->value;			    /* Try increasing the frame rate until a desired one... */
	 recordFrameRate < signalFrameRate;			    /* ...is found, if possible */
	 recordFrameRate++)
    {
      audioFrameSize = audioBytesPerSecond / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
      if ((audioFrameSize % frameSize) == 0)
	break;
    }
  }
  if ((audioFrameSize % frameSize) != 0)			    /* Was the new frame rate still unsatisfactory? */
  {
    for (recordFrameRate = signalFrameRate;			    /* Yes, try *decreasing* the frame rate until a desired... */
	 recordFrameRate > 1;					    /* ...one is found, if possible */
	 recordFrameRate--)
    {
      audioFrameSize = audioBytesPerSecond / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
      if ((audioFrameSize % frameSize) == 0)
	break;
    }
  }
  if (recordFrameRate != callData->value)			    /* Was it necessary to override the user's choice of... */
  {								    /* ...recording frame rate? */
    XtVaSetValues(recordSpeedSlider,				    /* Yes, forcibly update the Record Speed slider to... */
		XmNvalue, recordFrameRate, NULL);		    /* ...reflect the acceptable recording frame rate. */
  }
  return;
} /* end function ChangeRecordSpeed */



void ChangeImageQuality(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaGetValues(input1Widget,
		XtNwidth, &imageQualitySourceRect.width,
		XtNheight, &imageQualitySourceRect.height,
		NULL);
  XtVaSetValues(imageQualityWidget,				    /* Display the results of the quality adjustment */
		XtNsourceWin, XtWindow(input1Widget),
		XtNqFactor, callData->value,
		XtNgetNewImage, True,
		XtNputNewImage, True,
		NULL);
  XtVaSetValues(playbackWidget,					    /* Dynamically show adjustments to the image quality */
		XtNsourceWin, 0,
		XtNqFactor, callData->value,
		NULL);
} /* end function ChangeImageQuality */



void FullSize(Widget	w,
	      XtPointer	client_data,
	      XtPointer	xt_call_data)
{
  Dimension			signalWidth;
  Dimension			signalHeight;
  
  XtVaGetValues(input1Widget,					    /* Get the full size of this video widget's input signal */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
  XtVaSetValues(input1Widget,					    /* Make the video widget full-size */
		XtNwidth, signalWidth,
		XtNheight, signalHeight,
		NULL);
  XtVaSetValues(playbackWidget,					    /* Make the JPEG widget full-size */
		XtNwidth, signalWidth,
		XtNheight, signalHeight,
		NULL);
  return;
} /* end function FullSize */



void QuarterSize(Widget w,
		 XtPointer client_data,
		 XtPointer xt_call_data)
{
  Dimension			signalWidth;
  Dimension			signalHeight;
  
  XtVaGetValues(input1Widget,					    /* Get the full size of this video widget's input signal */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
  XtVaSetValues(input1Widget,					    /* Make the video widget quarter-size */
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
  XtVaSetValues(playbackWidget,					    /* Make the JPEG widget quarter-size */
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
  XtVaSetValues(movieShell,					    /* Make the JPEG widget's shell quarter-size */
		XtNwidth, (signalWidth / 2),
		XtNheight, (signalHeight / 2),
		NULL);
  return;
} /* end function QuarterSize */



void SmallSize(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  Dimension			signalWidth;
  Dimension			signalHeight;
  
  XtVaGetValues(input1Widget,					    /* Get the full size of this video widget's input signal */
		XtNsignalWidth, &signalWidth,
		XtNsignalHeight, &signalHeight,
		NULL);
  XtVaSetValues(input1Widget,					    /* Make the video widget sixteenth-size */
		XtNwidth, (signalWidth / 4),
		XtNheight, (signalHeight / 4),
		NULL);
  XtVaSetValues(playbackWidget,					    /* Make the JPEG widget sixteenth-size */
		XtNwidth, (signalWidth / 4),
		XtNheight, (signalHeight / 4),
		NULL);
  return;
} /* end function SmallSize */



void 
PerformRecordDiagnostics (Widget w, XtPointer client_data, XtPointer xt_call_data )
{
	XmPushButtonCallbackStruct *call_data = (XmPushButtonCallbackStruct *) xt_call_data;
}



Boolean QuitWorkProc(XtPointer unusedClientData)
{
  if (sender != (Sender*)NULL)
    SenderDisconnectFromPortMgr(sender, &(receiver->receivePort));  /* Unregister MovieTool's services with the PortManager */
  XtDestroyWidget(mainWindow);
  SetAudioMonitorVolume(audioFD, 0);				    /* Disable monitor output for now */
  AudioClose(audioFD);						    /* Close the audio device */
/*
  printf("QuitWorkProc(): Should check if the movie needs saving before quitting.\n");
*/
  exit(0);
} /* end function QuitWorkProc */



void Quit(Widget	w,
	  XtPointer	client_data,
	  XtPointer	xt_call_data)
{
  XtWorkProcId	quitProcID;
  
  quitProcID = XtAppAddWorkProc(app_context,			    /* Start a work-proc to quit the application; Xt is... */
				QuitWorkProc,			    /* ...poorly designed and this is the only way to assure... */
				(XtPointer)NULL);		    /* ...that shared memory will be freed from the JPEG... */
  return;							    /* ...widgets. */
} /* end function Quit */



int WriteAudio(int audioFD, char* audioData, int audioSlice)
{
  int	bytesLeftToPlay;
  int	bytesToWrite;
  int	bytesWritten;
  char*	audioPtr;

  audioPtr = audioData;

  bytesLeftToPlay = audioSlice;					    /* Indicate how many bytes to be played */
  while (bytesLeftToPlay > 0)
  {
    bytesToWrite = bytesLeftToPlay;                          	    /* How many bytes to try to send to the audio device? */

    bytesWritten = write(audioFD, audioPtr,			    /* Playback the audio for this frame */
			 bytesToWrite);
    while(bytesWritten < 0)                                         /* Error detected if bytesWritten = -1 */
    {								    /*   **This only works if in nonblocking mode */
      FlushAudioBuffer(audioFD);                                    /* . . . recover by removing all data in the buffer */
      bytesWritten = write(audioFD, audioPtr,			    /* . . . and write to the audio device again */
			   bytesToWrite);
    }
    audioPtr += bytesWritten;					    /* Advance through the audio buffer */
    bytesLeftToPlay -= bytesWritten;
    if(bytesLeftToPlay > 0)
    {

    }
  } /* end while there's still bytesLeftToPlay */

  return(audioSlice);						    /* Success; wrote all the bytes requested */
} /* end function WriteAudio */



int ReadAudio(int audioFD, char* audioData, int audioSlice)
{
  int	bytesLeftToRecord;
  int	bytesToRead;
  int	bytesRead;
  char*	audioPtr;

  audioPtr = audioData;

  bytesLeftToRecord = audioSlice;				    /* Indicate how many bytes to be Recorded */
  while (bytesLeftToRecord > 0)
  {
    bytesToRead = bytesLeftToRecord;                          	    /* How many bytes to try to receive from the audio device? */

    bytesRead = read(audioFD, audioPtr, 			    /* Record the audio for this frame */
			 bytesToRead);
    while(bytesRead < 0)                                            /* Error detected if bytesRead = -1 */
    {
      sprintf(diagString, "Error recording audio for frame %d.  Errno was %d.\n",
	      currentFrame,
	      errno);
      PrintDiagnostics(diagString);
      AudioRecordResume(audioFD);				    /* Try to reset the audio device so record can resume */
      bytesRead = read(audioFD, audioPtr,			    /* . . . and write to the audio device again */
			   bytesToRead);
    }
    audioPtr += bytesRead;					    /* Advance through the audio buffer */
    bytesLeftToRecord -= bytesRead;
  } /* end while there's still bytesLeftToRecord */

  return(audioSlice);						    /* Success; Read all the bytes requested */
} /* end function ReadAudio */




Boolean PlayOneFrame(int	whichFrame,
		     Boolean	playAudio,
		     Boolean	playVideo)
{
  struct timeval	after;
  int			audioBytesWritten;
  struct timeval	before;
  Boolean		done = False;
  unsigned char		instruction;
  struct timeval	now;
  Boolean		videoInThisFrame = False;
  
  if (whichFrame <= playbackEndFrame)				    /* Was a valid frame specified? */
  {								    /* Yes, play one frame of the movie */
    gettimeofday(&now, (struct timezone*)NULL);			    /* Clock when this function was called */
    playInterval[whichFrame] = ElapsedTime(&playbackStartTime,	    /* Determine how long it has been since the start of the... */
					   &now);		    /* ...movie. */
    fseek(movieFile, frameLocation[whichFrame], 0);		    /* Go to the start of the desired frame */
    while (done == False)					    /* Keep reading until all the instructions in this frame... */
    {								    /* ...have been executed. */
      fread(&instruction, 1, 1, movieFile);			    /* Read the 1st opcode for this frame */
      instruction &= 0x00FF;
      switch ((int)instruction)					    /* Do the appropriate action for this opcode */
      {
       case END_FRAME:						    /* Was the end of this frame reached? */
	done = True;						    /* Yes, set a variable saying so */
	UpdateTimecodeDisplay(movieHeader.fps);
	break;
       case LOAD_JPEG:
	videoFrameIndex[whichFrame] = 0x01;			    /* Note that this frame had a video image in it */
	fread((char*)&(jpegImage->size), sizeof(int), 1, movieFile); /* Read the size of this JPEG image in bytes */
	if (playVideo == True)					    /* Is video supposed to be shown for this frame? */
	{							    /* Yes, do the right thing */
	  fread(jpegImage->data, jpegImage->size, 1, movieFile);    /* Read the JPEG image data themselves */
	  XtVaSetValues(playbackWidget,				    /* Tell the widget to render the new image to the screen now */
			XtNputNewImage, True,
			NULL);
	}
	else
	  fseek(movieFile, jpegImage->size, 1);			    /* Skip past the JPEG image information for this frame */
	break;
       case LOAD_AUDIO_0:
       case LOAD_AUDIO_1:
       case LOAD_AUDIO_2:
       case LOAD_AUDIO_3:
	if ((playAudio == True) && (silentMovie == False))	    /* Is audio supposed to be played in this frame? */
	{							    /* Yes, do the right thing */
	  fread(audioData, 1, movieHeader.audioslice, movieFile);   /* Read the audio data for this frame */
	  gettimeofday(&before, (struct timezone*)NULL);
	  audioBytesWritten = WriteAudio(audioFD,		    /* Playback the audio for this frame */
					 audioData,
					 movieHeader.audioslice);
	  
	  if (audioBytesWritten < 0)				    /* Was there an error playing one frame of audio? */
	  {							    /* Yes, report the error and stop movie playback now */
	    sprintf(errorMessage,
		    "There was an error during playback trying to write %d bytes to the audio device.  Errno is %d.\n",
		    movieHeader.audioslice, errno);
	    PrintErrorMessage(errorMessage);
	    return(True);					    /* Tell the toolkit to halt playback right now */
	  }
	  gettimeofday(&after, (struct timezone*)NULL);
	  audioWriteTime[whichFrame] =				    /* Update stats on how long this frame's audio playback took */
	    ElapsedTime(&before, &after);
	}
	else
	{
	  fseek(movieFile, movieHeader.audioslice, 1);		    /* Skip past the audio data for this frame */
	}
	break;
       default:
/*
	printf("An unknown instruction (%d) was found in frame %d of the movie, skipping.\n",
	       instruction, whichFrame);
	printf("Should stop reporting this error after, say, 10 occurrances.\n");
*/
	break;
      }	/* end switch */
    } /* end while */
    return(False);						    /* Tell the toolkit to call this function again */
  }
  else								    /* No, there is no more movie left to play */
    return(True);						    /* Tell the toolkit not to bother calling again */
} /* end function PlayOneFrame */



void Search(Widget	w,
	    XtPointer	client_data,
	    XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* Search should not work unless a movie is either paused... */
      (movieToolStatus != Stopped))				    /* ...or stopped completely. */
    return;
  currentFrame = callData->value + firstFrame;			    /* Go to requested frame, taking into acct that movie...  */
								    /* ...may not begin at the 1st frame. */
  PlayOneFrame(currentFrame, False, True);			    /* Play a single frame from the movie, without audio */
} /* end function Search */




long	ElapsedTime(struct timeval*	before,
		    struct timeval*	after)
{
  return(((after->tv_sec - before->tv_sec) * 1000) +		    /* Convert the seconds part to milliseconds */
	 ((after->tv_usec - before->tv_usec) / 1000));		    /* Convert the microseconds part to milliseconds */
} /* end function ElapsedTime */



void PrintPlaybackStatistics(struct timeval*	stopTime,
			     jpheader*		movieHeaderInfo)
{
  int	counter;
  float	delta;
  float	timeBetweenFrames;
  float	timeForOneFrame;
  long	totalTimeTaken;
  long	totalNumberFramesPlayed;
  float	variance;
  
  if (printDiagnostics == False)				    /* Don't bother printing performance stats if not requested */
    return;
  totalTimeTaken = ElapsedTime(&playbackStartTime, stopTime);	    /* Calculate total milliseconds to play this clip */
  totalNumberFramesPlayed =					    /* Calculate total number of frames played in this clip */
    (currentFrame + 1) - playbackStartFrame;
  printf("\nPlayback Performance Information:\n");
  printf("Average frame rate: %f frames per second.\n",
	 (float)totalNumberFramesPlayed * 1000.0 / (float)totalTimeTaken);
  printf("Number of video frames skipped due to lagging: %d (%d percent of total).\n",
	 framesSkippedDuringPlayback,
	 framesSkippedDuringPlayback * 100 / totalNumberFramesPlayed);
  for (counter = 0;						    /* Report on each frame skipped during playback */
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    if (skippedFrameIndex[counter] != 0.0)
      printf("Frame %d was %f milliseconds (approx. %f frames) late.\n",
	     counter, skippedFrameIndex[counter],
	     skippedFrameIndex[counter] /
	     (float)(1000 / playbackSpeed));
  }
  printf("\n\nTime to play audio for each frame (only times greater than 1 msec are reported):\n");
  for (counter = 0;
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    if (audioWriteTime[counter] > 1)
      printf("Frame %d: %d milliseconds.\n", counter,
	     audioWriteTime[counter]);
  }
  timeForOneFrame = 1000.0 / playbackSpeed;			    /* Calculate # of millisecs to play one frame of this movie */
  delta = timeForOneFrame * .1;
  printf("\n\nTime passed between frames (only variances greater than 10%% (%f ms) are reported:\n",
	 delta);
  for (counter = 0;
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    timeBetweenFrames = (counter == 0) ? playInterval[counter] :    /* Determine how many milliseconds passed b/w 2... */
      playInterval[counter] - playInterval[counter - 1];	    /* ...successive frames. */
    variance = fabs((double)(timeBetweenFrames) -		    /* Calculate how far off ideal frame rate this frame was... */
		    (double)timeForOneFrame);			    /* ...started. */
    if ((variance > delta) &&					    /* Was variance greater than allowed, and was the frame... */
	(playInterval[counter] !=0))				    /* ...actually played? (i.e., non-zero) If so, report it. */
    {
      if (counter == 0)						    /* Is this the 1st frame? */
	printf("1st Frame: %d ms.\n", playInterval[counter]);	    /* Yes, report only the 1st frame's stats */
      else							    /* No, report each subsequent... */
	printf("Frame %d: %d ms.\n",				    /* ...frame's start time and report that difference. */
	       counter,
	       playInterval[counter] - playInterval[counter-1]);
      continue;
    }
  }
/*
  printf("\nFrames in this movie with no video:\n");
  for (counter = firstFrame; counter < currentFrame; counter++)
  {
    if (videoFrameIndex[counter] == 0x00)
      printf("Frame %d.\n", counter);
  }
*/
  printf("-------------\n\n\n");
  return;
} /* end function PrintPlaybackStatistics */



void PrintRecordStatistics(struct timeval*	stopTime,
			   jpheader*		movieHeaderInfo)
{
  long	totalTimeTaken;
  long	totalNumberFramesRecorded;
  int	counter;
  
  if (printDiagnostics == False)				    /* Don't bother printing performance stats if not requested */
    return;
  totalTimeTaken = ElapsedTime(&recordStartTime, stopTime);	    /* Calculate total milliseconds to record this movie */
  totalNumberFramesRecorded = currentFrame;			    /* Calculate total number of frames recorded in this movie */
  printf("\nRecord Performance Information:\n");
  printf("Average frame rate: %f frames per second.\n",
	 (float)totalNumberFramesRecorded * 1000.0 / (float)totalTimeTaken);
  printf("Number of video frames skipped due to lagging: %d (%d percent of total).\n",
	 framesSkippedDuringRecording,
	 framesSkippedDuringRecording * 100 / totalNumberFramesRecorded);
  for (counter = 0;						    /* Report on each frame skipped during recording */
       counter < MaxMovieDurationInFrames;
       counter++)
  {
    if (recordSkippedFrameIndex[counter] != 0.0)
      printf("Frame %d was %f milliseconds (approx. %f frames) late.\n",
	     counter, recordSkippedFrameIndex[counter],
	     recordSkippedFrameIndex[counter] /
	     (float)(1000.0 / recordFrameRate));
  }
  printf("\n\nTime to record audio for each frame:\n");
  for (counter = 0; counter < currentFrame; counter++)		    /* Print stats for audio recording time */
  {
    if (audioReadTime[counter] != 0)
      printf("Frame %d: %d milliseconds.\n", counter,
	     audioReadTime[counter]);
  }
  printf("\n\nTime to compress each frame:\n");
  for (counter = 0; counter < currentFrame; counter++)		    /* Print stats for video compression time */
  {
    if (compressTime[counter] != 0)
      printf("Frame %d: %d milliseconds.\n", counter,
	     compressTime[counter]);
  }
  printf("\n\nTime to write each frame to disk:\n");
  for (counter = 0; counter < currentFrame; counter++)		    /* Print stats for disk write time */
  {
    if (diskWriteTime[counter] != 0)
      printf("Frame %d: %d milliseconds.\n", counter,
	     diskWriteTime[counter]);
  }
  return;
} /* end function PrintRecordStatistics */



Boolean PlayWorkProc(XtPointer clientData)
{
  struct timeval	now;
  struct timeval	playbackStopTime;
  long			playbackClock;
  float			scheduledTime;
  float			timeForOneFrame;
  Boolean		result;
   
  if (movieToolStatus == Paused)				    /* Is the app in pause mode? */
    return(False);						    /* Yes, don't do anything */
  timeForOneFrame = 1000.0 / playbackSpeed;			    /* Calculate # of millisecs to play one frame of this movie */
  gettimeofday(&now, (struct timezone*)NULL);
  playbackClock = ElapsedTime(&playbackStartTime, &now);	    /* Determine the time relative to the start of the movie */
  scheduledTime = (currentFrame - playbackStartFrame) *		    /* Determine proper relative starting time for this frame */
    timeForOneFrame;
  if (((float)(playbackClock - totalTimePaused) >		    /* Is it past time to play this frame? (i.e., is the... */
      (scheduledTime + timeForOneFrame)) &&			    /* ...movie running late?  It can be up to 1 frame late) */
      (currentFrame < playbackEndFrame))			    /* And, is there still some movie left to play? */
  {
    framesSkippedDuringPlayback++;				    /* Yes, note that another video frame needed to be skipped */
    if (playbackSpeed == movieHeader.fps)			    /* Is the movie playing at its natural speed? */
      result = PlayOneFrame(currentFrame, True, False);		    /* Yes, play the audio for this frame but skip the video */
    else							    /* No, turn off audio during playback */
      result = PlayOneFrame(currentFrame, False, False);	    /* Skip both audio and video for this frame */
    skippedFrameIndex[currentFrame] =				    /* Note how late and for which frame this skip occurred */
      ((float)(playbackClock - totalTimePaused)) - scheduledTime;
    currentFrame++;						    /* Advance to the next frame in the movie */
    return(False);
  }
  if ((scheduledTime - (float)(playbackClock - totalTimePaused)) >  /* Is it too early to play this frame?  More than one... */
      timeForOneFrame)						    /* ...frame is too early. */
  {
    return(False);						    /* Yes, return without doing anything */
  }
  if (currentFrame < playbackEndFrame)				    /* Is there still some movie left to play? */
  {								    /* Yes, play one frame of the movie if there's time */
    if (playbackSpeed == movieHeader.fps)			    /* Is the movie playing at its natural speed? */
      result = PlayOneFrame(currentFrame, True, True);		    /* Yes, play both audio and video for this frame */
    else							    /* No, turn off audio during playback but go ahead and... */
      result = PlayOneFrame(currentFrame, False, True);		    /* ...play the video for this frame. */
    currentFrame++;						    /* Advance to the next frame in the movie */
  }
  else								    /* No, there is no more movie left to play */
  {
    result = True;						    /* Done playing this movie */
    if (autoQuit == True)					    /* Should this application quit right after playback is... */
    {								    /* ...done? */
      Quit((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Yes, quit the app. immediately */
    }
    gettimeofday(&playbackStopTime, (struct timezone*)NULL);	    /* Record the time this movie playback was stopped */
    PrintPlaybackStatistics(&playbackStopTime, &movieHeader);	    /* Print info about playback performance */
    if (autoRepeat == True)					    /* Should the movie go back to the start and play again? */
    {								    /* Yes, reset the movie and get ready to play again */
      playbackStartFrame = firstFrame;
      currentFrame = firstFrame;
      result = False;
      gettimeofday(&playbackStartTime, (struct timezone*)NULL);	    /* Record the time this movie playback was started */
      framesSkippedDuringPlayback = 0;				    /* Reset counter of number of video frames that were omitted */
      memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Re-initialize the skipped frames index before playback */
	     sizeof(float));
      return(result);						    /* Return now, without shutting off movie playback */
    }
    movieToolStatus = Stopped;					    /* Update the playback/record status of the app */
    if (userTurnedOffMonitor == False)				    /* Did the application itself disable the monitor during... */
      MonitorOn((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);    /* ...playback?  If so, turn it on again now that... */
  }								    /* ...playback is done. */
  return(result);
} /* end function PlayWorkProc */



void Play(Widget w,
	  XtPointer client_data,
	  XtPointer xt_call_data )
{
  char*	tempEnvVar = (char*)NULL;
  
  tempEnvVar = getenv("AudioBufferSize");			    /* Try to read user-specified size to use for audio... */
  if ((char*)NULL != tempEnvVar)				    /* ...playback buffer. */
  {
    audioBufferSize = atoi(tempEnvVar);
  }
  else								    /* No user-specified buffer existed; use an arbitrarily-... */
  {								    /* ...defined default value. */
    audioBufferSize = 8192;
  }
  if (movieToolStatus == Playing)				    /* Is a movie already playing?  If so, don't do anything */
    return;
  if (movieToolStatus == Stopped)				    /* Is a movie stopped?  If so, play from the start */
  {
    if (movieStatus == MovieNew)				    /* Does this movie have anything to play yet? */
      return;							    /* No, don't bother doing any work */
    movieToolStatus = Playing;					    /* Tell the application that it's playing a movie */
    XtPopup(movieShell, XtGrabNone);				    /* Make sure the movie playback shell shows during playback */
    framesSkippedDuringPlayback = 0;				    /* Reset counter of number of video frames that were omitted */
    if (currentFrame >= playbackEndFrame)			    /* Is the app sitting at the end of a movie? */
    {								    /* Yes, assume that the viewer wishes to view the whole... */
      playbackStartFrame = firstFrame;				    /* ...movie; set the appropriate variables. */
      currentFrame = firstFrame;
    }
    else
      playbackStartFrame = currentFrame;			    /* Remember which frame began playback */
    memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Initialize the skipped frames index before playback */
	   sizeof(float));
    memset(audioWriteTime, 0, MaxMovieDurationInFrames *	    /* Initialize the audio write time array before playback */
	   sizeof(long));
    memset(playInterval, 0, MaxMovieDurationInFrames *		    /* Initialize time passed b/w frames array before playback */
	   sizeof(long));
    XtVaSetValues(speedSlider, XmNsensitive, False, NULL);	    /* Don't allow playback speed to change during playback */
    playbackWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to play a movie; it's done this way... */
					  PlayWorkProc,		    /* ..so the user interface still works and is not blocked. */
					  (XtPointer)NULL);
    DeSensitizePlaybackControls();				    /* Disable controls for shuttling through the movie */
    gettimeofday(&playbackStartTime, (struct timezone*)NULL);	    /* Record the time this movie playback was started */
  }
  else if (movieToolStatus == Paused)				    /* The app is in pause mode; resume playback */
  {
    movieToolStatus = Playing;
    XtVaSetValues(speedSlider, XmNsensitive, False, NULL);	    /* Don't allow playback speed to change during playback */
    UpdateTotalTimePaused(False);				    /* Tell the pause counter to update total time paused */
  }
  audioFD = AudioPlayPause(audioFD);				    /* Pause the audio buffer to clear out unwanted audio data */
  audioFD = AudioPlayResume(audioFD);				    /* Get audio device ready to play right now */
  SyncAudioPlaybackDevice(audioFD);				    /* Try to synchronize audio device */
  MonitorOff((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);	    /* Make sure the monitor is turned off during playback. */
  return;
} /* end function Play */



void Pause(Widget	w,
	   XtPointer	client_data,
	   XtPointer	xt_call_data )
{
  static Boolean		paused = False;
  
  if (movieToolStatus == Paused)				    /* Is the app currently in pause mode? */
  {								    /* Yes, resume playback and get outtahere right now */
    movieToolStatus = Playing;
    UpdateTotalTimePaused(False);				    /* Tell the pause counter to update total time paused */
    return;
  }
  if (movieToolStatus == Stopped)				    /* Is the app stopped?  I.e., no movie is playing at all */
    return;							    /* Yes, Pause is not applicable; return now */
  if (movieToolStatus == Recording)				    /* is the app currently making a new movie?  If so, it... */
    return;							    /* ...cannot be interrupted so return now. */
  else								    /* The app is in a pausable state, like Playing */
  {
    movieToolStatus = Paused;					    /* Indicate that the app should pause */
    UpdateTotalTimePaused(True);				    /* Tell the paused counter to begin a new pause timer */
    return;
  }
} /* end function Pause */



Boolean RecordOneFrame(int	whichFrame,
		       Boolean	recordAudio,
		       Boolean	recordVideo)
{
  unsigned char		instruction;
  struct timeval	before;
  struct timeval	after;
  int			bytesWritten;
  
  recordFrameIndex[currentFrame] = totalBytesInRecording;	    /* Store the starting byte of this frame */
  if ((recordAudio == True) && (silentMovie == False))		    /* Should audio be recorded for this frame? */
  {
    instruction = LOAD_AUDIO_0;					    /* Prepare to write audio information for this frame to disk */
    bytesWritten = write(outputFile, &instruction, 1);		    /* Try to write the opcode to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return the app into a... */
      sprintf(errorMessage,
	      "Could not write the LOAD_AUDIO_0 instruction to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      movieToolStatus = Stopped;				    /* ...stopped state. */
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    bytesWritten = write(outputFile, audioData, audioBytesPerFrame); /* Attempt to write 1 frame's worth of audio to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return the app into a... */
      sprintf(errorMessage,
	      "There was an error recording audio to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      movieToolStatus = Stopped;				    /* ...stopped state. */
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
  } /* end if recording audio should be done */
  if (recordVideo == True)					    /* Should video be recorded for this frame? */
  {
    instruction = LOAD_JPEG;					    /* Prepare to write JPEG info for this frame to disk */
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(outputFile, &instruction, 1);		    /* Try to write the opcode to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return the app into a... */
      sprintf(errorMessage,
	      "There was an error recording frame %d's JPEG image to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      movieToolStatus = Stopped;				    /* ...stopped state. */
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    bytesWritten = write(outputFile, &(jpegImage->size),	    /* Write the size in bytes of this frame's JPEG image data */
			 sizeof(int));
    if (bytesWritten < sizeof(int))				    /* Did the attempt fail? */
    {								    /* Yes, report the error and return the app into a... */
      sprintf(errorMessage,
	      "There was an error recording this image's size information to disk.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      movieToolStatus = Stopped;				    /* ...stopped state. */
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    bytesWritten = write(outputFile, jpegImage->data, jpegImage->size); /* Write the JPEG image data for this frame */
    if (bytesWritten < jpegImage->size)				    /* Did the attempt fail? */
    {								    /* Yes, report the error and return the app into a... */
      sprintf(errorMessage,
	      "There was an error recording the image for frame number %d to disk.  Recording stopped.\n",
	      currentFrame);
      PrintErrorMessage(errorMessage);
      movieToolStatus = Stopped;				    /* ...stopped state. */
      return(True);						    /* Done recording */
    }
    totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
    gettimeofday(&after, (struct timezone*)NULL);
    diskWriteTime[whichFrame] = ElapsedTime(&before, &after);	    /* Record time taken to write this video frame to disk */
  } /* end if recording video should be done */
  instruction = END_FRAME;					    /* Indicate that this is the end of this frame */
  bytesWritten = write(outputFile, &instruction, 1);		    /* Try to write the opcode to disk */
  if (bytesWritten < 0)						    /* Did the attempt fail? */
  {								    /* Yes, report the error and return the app into a... */
    sprintf(errorMessage,
	    "There was an error writing the END_FRAME opcode to disk.  Recording stopped.\n");
    PrintErrorMessage(errorMessage);
    movieToolStatus = Stopped;					    /* ...stopped state. */
    return(True);						    /* Done recording */
  }
  totalBytesInRecording += bytesWritten;			    /* Update size of this movie in bytes */
  UpdateTimecodeDisplay(recordFrameRate);
  return(False);						    /* Recording was fine, it's okay to continue recording */
} /* end function RecordOneFrame */



Boolean RecordWorkProc(XtPointer clientData)
{
  int			audioBytesRead;
  struct timeval	now;
  struct timeval	playbackStopTime;
  struct timeval	before;
  struct timeval	after;
  long			recordClock;
  float			scheduledTime;
  float			timeForOneFrame;
  Boolean		result = False;
  
  timeForOneFrame = 1000.0 / recordFrameRate;			    /* Calculate # of millisecs to record a frame of a movie */
  gettimeofday(&now, (struct timezone*)NULL);
  recordClock = ElapsedTime(&recordStartTime, &now);		    /* Calculate time relative to the start of the recording */
  if ((currentFrame == 0) && (printDiagnostics == True))	    /* If requested, print the time elapsed between the time... */
  {								    /* ...the "Record" button was pressed and the 1st frame... */
    printf("Time b/w button press & 1st frame grab: %d ms.\n",	    /* was grabbed. */
	   recordClock);
  }
  scheduledTime = currentFrame * timeForOneFrame;		    /* Calculate proper (relative) starting time for this frame */
  if ((scheduledTime - (float)recordClock) >			    /* Is recording ahead of schedule at all?  The assumption... */
      (-audioDelayInFrames * timeForOneFrame))			    /* ...is that recording shouldn't begin until after the... */
  {								    /* ...time for that frame has passed; otherwise, there's... */
    return(False);						    /* ...no audio ready to read. */
  }
  gettimeofday(&before, (struct timezone*)NULL);
  if (silentMovie == False)					    /* Is it okay to record audio? */
  {								    /* Yes, read audio from the audio device */
    audioBytesRead = ReadAudio(audioFD, audioData, audioBytesPerFrame);  /* Read one frame's worth of audio from the audio device */
    if (audioBytesRead < 0)
    {
      sprintf(diagString, "Error recording audio for frame %d.  Errno was %d.\n",
	      currentFrame,
	      errno);
      PrintDiagnostics(diagString);
      AudioRecordResume(audioFD);				    /* Try to reset the audio device so record can resume */
      audioBytesRead = ReadAudio(audioFD, audioData, audioBytesPerFrame); /* Read one frame's worth of audio from the audio device */
    } /* end if audioBytesRead... */
  } /* end if silentMovie... */
  gettimeofday(&after, (struct timezone*)NULL);
  audioReadTime[currentFrame] = ElapsedTime(&before, &after);	    /* Update stats on how long this frame's audio read took */
  if (((float)recordClock) > (scheduledTime +			    /* Is recording late by more than two frames? (that is,... */
		     ((audioDelayInFrames + 1) * timeForOneFrame))) /* ...two frames plus the requested audio delay) */
  {								    /* Yes, record audio only */
    framesSkippedDuringRecording++;				    /* Note that a video frame needed to be skipped */
    recordSkippedFrameIndex[currentFrame] =			    /* Note how late and for which frame this skip occurred */
      ((float)recordClock) - scheduledTime -
	(timeForOneFrame * (audioDelayInFrames - 1));
    compressTime[currentFrame] = 0L;				    /* This frame was skipped so took no time to compress */
    result = RecordOneFrame(currentFrame++, recordAudioFlag,	    /* Record a single frame of audio (no video) to disk */
			    False);
  }
  else								    /* No, there's still time to record both audio and video */
  {
    gettimeofday(&before, (struct timezone*)NULL);
    XtVaSetValues(playbackWidget, XtNgetNewImage, True, NULL);	    /* Tell the JPEG widget to compress another image right now */
    if (usingSharedMemory == False)				    /* Is the JPEG widget using shared memory?  If not, then... */
    {								    /* ...the app must point to a new JPEG image each time... */
      XtVaGetValues(playbackWidget,				    /* ...the app grabs another frame. */
		    XtNjpegData, &jpegImage,
		    NULL);
    }
    gettimeofday(&after, (struct timezone*)NULL);
    compressTime[currentFrame] = ElapsedTime(&before, &after);	    /* Update stats on how long this frame's video compress took */
    result = RecordOneFrame(currentFrame++, recordAudioFlag, True); /* Record a single frame of audio and video to disk */
  }
  return(result);
} /* end function RecordWorkProc */



void Record(Widget	w,
	    XtPointer	client_data,
	    XtPointer	xt_call_data)
{
  int				bytesWritten;
  Dimension			movieWidth;
  Dimension			movieHeight;
  struct timeval		tempBefore;
  struct timeval		tempAfter;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus == Playing) ||				    /* Is a movie already playing or recording?  If so,... */
      (movieToolStatus == Recording))				    /* ...don't do anything. */
    return;
  if (movieToolStatus == Stopped)				    /* Is it okay to begin recording a new movie? */
  {								    /* Yes it is, do the right thing */
    outputFile = open(outputFileName,				    /* Try to open a file for writing a movie to disk; create... */
		      (O_CREAT | O_WRONLY | O_TRUNC),		    /* ...it if it doesn't already exist; reset its length if... */
		      0644);					    /* ...it does exist. */
    if (outputFile == -1)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and don't bother recording */
      sprintf(errorMessage,"Could not open the file %s for writing.\n",
	      outputFileName);
      PrintErrorMessage(errorMessage);
      return;
    }
    PrintAudioInfo(audioFD, &audioInfo);			    /* Get statistics about the audio device */
    switch (audioInfo.record.encoding)				    /* Make sure that the information written to the movie... */
    {								    /* ...file contains the old, device-specific audio... */
     case AudioULawEncoding:					    /* ...encoding information, in order to maintain... */
      recordHeader.audio.encoding = AUDIO_ENCODING_ULAW;	    /* ...compatibility with older JPEG Movie recorders and... */
      break;							    /* ...players. */
     case AudioALawEncoding:
      recordHeader.audio.encoding = AUDIO_ENCODING_ALAW;
      break;
     case AudioLinearEncoding:
      recordHeader.audio.encoding = AUDIO_ENCODING_LINEAR;
      break;
    }
    recordHeader.audio.sample_rate = audioInfo.record.sampleRate;
    recordHeader.audio.samples_per_unit = 1;
    recordHeader.audio.bytes_per_unit =
      audioInfo.record.bitsPerSample / 8;
    recordHeader.audio.channels = audioInfo.record.channels;
    audioBandwidth = audioInfo.play.sampleRate *		    /* Determine how much bandwidth is necessary for audio */
      audioInfo.play.channels *
	audioInfo.play.bitsPerSample / 8;
    recordHeader.audio.data_size = audioBandwidth;
    audioBytesPerFrame = audioBandwidth / recordFrameRate;	    /* Determine how much bandwidth for a single frame of audio */
    if (audioData != (char*)NULL)
      free(audioData);
    audioData = malloc(audioBytesPerFrame);			    /* Allocate enough memory to hold one frame of audio */
    if (audioData == (char*)NULL)				    /* Did the attempt fail? */
    {								    /* Yes, report the error and do nothing about it for now */
      sprintf(errorMessage,
	      "Could not allocate the memory necessary to record audio. Please try again, or quit MovieTool and restart it.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    movieToolStatus = Recording;				    /* Tell the application it's recording a movie */
    currentFrame = 0;						    /* Start at the 1st frame of the new movie */
    strcpy(recordHeader.magic, "j_movie");
    recordHeader.version = JPEG_VERSION_2;
    recordHeader.fps = recordFrameRate;				    /* Record at the best chosen frame rate */
    recordHeader.frames = 0;					    /* When starting to record, there are no frames in the movie */
    recordHeader.bandwidth = 0;					    /* Before recording, the data rate of the movie is unknown */
    XtVaSetValues(input1Widget,					    /* Force the video to turn itself off so I can force it... */
		  XtNlive, False,				    /* ...to turn itself on in the next statement.  This is... */
		  NULL);					    /* ...a short-term fix to get around channel allocation... */
    XtVaSetValues(input1Widget,					    /* ...deficiencies in the OpenWindows server (the X11R5... */
		  XtNlive, True,				    /* ...server does not seem to exhibit this problem). */
		  NULL);
    XtVaGetValues(input1Widget,					    /* Determine the dimensions of the window from which to... */
		  XtNwidth, &movieWidth,			    /* ...record. */
		  XtNheight, &movieHeight,
		  NULL);
    recordSourceRect.x = 0;
    recordSourceRect.y = 0;
    recordSourceRect.width = movieWidth;
    recordSourceRect.height = movieHeight;
    XtVaGetValues(playbackWidget,				    /* Set up width, height, and Q factor for each frame in... */
		  XtNqFactor, &recordHeader.qfactor,		    /* ...this movie. */
		  NULL);
    XtVaSetValues(playbackWidget,
		  XtNsourceWin, XtWindow(input1Widget),		    /* Point to the video input widget's window */
		  XtNdestWin, 0,				    /* Don't draw compressed images; this saves time */
		  XtNwidth, movieWidth,				    /* Resize the widget to fit the size of the video input... */
		  XtNheight, movieHeight,			    /* ...widget's window. */
		  XtNcompressSourceRect, &recordSourceRect,	    /* Record a rectangle exactly the size of the video input... */
		  XtNcompressDestRect, &recordSourceRect,	    /* ...widget's window. */
		  XtNgetNewImage, True,				    /* Also force a single compression to incur any delays... */
		  NULL);					    /* ...before actual movie recording takes place. */
    if (usingSharedMemory == False)				    /* This code is written in the hope that the GetValues... */
    {								    /* ...call forces X events to be flushed, thereby... */
      XtVaGetValues(playbackWidget,				    /* ...avoiding the horrendously slow compression for the... */
		    XtNjpegData, &jpegImage,			    /* ...1st frame of video. */
		    NULL);
    }
    recordHeader.width = (int)movieWidth;
    recordHeader.height = (int)movieHeight;
    recordHeader.mapsize = 256 * 256 * 256;			    /* The movie doesn't use pseudo-color; all 24 bits are valid */
    if (silentMovie == True)					    /* Should audio be recorded along with the video? */
      recordHeader.tracks = 0;					    /* No, note this in the movie's header */
    else
      recordHeader.tracks = 1;					    /* Currently hard-coding a single track of audio */
    recordHeader.audioslice = audioBytesPerFrame;		    /* Set up the size in bytes of one frame of audio */
    bytesWritten = write(outputFile, &recordHeader, sizeof(jpheader)); /* Try to write the movie header to disk */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return immediately */
      sprintf(errorMessage,
	      "Could not write anything to the movie file.  Recording stopped.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);
      return;
    }
    recordFrameIndex[currentFrame] = sizeof(jpheader);		    /* The 1st frame of the movie begins after the header */
    if (maestroFD != (XtInputId)NULL)				    /* Is the application listening for MAEstro messages? */
    {								    /* Yes, continue */
      XtRemoveInput(maestroFD);					    /* Stop listening for MAEstro messages during recording */
      maestroFD = (XtInputId)NULL;				    /* Initialize the MAEstro file descriptor to be nothing */
    }
    recordWorkProcID = XtAppAddWorkProc(app_context,		    /* Start a work-proc to record a movie; it's done this... */
					RecordWorkProc,		    /* ...way so as not to block the user interface. */
					(XtPointer)NULL);
    totalBytesInRecording = sizeof(jpheader);			    /* Update size of this movie */
    framesSkippedDuringRecording = 0;				    /* Reset counter of number of video frames that were omitted */
    memset(recordSkippedFrameIndex, 0, MaxMovieDurationInFrames *   /* Initialize the skipped frames index before recording */
	   sizeof(float));
    memset(audioReadTime, 0, MaxMovieDurationInFrames *		    /* Initialize the audio read time array before recording */
	   sizeof(long));
    memset(compressTime, 0, MaxMovieDurationInFrames *		    /* Initialize the video compress time array before recording */
	   sizeof(long));
    memset(diskWriteTime, 0, MaxMovieDurationInFrames *		    /* Initialize the disk write time array before recording */
	   sizeof(long));
    gettimeofday(&recordStartTime, (struct timezone*)NULL);	    /* Note the time that movie recording was started */
    gettimeofday(&tempBefore, (struct timezone*)NULL);		    /* Note the time that movie recording was started */
  }
/*
  printf("Should really test if there's any disk space left to record.\n");
*/
  XtPopup(input1Shell, XtGrabNone);				    /* Assure that the video input widget is visible */
  gettimeofday(&tempAfter, (struct timezone*)NULL);		    /* Note the time that movie recording was started */
  audioFD = AudioRecordResume(audioFD);				    /* Allow audio to begin recording again */
/*
  printf("Time between opening audio device and exiting Record: %ld.\n",
	 ElapsedTime(&tempBefore, &tempAfter));
*/
  return;
} /* end function Record */



void Stop(Widget	w,
	  XtPointer	client_data,
	  XtPointer	xt_call_data)
{
  int				bytesWritten;
  struct timeval		playbackStopTime;
  struct timeval		recordStopTime;
  
  totalTimePaused = 0;						    /* No matter what, reset the total time paused */
  XtVaSetValues(speedSlider, XmNsensitive, True, NULL);	    /* Allow playback speed to change since movie is stopped */
  if (movieToolStatus == Playing)
  {
    gettimeofday(&playbackStopTime, (struct timezone*)NULL);	    /* Record the time this movie playback was stopped */
    XtRemoveWorkProc(playbackWorkProcID);			    /* Tell the toolkit to stop playback now */
    movieToolStatus = Stopped;					    /* Tell the app about its new state */
    if (userTurnedOffMonitor == False)				    /* Did the application itself disable the monitor during... */
      MonitorOn((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);    /* ...playback?  If so, turn it on again now that... */
    PrintPlaybackStatistics(&playbackStopTime, &movieHeader);	    /* Print info about playback performance */
  }
  else if (movieToolStatus == Recording)
  {
    gettimeofday(&recordStopTime, (struct timezone*)NULL);	    /* Record the time this movie recording was stopped */
    movieToolStatus = Stopped;					    /* Tell the app about its new state */
    audioFD = AudioRecordPause(audioFD);			    /* Stop recording audio; don't fill up audio buffers anymore */
    XtRemoveWorkProc(recordWorkProcID);				    /* Tell the toolkit to stop recording now */
    if (receiver != (Receiver*)NULL)				    /* Is MAEstro messaging active? */
      maestroFD = XtAppAddInput(app_context,			    /* Yes, start listening for MAEstro messages again */
				receiver->receiveSocket,
				(XtPointer) XtInputReadMask,
				CheckMAEstroMessages,
				(XtPointer)NULL);
    if (currentFrame == 0)					    /* If this is zero, there must've been a problem... */
    {								    /* ...recording the movie; there *was* no movie. */
      sprintf(errorMessage, "This movie is empty.  Removing the file.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);					    /* Close the file being recorded */
      unlink(outputFileName);					    /* Remove the file from the filesystem since it's no good */
      return;
    }
    recordHeader.frames = currentFrame;				    /* Store the number of frames in this movie */
    recordHeader.firstValidFrame = 0;				    /* Mark the whole movie as valid for playback */
    recordHeader.lastValidFrame = currentFrame - 1;
    recordHeader.indexbuf = totalBytesInRecording;		    /* Mark the location of the frame index locations */
    bytesWritten = write(outputFile,				    /* Write the frame index location info to the movie file */
			 recordFrameIndex,
			 currentFrame * sizeof(int));
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return immediately */
      sprintf(errorMessage,
	      "Could not write frame start location information to the movie file.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);
      return;
    }
    recordHeader.bandwidth =					    /* Store the average data rate for this movie */
      ((totalBytesInRecording / currentFrame) *
       recordHeader.fps) / 1024;
    lseek(outputFile, 0L, SEEK_SET);				    /* Go back to the start of the movie to write updated... */
    bytesWritten = write(outputFile, &recordHeader, sizeof(jpheader)); /* ...movie header information. */
    if (bytesWritten < 0)					    /* Did the attempt fail? */
    {								    /* Yes, report the error and return immediately */
      sprintf(errorMessage,
	      "Could not write an updated movie header to the movie file.\n");
      PrintErrorMessage(errorMessage);
      close(outputFile);
      return;
    }
    close(outputFile);						    /* Done with the output file; close it now */
    PrintRecordStatistics(&recordStopTime, &recordHeader);	    /* Print info about record performance */
    OpenMovie((Widget)NULL,					    /* Tell the app to make this movie available for playback... */
	      (XtPointer)outputFileName,			    /* ...so it can be reviewed. */
	      (XtPointer)NULL);
    currentFrame = firstFrame;					    /* Go to the beginning of the movie */
  } /* end else if Recording */
  return;
} /* end function Stop */



void AdvanceFrame(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* Single frame advance should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if (currentFrame < playbackEndFrame)				    /* Is it okay to go forward one frame? */
    PlayOneFrame(++currentFrame, False, True);			    /* Yes, play next frame in the movie, but don't play audio */
  return;
} /* end function AdvanceFrame */



void ReverseFrame(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* Single frame reverse should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if (currentFrame > playbackStartFrame)			    /* Is it okay to go back one frame? */
    PlayOneFrame(--currentFrame, False, True);			    /* Yes, play previous frame in the movie, don't play audio */
  return;
} /* end function ReverseFrame */



void EndOfMovie(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* End-of-movie should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  currentFrame = lastFrame;
  playbackEndFrame = lastFrame;
  PlayOneFrame(playbackEndFrame, False, True);			    /* Play the last frame of the movie, don't play audio */
  return;
} /* end function EndOfMovie */



void StartOfMovie(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (movieToolStatus == Playing)				    /* Did the viewer want to jump to the start of the movie... */
  {								    /* ...during playback?  If so, reset movie for playback: */
    framesSkippedDuringPlayback = 0;				    /* Reset counter of number of video frames that were omitted */
    memset(skippedFrameIndex, 0, MaxMovieDurationInFrames *	    /* Initialize the skipped frames index before playback */
	   sizeof(float));
    gettimeofday(&playbackStartTime, (struct timezone*)NULL);	    /* Record the time this movie playback was started */
    currentFrame = firstFrame;					    /* Go to the 1st valid frame of this movie */
    playbackStartFrame = firstFrame;				    /* Note that playback began at the 1st valid frame */
    return;
  }
  if ((movieToolStatus != Paused) &&				    /* Don't play a single frame unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  currentFrame = firstFrame;					    /* Go to the 1st valid frame of this movie */
  playbackStartFrame = firstFrame;				    /* Note that playback began at the 1st valid frame */
  PlayOneFrame(playbackStartFrame, False, True);		    /* Play the 1st frame of the movie, don't play audio */
  return;
} /* end function StartOfMovie */



void PrintMovieDiagnostics(char* 	fileName,
			   jpheader*	movieHeaderInfo)
{
  if (printDiagnostics == False)				    /* Don't bother printing performance stats if not requested */
    return;
  printf("Movie Information for the movie file %s:\n", fileName);
  printf("Frames per second: %d ", movieHeaderInfo->fps);
  printf("(%d milliseconds per frame.)\n", 1000 / movieHeaderInfo->fps);
  printf("Number of useful frames in the movie: %d.\n",
	 (movieHeaderInfo->lastValidFrame - movieHeaderInfo->firstValidFrame) + 1);
  printf("Total number of frames in the movie: %d.\n",
	 movieHeaderInfo->frames);
  return;
} /* end function PrintMovieDiagnostics */



void StartMovieHere(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (firstFrame == currentFrame)				    /* Don't bother doing work if the movie already starts here */
    return;
  firstFrame = currentFrame;					    /* Set the new starting frame for this movie */
  if (playbackStartFrame < firstFrame)				    /* Is the 1st frame of playback for this movie now invalid? */
    playbackStartFrame = firstFrame;				    /* Yes, update it to reflect the new movie size. */
  SetMovieStatus(MovieModified);				    /* Update interface to reflect change in movie file status */
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  UpdateTimecodeDisplay(movieHeader.fps);
  return;
} /* end function StartMovieHere */



void EndMovieHere(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (lastFrame == currentFrame)				    /* Don't bother doing work if the movie already ends here */
    return;
  lastFrame = currentFrame;					    /* Set the new ending frame for this movie */
  if (playbackEndFrame > lastFrame)				    /* Is the last frame for this movie now invalid? */
    playbackEndFrame = lastFrame;				    /* Yes, update it to reflect the new movie size */
  SetMovieStatus(MovieModified);				    /* Update interface to reflect change in movie file status */
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  UpdateTimecodeDisplay(movieHeader.fps);
  return;
} /* end function EndMovieHere */



void UseOriginalStartAndEndFrames(Widget	w,
				  XtPointer	client_data,
				  XtPointer	xt_call_data)
{
  firstFrame = (movieHeader.firstValidFrame < 0 ? 0 :		    /* Use the suggested start and end frames for this movie */
		movieHeader.firstValidFrame);
  lastFrame = (movieHeader.lastValidFrame > (movieHeader.frames -1) ?
	       movieHeader.frames -1 :
	       movieHeader.lastValidFrame);
  playbackStartFrame = firstFrame;
  playbackEndFrame = lastFrame;
  SetMovieStatus(MovieOK);
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  UpdateTimecodeDisplay(movieHeader.fps);
  return;
} /* end function UseOriginalStartAndEndFrames */



void PlusButtonCallback(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* Frame advance should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if ((w == hourPlusButton) &&					    /* Was the hours button pressed, and is there at least... */
      ((playbackEndFrame - currentFrame) >=			    /* ...one hour remaining in this movie? */
       (movieHeader.fps * 3600)))
  {
    currentFrame += movieHeader.fps * 3600;			    /* Yes, advance by one hour */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == minutePlusButton) &&				    /* Was the minutes button pressed, and is there at least... */
      ((playbackEndFrame - currentFrame) >=			    /* ...one minute remaining in this movie? */
       (movieHeader.fps * 60)))
  {
    currentFrame += movieHeader.fps * 60;			    /* Yes, advance by one minute */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == secondPlusButton) &&				    /* Was the seconds button pressed, and is there at least... */
      ((playbackEndFrame - currentFrame) >= movieHeader.fps))	    /* ...one second remaining in this movie? */
  {
    currentFrame += movieHeader.fps;				    /* Yes, advance by one second */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if (w == framePlusButton)					    /* Was the frames button pressed, and is there at least... */
  {
    if (currentFrame < playbackEndFrame)			    /* ...one frame remaining in this movie? */
      PlayOneFrame(++currentFrame, False, True);		    /* Yes, advance by one frame and play it */
  }
  return;
} /* end function PlusButtonCallback */



void MinusButtonCallback(Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if ((movieToolStatus != Paused) &&				    /* Frame reverse should not work unless a movie is... */
      (movieToolStatus != Stopped))				    /* ...either paused or stopped completely. */
    return;
  if ((w == hourMinusButton) &&
      ((currentFrame - firstFrame) >= (movieHeader.fps * 3600)))
  {
    currentFrame -= movieHeader.fps * 3600;			    /* Yes, rewind by one hour */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == minuteMinusButton) &&
      ((currentFrame - firstFrame) >= (movieHeader.fps * 60)))
  {
    currentFrame -= movieHeader.fps * 60;			    /* Yes, rewind by one minute */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == secondMinusButton) &&
      ((currentFrame - firstFrame) >= movieHeader.fps))
  {
    currentFrame -= movieHeader.fps;				    /* Yes, rewind by one second */
    PlayOneFrame(currentFrame, False, True);			    /* Play the frame just found */
  }
  if ((w == frameMinusButton) &&
      (currentFrame > firstFrame))
  {
    PlayOneFrame(--currentFrame, False, True);
  }
  return;
} /* end function MinusButtonCallback */



void ChangePlaybackVolume(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  SetAudioPlayVolume(audioFD, callData->value);
  return;
} /* end function ChangePlaybackVolume */



void ChangeRecordVolume(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  SetAudioRecordVolume(audioFD, callData->value);
  return;
} /* end function ChangeRecordVolume */



void ChangeMonitorVolume(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  Boolean			isMonitorOn;
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  isMonitorOn = XmToggleButtonGetState(monitorOnToggle);	    /* Ask the "Monitor On" toggle if it is on */
  if (isMonitorOn == False)					    /* Nope, it's not; forcibly turn it on */
  {
    MonitorOn((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  }
  SetAudioMonitorVolume(audioFD, callData->value);
  lastMonitorVolume = callData->value;				    /* Remember the new monitor volume in case monitor is... */
  return;							    /* ...turned off then on again later. */
} /* end function ChangeMonitorVolume */



void UseSpeakerForPlayback(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  unsigned int			audioDevice;
  
  SetPlaybackDevice(audioFD, AudioSpeaker);
  return;
} /* end function UseSpeakerForPlayback */



void UseHeadphonesForPlayback(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  unsigned int			audioDevice;
  
  SetPlaybackDevice(audioFD, AudioHeadphone);
  return;
} /* end function UseHeadphonesForPlayback */



void UseLineOutForPlayback(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct*	call_data =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  SetPlaybackDevice(audioFD, AudioLineOut);
} /* end function UseLineOutForPlayback */



void UseLineInForRecording(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct*	call_data =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  SetRecordDevice(audioFD, AudioLineIn);
} /* end function UseLineInForRecording */



void UseMicForRecording(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmToggleButtonCallbackStruct *call_data = (XmToggleButtonCallbackStruct *) xt_call_data;
  
  SetRecordDevice(audioFD, AudioMicrophone);
} /* end function UseMicForRecording */



void SetMovieFileFormat(Widget		w,
			XtPointer	clientData,
			XtPointer	xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
/*
  printf("Called SetMovieFileFormat; movie type is ::%s::.\n", clientData);
*/
  if (fileFormat != (char*)NULL)				    /* Free up any space previously pointed to by this variable */
    free(fileFormat);
  fileFormat = strdup(clientData);				    /* Set a variable determining file format for movies */
  return;
} /* end function SetMovieFileFormat */



void PrintDiagnostics(char* diagString)
{
  if (printDiagnostics == True)
    printf(diagString);
  return;
} /* end function PrintDiagnostics */



void HoursTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function HoursTextFieldCallback */



void MinutesTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function MinutesTextFieldCallback */



void SecondsTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function SecondsTextFieldCallback */



void FramesTextFieldCallback(Widget	w,
			    XtPointer	client_data,
			    XtPointer	xt_call_data)
{
  char*	textString;
  
  textString = XmTextFieldGetString(w);
  XtFree(textString);
  return;
} /* end function FramesTextFieldCallback */


Boolean AutoPlayWorkProc(XtPointer unusedClientData)
{
  Play((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  return(True);
} /* end function QuitWorkProc */


void AutoPlay(void)
{
  XtAppAddWorkProc(app_context,		
		   AutoPlayWorkProc,	
		   (XtPointer)NULL);	
  return;				
}


static void ResetAudioDevice(XtPointer client_data, XtIntervalId *id)
{
  /* Audio device is hung. */
  printf("congratulations on your sparc-5 purchase\n");
  Stop((Widget)NULL,
       (XtPointer)NULL,
       (XtPointer)NULL);
  AudioPlayPause(audioFD);
  sleep(1);
  AudioPlayResume(audioFD);
  sleep(1);
  AudioPlayPause(audioFD);
  sleep(1);
  Play((Widget)NULL,
       (XtPointer)NULL,
       (XtPointer)NULL);
}


void WMDeleteWindowCallback(Widget	widget,
			    XtPointer	clientData,
			    XtPointer	callData)
{
  if (widget == mainWindow)					    /* Was the window manager used to quit the app instead of... */
  {								    /* ...the app's File menu "Quit" item? */
    Quit(widget, (XtPointer)NULL, (XtPointer)NULL);		    /* Yes, call the normal Quit callback anyway */
  }
  return;
} /* end function WMDeleteWindowCallback */
