/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/fileUtils.c,v 1.18 1994/09/13 08:09:33 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: fileUtils.c,v $
 * Revision 1.18  1994/09/13  08:09:33  drapeau
 * Several changes:
 *
 * * Removed memory leaks in CompactMovie().
 *
 * * Fixed an error in CompactMovie(), which had allocated one too few
 *   integers that kept track of the starting points for each frame on
 *   the compacted movie file.
 *
 * * Fixed OpenMovie() so that it is less severe when opening a movie for
 *   which it cannot play audio.  It will now agree to play the movie,
 *   but will simply enter "silent" mode.
 *
 * Revision 1.17  1994/08/25  04:56:02  drapeau
 * Removed memory leaks found by Purify.  All known memory leaks are now
 * gone.
 *
 * Revision 1.16  1994/08/23  01:33:42  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.15  1994/06/27  22:58:57  drapeau
 * Modified the OpenMovie() function, removing an extraneous call to
 * PlayOneFrame(), since the StartOfMovie() function already calls it.
 * This fixed a problem in which the Motif toolkit displayed an error
 * message saying that the searchSlider widget was set to a value greater
 * than its maximum value.
 *
 * Revision 1.14  1994/06/15  03:55:06  drapeau
 * Modified OpenMovie() to support reading JPEG Movie files that were
 * made by older programs.  Those programs (and this version of
 * MovieTool) use Sun-specific audio encoding information.  The new
 * modifications translate the Sun-specific information into the
 * PortableAudio defined constants.
 *
 * Revision 1.13  1994/06/10  01:24:59  drapeau
 * Minor change to OpenMovie(), assuring that the movie shell sizes to
 * fit the newly-opened movie's dimensions.
 *
 * Revision 1.12  1994/05/31  20:10:48  drapeau
 * Removed a memory allocation problem during playback, when not using
 * shared memory JPEG images.  Also, cleaned up diagnostic reporting.
 *
 * Revision 1.11  1994/01/20  18:04:12  drapeau
 * Fixed an error in closing the Movie file: it was wrongly considered as
 * type "int" when in fact it was of type "FILE*".
 *
 * Also, modified code to support the PortableAudio interface.
 *
 * Also, modified code to support the case where the JPEG widget cannot
 * use shared memory.
 *
 * Revision 1.10  1994/01/10  22:38:49  drapeau
 * First working version of this module.
 * */
static char fileUtilsRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/fileUtils.c,v 1.18 1994/09/13 08:09:33 drapeau Exp $";

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/PushBG.h>
#include "MovieTool.h"



void OpenMovie(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  char*					fileName = NULL;
  char*					imageData = NULL;
  char					tempString[32];
  int					fileType;
  int					bytesRead;
  int					audioResult;
  Boolean				result;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  if (w != (Widget)NULL)					    /* Was this function called by the toolkit? */
  {								    /* Yes, not called by the application programmer  */
    result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			     XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			     &fileName);
    if (result == False)					    /* Could not determine the filename that was typed */
    {
      XtUnmanageChild(fileDialog);
      sprintf(errorMessage,
	      "The filename you typed included unprintable characters.  Please try again.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
  }
  else								    /* The function was called by the application programmer */
    fileName = client_data;					    /* The filename to open was passed in as an argument */
  if (movieFile != (FILE*)NULL)					    /* Was there an already open movie? */
    fclose(movieFile);						    /* Yes, close it before opening a new movie */
  movieFile = fopen(fileName, "r");				    /* Attempt to open the requested file */
  if (movieFile == (FILE*)NULL)					    /* Did the attempt fail? */
  {
    XtUnmanageChild(fileDialog);
    sprintf(errorMessage,
	    "Could not open the movie file %s.\n", fileName);
    PrintErrorMessage(errorMessage);
    return;
  }
  memset(tempString, 0, 10);					    /* Initialize the temporary string */
  bytesRead = fread(tempString, 1, strlen("j_movie"), movieFile);   /* Read in the 1st few bytes of the requested file */
  if (strcmp(tempString, "j_movie") != 0)			    /* Is this a JPEG Movie file?  If not, report the error... */
  {								    /* ...and return now. */
    XtUnmanageChild(fileDialog);
    sprintf(errorMessage,
	    "The file %s is not a JPEG Movie file.\n", fileName);
    PrintErrorMessage(errorMessage);
    return;
  }
  fseek(movieFile, 0L, 0);					    /* Go back to the beginning of the movie file */
  fread((char*)&movieHeader, sizeof(jpheader), 1, movieFile);	    /* Read in the movie's header information */
  if (printDiagnostics == True)					    /* Should movie info be printed? */
  {								    /* Yes, tell the author something about the movie */
    printf("Movie Information for %s:\n", fileName);
    printf("Audio:\tChannels: %d\n", movieHeader.audio.channels);
    printf("\tSample Rate: %d / second\n", movieHeader.audio.sample_rate);
    printf("\tEncoding: %d\n", movieHeader.audio.encoding);
    printf("\tBytes per unit: %d\n", movieHeader.audio.bytes_per_unit);
    printf("\tSamples per unit: %d\n", movieHeader.audio.samples_per_unit);
  }
  if (movieHeader.tracks > 0)					    /* Does this movie have audio?  If so, do the right thing. */
  {
    if (audioData != (char*)NULL)
      free(audioData);
    audioData = malloc(movieHeader.audioslice);			    /* Allocate enough memory to hold one frame of audio */
    if (audioData == (char*)NULL)				    /* Did the attempt fail? */
    {								    /* Yes, report the error and do nothing about it for now */
      sprintf(errorMessage,
	      "Could not allocate the memory necessary to playback audio.  Please try again or quit MovieTool and restart it.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    else
      silentMovie = False;					    /* Indicate that it's okay to play audio with this movie */
  }
  else
    silentMovie = True;						    /* Indicate that it's not okay to play audio w/this movie */
  audioPlayHeader.sample_rate = movieHeader.audio.sample_rate;	    /* Load the movie's audio info to configure the audio... */
  audioPlayHeader.samples_per_unit =				    /* ...device correctly. */
    movieHeader.audio.samples_per_unit;
  audioPlayHeader.bytes_per_unit = movieHeader.audio.bytes_per_unit;
  audioPlayHeader.channels = movieHeader.audio.channels;
  audioPlayHeader.data_size = movieHeader.audio.data_size;
  switch (movieHeader.audio.encoding)				    /* Convert audio encoding information read from the movie... */
  {								    /* ...file into the new, portable audio... */
   case AUDIO_ENCODING_ULAW:					    /* ...encoding information, in order to maintain... */
    audioPlayHeader.encoding = AudioULawEncoding;		    /* ...compatibility with older JPEG Movie recorders and... */
    break;							    /* ...players. */
   case AUDIO_ENCODING_ALAW:
    audioPlayHeader.encoding = AudioALawEncoding;
    break;
   case AUDIO_ENCODING_LINEAR:
    audioPlayHeader.encoding = AudioLinearEncoding;
    break;
  }
  audioResult = SetAudioPlaybackConfiguration(audioFD,		    /* Try to correctly configure the audio device */
					      &audioPlayHeader);
/*
  printf("Just after setting the play configuration in OpenMovie().\n");
*/
  PrintAudioInfo(audioFD, &audioInfo);
  if (audioResult != 0)						    /* Did the attempt fail? */
  {
    silentMovie = True;						    /* Yes, turn off audio for this movie */
    sprintf(errorMessage,
	    "OpenMovie(): Could not prepare the audio device to play this movie; the movie will play back in silence.");
    PrintErrorMessage(errorMessage);
/*
    return;
*/
  }
  if (usingSharedMemory == False)				    /* Is the JPEG widget using shared memory?  If not,... */
  {								    /* ...manually create a Parallax JPEG image data... */
    jpegImage = XPlxCreateCImage(XtDisplay(playbackWidget),	    /* ...structure (the function will allocate the necessary... */
				 NULL,				    /* ...memory for storing a single JPEG image). */
				 OneImageSizeInBytes,
				 movieHeader.width,
				 movieHeader.height);
    XtVaSetValues(playbackWidget,				    /* Tell the JPEG widget to point to the newly-created... */
		  XtNjpegData, jpegImage,			    /* Parallax JPEG image structure. */
		  NULL);
  }
  XtVaSetValues(movieShell,					    /* Make the Movie Shell fit the movie size perfectly */
		XtNwidth, movieHeader.width,
		XtNheight, movieHeader.height,
		NULL);
  XtVaSetValues(playbackWidget,					    /* Resize the movie playback widget to fit the movie */
		XtNwidth, movieHeader.width,
		XtNheight, movieHeader.height,
		XtNqFactor, movieHeader.qfactor,
		XtNdestWin, XtWindow(playbackWidget),		    /* If drawing was turned off, turn it on again for playback */
		NULL);
  jpegImage->width = movieHeader.width;				    /* Set variables for the width and height of the JPEG... */
  jpegImage->height = movieHeader.height;			    /* ...images in this movie. */
  playbackSpeed = movieHeader.fps;				    /* Reset the playback speed for MovieTool to that of... */
  XtVaSetValues(speedSlider,					    /* ...the movie; also update the proper interface component. */
		XmNvalue, playbackSpeed, NULL);
  
  fseek(movieFile, movieHeader.indexbuf, 0);			    /* Look for the information about frame start positions */
  if (frameLocation != (int*)NULL)				    /* Free up any previously-allocated space no longer needed */
    free(frameLocation);
  frameLocation = (int*)malloc(movieHeader.frames * sizeof(int));   /* Allocate space for starting byte #s for each frame */
  fread((char*)frameLocation, sizeof(int), movieHeader.frames,	    /* Read the frame index info into memory */
	movieFile);
  if (videoFrameIndex != (char*)NULL)				    /* Free up any previously-allocated space no longer needed */
    free(videoFrameIndex);
  videoFrameIndex = malloc(movieHeader.frames * sizeof(char));	    /* Allocate space to note if video is present in each frame */
  memset(videoFrameIndex, 0, movieHeader.frames * sizeof(char));    /* Reset the array */
  if (movieHeader.version == JPEG_VERSION_3)			    /* Is this the latest JPEG Movie file format? */
  {								    /* Yes, use the new information for stand & end frames */
    firstFrame = (movieHeader.firstValidFrame < 0 ? 0 :		    /* Read in the suggested start and end frames for this movie */
		  movieHeader.firstValidFrame);
    lastFrame = (movieHeader.lastValidFrame > (movieHeader.frames -1) ?
		 movieHeader.frames -1 :
		 movieHeader.lastValidFrame);
  }
  else								    /* No, use the whole movie for playback */
  {
    firstFrame = 0;
    lastFrame = movieHeader.frames - 1;
    movieHeader.firstValidFrame = firstFrame;
    movieHeader.lastValidFrame = lastFrame;
  }
  playbackStartFrame = firstFrame;				    /* By default, play all of the valid movie, as suggested... */
  playbackEndFrame = lastFrame;					    /* ...by the first and last ValidFrame fields. */
  UpdateMovieLengthIndicators();				    /* Update interface parts for movie length, start, & end */
  XtUnmanageChild(fileSelectionBox);
  PrintMovieDiagnostics(fileName, &movieHeader);
  strcpy(currentMovieFileName, fileName);			    /* Update MovieTool's idea of its currently running movie */
  SetMovieStatus(MovieOK);					    /* This just-opened movie has not been edited */
  StartOfMovie((Widget)NULL, (XtPointer)NULL, (XtPointer)NULL);
  EnableTransportButtons();
  XtPopup(movieShell, XtGrabNone);
  return;
} /* end function OpenMovie */



void SaveMovie(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  char*					fileName = NULL;
  Boolean				result;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			   XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			   &fileName);
  if (result == False)						    /* Could not determine the filename that was typed */
  {
    XtUnmanageChild(fileSelectionBox);
    sprintf(errorMessage,
	    "The filename you typed included unprintable characters.  Please try again.\n");
    PrintErrorMessage(errorMessage);
    free(fileName);						    /* Free space to hold the file name no longer needed */
    return;
  }
  sprintf(outputFileName, "%s", fileName);			    /* Tell the app the new file in which to store movies */
  free(fileName);						    /* Free space to hold the file name no longer needed */
  strcpy(currentMovieFileName, outputFileName);
  SetMovieStatus(MovieNew);					    /* Update interface to reflect change in movie file status */
  XtUnmanageChild(fileSelectionBox);
  return;
} /* end function SaveMovie */



void CompactMovie(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  int					i;
  int					counter;
  int					statResult;
  int					bytesWritten = 0;
  int					jpegImageSize;
  static int*				compactIndex = NULL;
  char*					fileName = NULL;
  static char*				jpegImageData = NULL;
  FILE*					compactFile;
  unsigned char				instruction;
  static Boolean			firstTime = True;
  Boolean				done = False;
  Boolean				result;
  struct stat				fileInfo;
  int					totalFrames;
  jpheader				compactHeader;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
/*
  printf("-----CompactMovie(): Need to check if source movie file is open or something.\n");
  printf("-----Maybe disable the menu item until a valid movie is open.\n");
*/
  if (firstTime == True)
  {
    firstTime = False;
    jpegImageData = malloc(256 * 1024);				    /* Allocate 256KBytes for JPEG image storage */
  }
  result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			   XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			   &fileName);
  if (result == False)						    /* Could not determine the filename that was typed */
  {
    XtUnmanageChild(fileSelectionBox);
    sprintf(errorMessage,
	    "The filename you typed included unprintable characters.  Please try again.\n");
    PrintErrorMessage(errorMessage);
    return;
  }
  if (strcmp(fileName, currentMovieFileName) == 0)		    /* Did the author try to save the new movie with the same... */
  {								    /* ...name as the existing one?  This is a problem */
    sprintf(errorMessage,
	    "Please choose another name for your movie; %s is your current movie.\n",
	    currentMovieFileName);
    PrintErrorMessage(errorMessage);
    free(fileName);						    /* Return filename string no longer needed */
    return;
  }
  statResult = stat(fileName, &fileInfo);			    /* Try to get information about the requested file */
  if (statResult == 0)						    /* Did the file already exist? */
  {								    /* Yes, do further error prevention checking */
    if (S_ISREG(fileInfo.st_mode) == 0)				    /* Is the requested file a regular file?  Anything else... */
    {								    /* ...is an error. */
      sprintf(errorMessage,
	      "The file you asked for (%s) is not a regular file; please choose another.\n",
	      fileName);
      PrintErrorMessage(errorMessage);
      free(fileName);						    /* Return filename string no longer needed */
      return;
    }
/*
    printf("-----Here I should prompt if the movie should be overwritten.\n");
    printf("-----If not, then close the compactFile file and return immediately,\n");
    printf("-----first suggesting the author choose another name for the file.\n");
*/
  }
  compactFile = fopen(fileName, "w");
  if (compactFile == (FILE*)NULL)
  {
    sprintf(errorMessage,
	    "Could not open the new file for writing.  Outtahere.\n");
    PrintErrorMessage(errorMessage);
    free(fileName);						    /* Return filename string no longer needed */
    return;
  }
/*
  printf("\n-----Should pop down file dialog and place an alert message or something.\n");
*/
  if (compactIndex != (int*)NULL)				    /* Was space already allocated for the newly-compacted... */
    free(compactIndex);						    /* ...movie's index?  If so, free that space. */
  totalFrames = lastFrame - firstFrame + 1;			    /* Calculate how many frames to compact and save */
  compactIndex = (int*)malloc(totalFrames *			    /* Try to allocate space to store frame start points ... */
			      sizeof(int));			    /* ...for the new movie.  */
  if (compactIndex == (int*)NULL)				    /* Did the attempt fail? */
  {
    sprintf(errorMessage,
	    "Could not allocate enough space to store info about the new movie's starting frame points; ran out of memory.\n");
    PrintErrorMessage(errorMessage);
    free(fileName);						    /* Return filename string no longer needed */
    return;
  }
  memset(compactIndex, 0, totalFrames);				    /* Initialize the starting point array before compacting */
  strcpy(compactHeader.magic, "j_movie");			    /* Set up the movie header for the compacted movie */
  compactHeader.version = JPEG_VERSION_2;
  compactHeader.fps = movieHeader.fps;
  compactHeader.frames = totalFrames;
  compactHeader.width = movieHeader.width;
  compactHeader.height = movieHeader.height;
  compactHeader.bandwidth = movieHeader.bandwidth;
  compactHeader.qfactor = movieHeader.qfactor;
  compactHeader.mapsize = movieHeader.mapsize;
  compactHeader.tracks = movieHeader.tracks;
  compactHeader.volbase = movieHeader.volbase;
  compactHeader.audioslice = movieHeader.audioslice;
  compactHeader.audio.sample_rate = movieHeader.audio.sample_rate;
  compactHeader.audio.samples_per_unit = movieHeader.audio.samples_per_unit;
  compactHeader.audio.bytes_per_unit = movieHeader.audio.bytes_per_unit;
  compactHeader.audio.channels = movieHeader.audio.channels;
  compactHeader.audio.encoding = movieHeader.audio.encoding;
  compactHeader.audio.data_size = movieHeader.audio.data_size;
  fwrite(&compactHeader, sizeof(jpheader), 1, compactFile);	    /* Write the compact movie's header to the new compact file */
  bytesWritten = sizeof(jpheader);
  for (i = 0, counter = firstFrame; i < compactHeader.frames;	    /* Copy only the valid part of the original movie into... */
       i++, counter++)						    /* ...the newly-compacted movie. */
  {
    compactIndex[i] = bytesWritten;				    /* Mark the start byte for the compacted movie */
    fseek(movieFile, frameLocation[counter], 0);		    /* Go to the next frame in the original movie */
    done = False;						    /* Reset variable, telling the program to read a new frame */
    while (done == False)					    /* Keep reading until all the instructions in this frame... */
    {								    /* ...have been executed. */
      fread(&instruction, 1, 1, movieFile);			    /* Read the 1st opcode for this frame */
      fwrite(&instruction, 1, 1, compactFile);			    /* Write the opcode to the compact movie */
      bytesWritten += 1;
      instruction &= 0x00FF;
      switch ((int)instruction)					    /* Do the appropriate action for this opcode */
      {
       case END_FRAME:						    /* Was the end of this frame reached? */
	done = True;						    /* Yes, set a variable saying so */
	break;
       case LOAD_JPEG:
	fread((char*)&(jpegImageSize), sizeof(int), 1, movieFile);  /* Read the size of this JPEG image in bytes */
	fwrite((char*)&(jpegImageSize), sizeof(int), 1, compactFile); /* Write the size of this JPEG image to the compact movie */
	bytesWritten += sizeof(int);
	fread(jpegImageData, jpegImageSize, 1, movieFile);	    /* Read the JPEG image data */
	fwrite(jpegImageData, jpegImageSize, 1, compactFile);	    /* Write the JPEG image data to the compact movie */
	bytesWritten += jpegImageSize;
	break;
       case LOAD_AUDIO_0:
       case LOAD_AUDIO_1:
       case LOAD_AUDIO_2:
       case LOAD_AUDIO_3:
	fread(audioData, 1, movieHeader.audioslice, movieFile);	    /* Read the audio data for this frame */
	fwrite(audioData, 1, movieHeader.audioslice, compactFile);  /* Write the audio data for this frame to the compact movie */
	bytesWritten += movieHeader.audioslice;
	break;
       default:
/*
	printf("An unknown instruction (%d) was found in frame %d of the movie, skipping.\n",
	       instruction, currentFrame);
*/
	break;
      }	/* end switch */
    } /* end while */
  } /* end for */
  compactHeader.indexbuf = bytesWritten;			    /* Update where the index markers will begin for this movie */
  fwrite(compactIndex, compactHeader.frames, sizeof(int), compactFile);	/* Write the index markers for the compacted movie */
  fseek(compactFile, 0, 0);					    /* Go to the beginning on the compacted movie */
  fwrite(&compactHeader, sizeof(jpheader), 1, compactFile);	    /* Write the updated header to the new compact file */
  fclose(compactFile);
  SetMovieStatus(MovieOK);					    /* Update interface to reflect change in movie file status */
  if (strcmp(fileFormat, "MPEG") == 0)				    /* Shall this movie be saved as an MPEG movie? */
  {
    SaveMPEGFile(fileName);					    /* Yes, run the appropriate program to save an MPEG movie */
  }
  XtUnmanageChild(fileSelectionBox);
  free(fileName);						    /* Return filename string no longer needed */
  return;
} /* end function CompactMovie */



void ShowOpenFileDialog(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(fileDialog,
		XmNtitle, "Open",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie,NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie,NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtAddCallback(fileSelectionBox, XmNokCallback, OpenMovie,NULL);
  XtVaSetValues(saveAsOptionButton, XmNsensitive, False, NULL);	    /* Don't allow author to choose final movie output format */
  XtManageChild(fileSelectionBox);
  return;
}								    /* end function ShowOpenFileDialog */



void ShowSaveFileDialog(Widget		w,
			XtPointer	client_data,
			XtPointer	xt_call_data)
{
  XtVaSetValues(fileDialog,
		XmNtitle, "Save",
		NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, SaveMovie,NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, OpenMovie,NULL);
  XtRemoveCallback(fileSelectionBox, XmNokCallback, CompactMovie, NULL);
  XtAddCallback(fileSelectionBox, XmNokCallback, SaveMovie,NULL);
  XtVaSetValues(saveAsOptionButton, XmNsensitive, False, NULL);	    /* Don't allow author to choose final movie output format */
  XtManageChild(fileSelectionBox);
  return;
}								    /* end function ShowSaveFileDialog */


void SaveMovieChanges(Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data)
{
  int	theFile;
  int	bytesWritten;
  
  movieHeader.firstValidFrame = firstFrame;
  movieHeader.lastValidFrame = lastFrame;
  fclose(movieFile);						    /* Close the currently-opened movie being modified */
  theFile = open(currentMovieFileName, (O_WRONLY), 0644);	    /* Try to open the current movie for writing */
  if (theFile == -1)
  {
    sprintf(errorMessage, "Could not save your changes.\n");
    PrintErrorMessage(errorMessage);
    return;
  }
  bytesWritten = write(theFile, &movieHeader, sizeof(jpheader));    /* Try to write the new movie header to disk */
  if (bytesWritten < 0)						    /* Did the attempt fail? */
  {								    /* Yes, report the error and return immediately */
    sprintf(errorMessage,
	    "Could not write anything to the movie file.\n");
    PrintErrorMessage(errorMessage);
    close(theFile);
    return;
  }
  close(theFile);						    /* Done writing changes; close the file */
  OpenMovie((Widget)NULL,					    /* Re-open the newly-changed file for playback */
	    (XtPointer)currentMovieFileName,
	    (XtPointer)NULL);
  return;
} /* end function SaveMovieChanges */
