/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/hpAudio.c,v 0.97 1994/08/23 05:09:34 drapeau Exp $ */
/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: hpAudio.c,v $
 * Revision 0.97  1994/08/23  05:09:34  drapeau
 * Modified InitializeAudio(), adding code to initialize the default
 * audioPlayHeader and audioRecordHeader data structures, thereby
 * removing a divide-by-zero error.
 *
 * Revision 0.96  1994/08/23  01:34:03  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 0.95  1994/08/11  03:17:26  drapeau
 * Added an experimental function for synchronizing audio with video.  It
 * didn't really do any good thus far, so a return stmt was placed at the
 * beginning of the function.
 *
 * Revision 0.94  1994/06/24  01:16:24  drapeau
 * Added the function AudioClose() to properly close the audio device.
 *
 * Revision 0.93  1994/06/10  01:25:50  drapeau
 * Minor changes to the HP audio support for the first St. Filbert
 * release.  Also, added code to support command-line specification of a
 * sample rate.
 *
 * Revision 0.92  1994/05/31  20:11:50  drapeau
 * Enhanced support for audio on the HP platform, removing several errors
 * and making the software more complete.  Also, cleaned up diagnostic
 * reporting.
 *
 * Revision 0.91  1994/01/20  22:49:05  drapeau
 * Modified PortableAudio functions to accommodate a change in the Sun
 * implementation, which in my opinion is due to a flawed audio device
 * driver.
 *
 * Revision 0.90  1994/01/20  17:50:45  drapeau
 * This is the first working prototype, developed with the HP
 * prototype board.
 * */
static char audioFunctionsRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/hpAudio.c,v 0.97 1994/08/23 05:09:34 drapeau Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <sys/audio.h>
#include "PortableAudio.h"
#include "MovieTool.h"

static struct audio_gain	hpAudioVolumeInfo;
static struct audio_describe	hpGainInfo;
static int			playRange;
static int			recordRange;
static int			monitorRange;


int AudioRecordOpen(void)
{
  int		audioFD;
  int		result;
  
  audioFD = open("/dev/audio", O_RDONLY);			    /* Try to open the audio device for reading (record) only */
  if (audioFD == -1)						    /* Did the open fail? */
  {								    /* Yes, report the error and exit immediately */
    sprintf(errorMessage,
	    "Sorry, could not open the audio device for recording.  Exiting.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
  return(audioFD);
} /* end function AudioRecordOpen */



int AudioRecordPause(int audioFD)
{
  ioctl(audioFD, AUDIO_RESET, RESET_RX_BUF);
  ioctl(audioFD, AUDIO_PAUSE, AUDIO_RECEIVE);
  return(audioFD);						    /* The same device can be used for play & record, return it */
} /* end function AudioRecordPause */



int AudioRecordResume(int audioFD)
{
  ioctl(audioFD, AUDIO_RESUME, AUDIO_RECEIVE);
  return(audioFD);						    /* The same device can be used for play & record, return it */
} /* end function AudioRecordResume */



int AudioPlaybackOpen(void)
{
  int			audioFD;
  int			result;
  
  audioFD = open("/dev/audio", (O_WRONLY));
  if (audioFD < 0)						    /* Did the open fail? */
  {								    /* Yes, report the error and exit immediately */
    sprintf(errorMessage,
	    "Sorry, could not open the audio device for playback.\n");
    PrintErrorMessage(errorMessage);
    return(audioFD);
  }
  return(audioFD);
} /* end function AudioPlaybackOpen */



int AudioPlayPause(int audioFD)
{
  ioctl(audioFD, AUDIO_PAUSE, AUDIO_TRANSMIT);
  return(audioFD);						    /* The same device can be used for play & record, return it */
} /* end function AudioPlayPause */



int AudioPlayResume(int audioFD)
{
  ioctl(audioFD, AUDIO_RESUME, AUDIO_TRANSMIT);
  return(audioFD);						    /* The same device can be used for play & record, return it */
} /* end function AudioPlayResume */



void PrintAudioInfo(int			audioInfoFD,
		    PortableAudio*	audioInfo)
{
  int			deviceType;
  int			result;
  int			audioFormat;
  struct audio_describe	hpAudioInfo;
  struct audio_gain	audioVolumeInfo;
  char			diagString[128];
  
  PrintDiagnostics("Entered PrintAudioInfo.\n");
  result = ioctl(audioInfoFD, AUDIO_DESCRIBE, &hpAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  audioInfo->play.sampleRate =					    /* What the hell, choose the highest-quality sample rate... */
    hpAudioInfo.sample_rate[hpAudioInfo.nrates - 1];		    /* ...available. */
  audioInfo->record.sampleRate =
    hpAudioInfo.sample_rate[hpAudioInfo.nrates - 1];
  audioInfo->play.sampleRate = sampleRate;
  audioInfo->record.sampleRate = sampleRate;
  sprintf(diagString, "PrintAudioInfo(): Highest alleged sample rate is %d.\n",
	 hpAudioInfo.sample_rate[hpAudioInfo.nrates - 1]);
  PrintDiagnostics(diagString);
  audioInfo->play.channels = hpAudioInfo.nchannels;
  audioInfo->record.channels = hpAudioInfo.nchannels;
  audioInfo->play.bitsPerSample = hpAudioInfo.max_bits_per_sample;
  audioInfo->record.bitsPerSample = hpAudioInfo.max_bits_per_sample;
  result = ioctl(audioInfoFD, AUDIO_GET_DATA_FORMAT, &audioFormat);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EINVAL:
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  switch (audioFormat)						    /* What format audio is currently being used? */
  {
   case AUDIO_FORMAT_ULAW:
    PrintDiagnostics("Encoding: uLaw.\n");
    audioInfo->play.encoding = AudioULawEncoding;
    audioInfo->record.encoding = AudioULawEncoding;
    break;
   case AUDIO_FORMAT_ALAW:
    PrintDiagnostics("Encoding: aLaw.\n");
    audioInfo->play.encoding = AudioALawEncoding;
    audioInfo->record.encoding = AudioALawEncoding;
    break;
   case AUDIO_FORMAT_LINEAR16BIT:
    PrintDiagnostics("Encoding: Linear.\n");
    audioInfo->play.encoding = AudioLinearEncoding;
    audioInfo->record.encoding = AudioLinearEncoding;
    break;
  } /* end switch */
  result = ioctl(audioInfoFD, AUDIO_GET_GAINS, &audioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  result = ioctl(audioInfoFD, AUDIO_GET_GAINS, &hpAudioVolumeInfo);
  audioInfo->play.volume = audioVolumeInfo.cgain[0].transmit_gain;
  audioInfo->record.volume = audioVolumeInfo.cgain[0].receive_gain;
  audioInfo->monitorVolume = audioVolumeInfo.cgain[0].monitor_gain;
  hpAudioVolumeInfo.channel_mask = (AUDIO_CHANNEL_LEFT | AUDIO_CHANNEL_RIGHT);
  audioVolumeInfo.channel_mask = (AUDIO_CHANNEL_LEFT | AUDIO_CHANNEL_RIGHT);
  XtVaSetValues(playbackScale,
		XmNvalue,
		((audioInfo->play.volume - hpGainInfo.min_transmit_gain)
		 * 100) / playRange,
		NULL);
  XtVaSetValues(recordScale,
		XmNvalue,
		((audioInfo->record.volume - hpGainInfo.min_receive_gain)
		 * 100) / recordRange,
		NULL);
  XtVaSetValues(monitorScale,
		XmNvalue,
		((audioInfo->monitorVolume - hpGainInfo.min_monitor_gain)
		 * 100) / monitorRange,
		NULL);
  PrintDiagnostics("Audio information:\n");
  PrintDiagnostics("Device type: HP audio.\n");
  if (hpAudioInfo.audio_id == AUDIO_ID_PSB2160)
    PrintDiagnostics("Low quality audio.\n");
  else
    PrintDiagnostics("High quality audio.\n");
  
  sprintf(diagString, "Sample rate: %d.\n", audioInfo->play.sampleRate);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Number of channels: %d.\n", audioInfo->play.channels);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Bits per sample: %d.\n", audioInfo->play.bitsPerSample);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Playback volume: %d.\n", audioInfo->play.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Record volume: %d.\n", audioInfo->record.volume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Monitor volume: %d.\n", audioInfo->monitorVolume);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum transmit gain: %d; Max: %d.\n",
	 hpAudioInfo.min_transmit_gain,
	 hpAudioInfo.max_transmit_gain);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum receive gain: %d; Max: %d.\n",
	 hpAudioInfo.min_receive_gain,
	 hpAudioInfo.max_receive_gain);
  PrintDiagnostics(diagString);
  sprintf(diagString, "Minimum monitor gain: %d; Max: %d.\n",
	 hpAudioInfo.min_monitor_gain,
	 hpAudioInfo.max_monitor_gain);
  PrintDiagnostics(diagString);
  return;
} /* end function PrintAudioInfo */



int InitializeAudio(void)
{
  int			audioFD;
  int			audioFormat;
  int			result;
  int			tempValue;
  struct audio_describe	hpAudioInfo;
  struct audio_gain	audioVolumeInfo;
  
  PrintDiagnostics("Entered InitializeAudio.\n");
  audioFD = open("/dev/audio", O_RDWR);				    /* Try to open the audio device */
  if (audioFD == -1)						    /* Did the open fail? */
  {								    /* Yes, report the error and exit immediately */
    sprintf(errorMessage,
	    "Sorry, could not open the audio device.  Exiting.\n");
    PrintErrorMessage(errorMessage);
    return(audioFD);
  }
  result = ioctl(audioFD, AUDIO_DESCRIBE, &hpAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return(audioFD);
  }
  result = ioctl(audioFD, AUDIO_GET_DATA_FORMAT, &audioFormat);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EINVAL:
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  switch (audioFormat)						    /* What format audio is currently being used? */
  {
   case AUDIO_FORMAT_ULAW:
    audioPlayHeader.encoding = AudioULawEncoding;
    audioRecordHeader.encoding = AudioULawEncoding;
    break;
   case AUDIO_FORMAT_ALAW:
    audioPlayHeader.encoding = AudioALawEncoding;
    audioRecordHeader.encoding = AudioALawEncoding;
    break;
   case AUDIO_FORMAT_LINEAR16BIT:
    audioPlayHeader.encoding = AudioLinearEncoding;
    audioRecordHeader.encoding = AudioLinearEncoding;
    break;
  } /* end switch */
  audioPlayHeader.sample_rate = sampleRate;
  audioPlayHeader.samples_per_unit = 1;
  audioPlayHeader.bytes_per_unit =
    hpAudioInfo.max_bits_per_sample / 8;
  audioPlayHeader.channels = hpAudioInfo.nchannels;
  audioPlayHeader.data_size = sampleRate * 2;
  audioRecordHeader.sample_rate = sampleRate;
  audioRecordHeader.samples_per_unit = 1;
  audioRecordHeader.bytes_per_unit =
    hpAudioInfo.max_bits_per_sample / 8;
  audioRecordHeader.channels = hpAudioInfo.nchannels;
  audioRecordHeader.data_size = sampleRate * 2;
  result = ioctl(audioFD, AUDIO_DESCRIBE, &hpGainInfo);		    /* Try to get info about the audio device again */
  playRange = hpGainInfo.max_transmit_gain - hpGainInfo.min_transmit_gain;
  recordRange = hpGainInfo.max_receive_gain - hpGainInfo.min_receive_gain;
  monitorRange = hpGainInfo.max_monitor_gain - hpGainInfo.min_monitor_gain;
  audioVolumeInfo.cgain[0].receive_gain =		    /* Set record to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_receive_gain;
  audioVolumeInfo.cgain[1].receive_gain =		    /* Set record to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_receive_gain;
  audioVolumeInfo.cgain[0].transmit_gain =		    /* Set playback to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_transmit_gain;
  audioVolumeInfo.cgain[1].transmit_gain =		    /* Set playback to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_transmit_gain;
  audioVolumeInfo.cgain[0].monitor_gain =		    /* Set monitor to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_monitor_gain;
  audioVolumeInfo.cgain[1].monitor_gain =		    /* Set monitor to half volume */
    ((50 * recordRange) / 100) + hpGainInfo.min_monitor_gain;
  audioVolumeInfo.channel_mask = (AUDIO_CHANNEL_LEFT | AUDIO_CHANNEL_RIGHT);
  result = ioctl(audioFD, AUDIO_SET_GAINS, &audioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return(audioFD);
  }
  tempValue = AUDIO_OUT_SPEAKER;
  ioctl(audioFD, AUDIO_SET_OUTPUT, tempValue);
  tempValue = AUDIO_IN_MIKE;
  ioctl(audioFD, AUDIO_SET_INPUT, tempValue);
  if (hpAudioInfo.audio_id == AUDIO_ID_PSB2160)
  {
    PrintDiagnostics("Low quality audio.\n");
  }
  else
  {
    PrintDiagnostics("High quality audio.\n");
    tempValue = 2;
    result = ioctl(audioFD, AUDIO_SET_CHANNELS, tempValue);
    if (result == -1)
    {
      perror( "audio set channels ioctl" );
    }
    tempValue = sampleRate;
    result = ioctl(audioFD, AUDIO_SET_SAMPLE_RATE, tempValue);
    if (result == -1)
    {
      perror( "audio set sample rate ioctl" );
    }
    tempValue = AUDIO_FORMAT_LINEAR16BIT;
    result = ioctl(audioFD, AUDIO_SET_DATA_FORMAT, tempValue);
    if (result == -1)
    {
      perror( "audio set data format" );
    }
    result = ioctl(audioFD, AUDIO_SET_RXBUFSIZE, 65536*2);
    if (result == -1)
    {
      perror( "audio set rx buffer size ioctl" );
    }
    result = ioctl(audioFD, AUDIO_SET_TXBUFSIZE, 65536*2);
    if (result == -1)
    {
      perror( "audio set tx buffer size ioctl" );
    }
  }
  AudioRecordPause(audioFD);
  ioctl(audioFD, AUDIO_DRAIN, AUDIO_RECEIVE);
  return(audioFD);
} /* end function InitializeAudio */



void SetAudioPlayVolume(int		audioDevice,
			int		volume)
{
  int	result;
  
  hpAudioVolumeInfo.cgain[0].transmit_gain =		    /* Scale playback volume to fit accepted HP range */
    ((volume * playRange) / 100) + hpGainInfo.min_transmit_gain;
  hpAudioVolumeInfo.cgain[1].transmit_gain =
    ((volume * playRange) / 100) + hpGainInfo.min_transmit_gain;
  result = ioctl(audioDevice, AUDIO_SET_GAINS, &hpAudioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy and so cannot set playback volume.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
  }
  return;
} /* end function SetAudioPlayVolume */



void SetAudioRecordVolume(int		audioDevice,
			  int		volume)
{
  int	result;
  
  hpAudioVolumeInfo.cgain[0].receive_gain =		    /* Scale playback volume to fit accepted HP range */
    ((volume * recordRange) / 100) + hpGainInfo.min_receive_gain;
  hpAudioVolumeInfo.cgain[1].receive_gain =
    ((volume * recordRange) / 100) + hpGainInfo.min_receive_gain;
  result = ioctl(audioDevice, AUDIO_SET_GAINS, &hpAudioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy and so cannot set record volume.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
  }
} /* end function SetAudioRecordVolume */




void SetAudioMonitorVolume(int		audioDevice,
			   int		volume)
{
  int	result;
  
  hpAudioVolumeInfo.cgain[0].monitor_gain =		    /* Scale playback volume to fit accepted HP range */
    ((volume * monitorRange) / 100) + hpGainInfo.min_monitor_gain;
  hpAudioVolumeInfo.cgain[1].monitor_gain =
    ((volume * monitorRange) / 100) + hpGainInfo.min_monitor_gain;
  result = ioctl(audioDevice, AUDIO_SET_GAINS, &hpAudioVolumeInfo);
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy and so cannot set monitor volume.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
  }
} /* end function SetAudioMonitorVolume */



void SetPlaybackDevice(int deviceFD, int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioSpeaker)
  {
    PrintDiagnostics("SetPlaybackDevice(): Speaker set.\n");
    localDeviceType = AUDIO_OUT_SPEAKER;
  }
  if (deviceType == AudioLineOut)
  {
    PrintDiagnostics("SetPlaybackDevice(): Line Out set.\n");
    localDeviceType = AUDIO_OUT_LINE;
  }
  if (deviceType == AudioHeadphone)
  {
    PrintDiagnostics("SetPlaybackDevice(): Headphone set.\n");
    localDeviceType = AUDIO_OUT_HEADPHONE;
  }
  ioctl(deviceFD, AUDIO_SET_OUTPUT, localDeviceType);
  return;
} /* end function SetPlaybackDevice */



void SetRecordDevice(int deviceFD, int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioMicrophone)
    localDeviceType = AUDIO_IN_MIKE;
  else
    localDeviceType = AUDIO_IN_LINE;
  ioctl(audioFD, AUDIO_SET_INPUT, localDeviceType);
} /* end function SetPlaybackDevice */



int SetAudioPlaybackConfiguration(int		audioDeviceFD,
				  Audio_hdr*	audioInfo)
{
  int	audioResult;
  int	tempValue;
  int	result;
  
  PrintDiagnostics("Entered SetAudioPlaybackConfiguration().\n");
  result = ioctl(audioFD, AUDIO_SET_CHANNELS, audioInfo->channels);
  if (result == -1)
  {
    perror( "audio set channels ioctl" );
  }
  result = ioctl(audioFD, AUDIO_SET_SAMPLE_RATE, audioInfo->sample_rate);
  if (result == -1)
  {
    perror( "audio set sample rate ioctl" );
  }
  switch (audioInfo->encoding)					    /* Translate the PortableAudio encoding constant into the... */
  {								    /* ...appropriate HP-specific constant. */
   case AudioULawEncoding:
    tempValue = AUDIO_FORMAT_ULAW;
    break;
   case AudioALawEncoding:
    tempValue = AUDIO_FORMAT_ALAW;
    break;
   case AudioLinearEncoding:
    tempValue = AUDIO_FORMAT_LINEAR16BIT;
    break;
  }
  result = ioctl(audioFD, AUDIO_SET_DATA_FORMAT, tempValue);
  if (result == -1)
  {
    perror("audio set data format");
  }
  return(0);							    /* Yes, return zero */
} /* end function SetAudioPlaybackConfiguration */



void AudioClose(int audioFD)
{
  ioctl(audioFD, AUDIO_RESET, (RESET_RX_BUF | RESET_TX_BUF));	    /* Clear out all audio in the audio device */
  close(audioFD);						    /* Close the audio device properly */
  return;
} /* end function AudioClose */



void SyncAudioPlaybackDevice(int audioFD)
{
  struct timeval	after;
  struct timeval	before;
  int			bytesWritten;
  int			counter = 0;
  int			elapsedTime = 0;
  static char*		syncBuffer = NULL;
  
  return;
  if (syncBuffer == NULL)					    /* Allocate some memory for holding audio to be pushed... */
  {								    /* ...through the audio device. */
    syncBuffer = malloc(48000);					    /* I'm guessing that 48K is the max. sample rate and will... */
    if (syncBuffer == NULL)					    /* ...be an upper-bound on sound size used. */
    {
      sprintf(errorMessage,
	      "Could not allocate enough audio memory to properly sync audio.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    memset(syncBuffer, '\0', 48000);				    /* Fill the audio buffer with zeros */
  }
  while (elapsedTime < (1000 / playbackSpeed))			    /* Keep pushing audio until time to play 1 frame is close... */
  {								    /* ...to real-time */
    if (printDiagnostics == True)
    {
      printf("Playing %d bytes of audio (iteration %d).\n",
	     movieHeader.audioslice, counter++);
    }
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(audioFD, syncBuffer,			    /* Playback 1 frame's worth of audio */
			 movieHeader.audioslice);
    gettimeofday(&after, (struct timezone*)NULL);
    if ((bytesWritten < 0) && (printDiagnostics == True))
    {
      printf("SyncAudioPlaybackDevice(): Error %d (errno is %d) writing audio.\n",
	     bytesWritten, errno);
    }
    elapsedTime = ElapsedTime(&before, &after);			    /* Count time in milliseconds to play 1 frame of audio */
  }
  return;
} /* end function SyncAudioPlaybackDevice */
