/* $Header: /plx/S/source/Xclients/solaris2.4/3.4a/src/client/motif/MovieTool/RCS/sunAudio.c,v 1.46.1.1 1995/04/05 20:29:53 lovvik Exp $ */
/* $Log: sunAudio.c,v $
 * Revision 1.46.1.1  1995/04/05  20:29:53  lovvik
 * 2 changes:
 *   1)  Modified InitializeAudio() to do non-blocking reads and
 *       writes to the audio device.
 *
 *   2)  Added a function called FlushAudioBuffer().
 *
 * Revision 1.46  1995/02/13  19:44:29  drapeau
 * Made more changes to support SPARC-5 audio, especially for Solaris
 * 2.X.
 *
 * Revision 1.45  1995/02/07  22:47:33  lovvik
 * Added support for Sparc 5 Audio.  This was a simple change to
 * PrintAudioInfo() and InitializeAudio().
 *
 * Revision 1.44  1994/09/27  20:56:27  drapeau
 * Added a function, SyncAudioRecordDevice(), which for the Sun platform
 * does nothing at present.
 *
 * Revision 1.43  1994/09/14  03:23:19  drapeau
 * Enabled audio output on line out and headphones as well as speakers.
 *
 * Revision 1.42  1994/09/13  08:10:27  drapeau
 * Check for internal high-quality audio as well as speakerbox
 * high-quality audio.
 *
 * Revision 1.41  1994/08/24  01:53:56  drapeau
 * Slight modification to PrintAudioInfo(), to get information about the
 * current monitor volume.
 *
 * Revision 1.40  1994/08/23  05:07:35  drapeau
 * Made changes to the record side of the device controller, trying to
 * obey SPARC audio device rules better and therefore eliminate popping
 * noises during initial recording.
 *
 * Revision 1.39  1994/08/23  01:35:08  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.38  1994/08/19  06:05:28  drapeau
 * Removed extraneous printf stmts, and made a small modification to
 * account for Sun audio on Solaris 2.3 (which includes an extra data
 * field for byte ordering).
 *
 * Revision 1.37  1994/08/08  01:04:05  drapeau
 * Two changes:
 *
 * 1) Added a function, SyncAudioPlaybackDevice(), to test
 *    synchronization b/w audio and video.  So far, it is inoperative.
 *
 * 2) Added proper support for high-quality audio on the Solaris 2.X
 *    platform.
 *
 * Revision 1.36  1994/06/27  22:57:27  drapeau
 * Removed an extraneous diagnostic message that was used for debugging.
 *
 * Revision 1.35  1994/06/24  01:16:47  drapeau
 * Added the function AudioClose() to properly close the audio device.
 *
 * Revision 1.34  1994/01/20  22:50:10  drapeau
 * Made implementation more portable, in support of the PortableAudio
 * interface.  Also had to modify the PortableAudio interface to support
 * what I consider to be flaws in the Sun audio device driver: it appears
 * that you cannot have a single file descriptor to be used for both
 * recording and playback of audio.
 *
 * Revision 1.33  1994/01/20  18:02:00  drapeau
 * Several major changes, to support audio under both SunOS 4.X and
 * Solaris 2.X, and to support the new PortableAudio structure in order
 * that MovieTool is more portable across platforms.
 *
 * Revision 1.32  1994/01/10  22:34:31  drapeau
 * Added support for the Sun speakerbox (high-quality audio).
 * Also, cleaned up diagnostics reporting.
 *
 * Revision 1.31  1993/12/03  02:09:52  drapeau
 * Minor change made; mostly it was done to allow me to change the RCS
 * numbering scheme for this file.
 *
 * Revision 1.3  1993/12/03  01:31:44  drapeau
 * Added functions to control audio volume;  also made the code more
 * robust in general.
 *
 * Revision 1.2  1993/11/01  17:08:07  drapeau
 * No functional changes; just using a newer RCS version.
 *
 * Revision 1.1  93/09/09  16:09:56  drapeau
 * Initial revision
 *  */
static char	audioFunctionsRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4a/src/client/motif/MovieTool/RCS/sunAudio.c,v 1.46.1.1 1995/04/05 20:29:53 lovvik Exp $";

#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <stropts.h>
#include "PortableAudio.h"
#include "MovieTool.h"



int AudioRecordOpen(int audioFD)
{
  int			result;
  Audio_info		audioInfo;
#ifdef SVR4
  audio_device_t	deviceType;
#else  
  int			deviceType;
#endif
  
  result = ioctl(audioFD, AUDIO_GETDEV, &deviceType);		    /* Determine which type of audio device is connected */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return an error code */
    sprintf(errorMessage,
	    "Could not get information about the audio device for recording.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
#ifdef SVR4
  if (strcmp(deviceType.name, "SUNW,am79c30") == 0)		    /* Solaris 2.X way of telling me it's low-quality audio */
#else
  if (deviceType == AUDIO_DEV_AMD)				    /* SunOS 4.X way of telling me it's low-quality audio */
#endif
    audioInfo.record.port = AUDIO_MICROPHONE;			    /* Yes, set up for low-quality audio */
  else								    /* No, use the highest available quality audio */
    audioInfo.record.port = AUDIO_LINE_IN;
  result = ioctl(audioFD, AUDIO_GETINFO, &audioInfo);		    /* Try to determine current state of the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    sprintf(errorMessage,
	    "AudioRecordOpen could not determine state of the the audio device for recording.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
  result = ioctl(audioFD, AUDIO_SETINFO, &audioInfo);		    /* Try to reconfigure audio device to the app's preferences */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    sprintf(errorMessage, "AudioRecordOpen: could not reconfigure the audio device for recording.\n");
    PrintErrorMessage(errorMessage);
    return(-1);
  }
  return(audioFD);						    /* Return a valid file descriptor for the audio device */
} /* end function AudioRecordOpen */





int AudioRecordPause(int audioFD)
{
  audio_pause_record(audioFD);					    /* Stop recording data now */
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
  return(audioRecordFD);					    /* Point to the record file descriptor */
} /* end function AudioRecordPause */



int AudioRecordResume(int audioFD)
{
  audio_resume_record(audioFD);
  return(audioRecordFD);					    /* Point to the record file descriptor */
} /* end function AudioRecordResume */



int AudioPlaybackOpen(int audioFD)
{
  int		soundFileFD = 0;
  int		result;
  int		audioDeviceType;
  
  return(audioFD);
} /* end function AudioPlaybackOpen */



int AudioPlayPause(int audioFD)
{
  audio_pause_play(audioFD);
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
  return(audioPlayFD);						    /* Point to the playback file descriptor */
} /* end function AudioPlayPause */



int AudioPlayResume(int audioFD)
{
  audio_resume_play(audioFD);
  return(audioPlayFD);						    /* Point to the playback file descriptor */
} /* end function AudioPlayResume */



void PrintAudioInfo(int			audioInfoFD,
		    PortableAudio*	audioInfo)
{
  int			result;
  Audio_info		tempAudioInfo;
#ifdef SVR4
  audio_device_t	deviceType;
#else  
  int			deviceType;
#endif
  
  result = ioctl(audioInfoFD, AUDIO_GETINFO, &tempAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  result = ioctl(audioInfoFD, AUDIO_GETDEV, &deviceType);
  audioInfo->play.sampleRate = tempAudioInfo.play.sample_rate;	    /* Fill in the PortableAudio data structure containing... */
  audioInfo->play.channels = tempAudioInfo.play.channels;	    /* ...device-specific information about this audio device. */
  audioInfo->play.bitsPerSample = tempAudioInfo.play.precision;
  audioInfo->play.volume = tempAudioInfo.play.gain;
  switch (tempAudioInfo.play.encoding)
  {
   case AUDIO_ENCODING_ULAW:
    audioInfo->play.encoding = AudioULawEncoding;
    PrintDiagnostics("Playback Encoding: uLaw.\n");
    break;
   case AUDIO_ENCODING_ALAW:
    audioInfo->play.encoding = AudioALawEncoding;
    PrintDiagnostics("Playback Encoding: aLaw.\n");
    break;
   case AUDIO_ENCODING_LINEAR:
    PrintDiagnostics("Playback Encoding: Linear.\n");
    audioInfo->play.encoding = AudioLinearEncoding;
    break;
  }
  audioInfo->record.sampleRate = tempAudioInfo.record.sample_rate;
  audioInfo->record.channels = tempAudioInfo.record.channels;
  audioInfo->record.bitsPerSample = tempAudioInfo.record.precision;
  audioInfo->record.volume = tempAudioInfo.record.gain;
  audioInfo->monitorVolume = tempAudioInfo.monitor_gain;
  switch (tempAudioInfo.record.encoding)
  {
   case AUDIO_ENCODING_ULAW:
    audioInfo->record.encoding = AudioULawEncoding;
    PrintDiagnostics("Record Encoding: uLaw.\n");
    break;
   case AUDIO_ENCODING_ALAW:
    audioInfo->record.encoding = AudioALawEncoding;
    PrintDiagnostics("Record Encoding: aLaw.\n");
    break;
   case AUDIO_ENCODING_LINEAR:
    audioInfo->record.encoding = AudioLinearEncoding;
    PrintDiagnostics("Record Encoding: Linear.\n");
    break;
  }
  if (printDiagnostics == True)					    /* Should diagnostic information actually be printed? */
  {								    /* Yes, tell the author about the device characteristics */
    printf("Audio information:\n");
    printf("Sample rate: %d.\n", audioInfo->play.sampleRate);
    printf("Number of channels: %d.\n", audioInfo->play.channels);
    printf("Bits per sample: %d.\n", audioInfo->play.bitsPerSample);
    printf("Playback volume: %d.\n", audioInfo->play.volume);
    printf("Record volume: %d.\n", audioInfo->record.volume);
    printf("Monitor volume: %d.\n", audioInfo->monitorVolume);
#ifdef SVR4
    printf("Solaris audio: device name is %s, version is %s, config is %s.\n",
	   deviceType.name, deviceType.version, deviceType.config);
    if ((strcmp(deviceType.name, "SUNW,dbri") == 0) ||		    /* Solaris 2.X way of telling me it's high-quality audio */
	(strcmp(deviceType.name, "SUNW,CS4231") == 0))
#else
    if ((deviceType == AUDIO_DEV_SPEAKERBOX) ||			    /* SunOS 4.X way of telling me it's high-quality audio */
	(deviceType == 5) ||                                        /* Sparc 5 support */
	(deviceType == AUDIO_DEV_CODEC))
#endif
      printf("Device type: High quality audio.\n");
    else
      printf("Device type: Low quality audio.\n");
  } /* end if printDiagnostics... */
  return;
} /* end function PrintAudioInfo */



int InitializeAudio(void)
{
  Audio_info		tempAudioInfo;
  int			result;
#ifdef SVR4
  audio_device_t	deviceType;
#else  
  int			deviceType;
#endif
  
  audioRecordFD = open("/dev/audio", (O_RDWR | O_NONBLOCK));        /* Try to open the audio control device for record & play */
  if (audioRecordFD == -1)					    /* Did the open fail? */
  {								    /* Yes, report the error and exit immediately */
    audioPlayFD = audioRecordFD;				    /* Point all audio file descriptors to the bad one */
    audioFD = audioRecordFD;
    sprintf(errorMessage, "Sorry, could not open the audio device for record and play.\n");
    PrintErrorMessage(errorMessage);
    return(audioRecordFD);					    /* Return the bad file descriptor */
  }
  audioPlayFD = audioRecordFD;					    /* Point all audio file descriptors to the one just opened */
  audioFD = audioRecordFD;
  result = ioctl(audioFD, AUDIO_GETDEV, &deviceType);
#ifdef SVR4
  if ((strcmp(deviceType.name, "SUNW,dbri") == 0) ||		    /* Solaris 2.X way of telling me it's high-quality audio */
      (strcmp(deviceType.name, "SUNW,CS4231") == 0))
#else
  if ((deviceType == AUDIO_DEV_SPEAKERBOX) ||			    /* SunOS 4.X way of telling me it's high-quality audio */
      (deviceType == AUDIO_DEV_CODEC) ||
      (deviceType == 5))
#endif
  {
    audioPlayHeader.sample_rate = sampleRate;
    audioPlayHeader.samples_per_unit = 1;
    audioPlayHeader.bytes_per_unit = 2;
    audioPlayHeader.channels = numAudioChannels;
    audioPlayHeader.encoding = AUDIO_ENCODING_LINEAR;
    audioPlayHeader.data_size = sampleRate * 2;
    result = audio_set_play_config(audioFD, &audioPlayHeader);
    audioRecordHeader.sample_rate = sampleRate;
    audioRecordHeader.samples_per_unit = 1;
    audioRecordHeader.bytes_per_unit = 2;
    audioRecordHeader.channels = numAudioChannels;
    audioRecordHeader.encoding = AUDIO_ENCODING_LINEAR;
    audioRecordHeader.data_size = sampleRate * 2;
    AudioRecordPause(audioFD);					    /* Clear out the record buffer and pause audio recording... */
    result = audio_set_record_config(audioFD, &audioRecordHeader);  /* ...before changing record parameters. */
  }
#ifdef SVR4
  if (strcmp(deviceType.name, "SUNW,am79c30") == 0)		    /* Solaris 2.X way of telling me it's low-quality audio */
#else
  if (deviceType == AUDIO_DEV_AMD)				    /* SunOS 4.X way of telling me it's low-quality audio */
#endif
  {
    audioPlayHeader.sample_rate = 8000;
    audioPlayHeader.samples_per_unit = 1;
    audioPlayHeader.bytes_per_unit = 1;
    audioPlayHeader.channels = 1;
    audioPlayHeader.encoding = AUDIO_ENCODING_ULAW;
    audioPlayHeader.data_size = 8000;
    result = audio_set_play_config(audioFD, &audioPlayHeader);
    audioRecordHeader.sample_rate = 8000;
    audioRecordHeader.samples_per_unit = 1;
    audioRecordHeader.bytes_per_unit = 1;
    audioRecordHeader.channels = 1;
    audioRecordHeader.encoding = AUDIO_ENCODING_ULAW;
    audioRecordHeader.data_size = 8000;
    result = audio_set_record_config(audioFD, &audioRecordHeader);
  }
  result = ioctl(audioFD, AUDIO_GETINFO, &tempAudioInfo);	    /* Try to get info about the audio device */
  if (result == -1)						    /* Did the query fail? */
  {								    /* Yes, report the error and return a bad file descriptor */
    switch (errno)
    {
     case EBUSY:
      sprintf(errorMessage, "The audio device is busy.  Check if other applications are currently using the audio device.\n");
      PrintErrorMessage(errorMessage);
      break;
    }
    return;
  }
  audioInfo.play.sampleRate =	 	tempAudioInfo.play.sample_rate;	/* Fill in PortableAudio data structure containing... */
  audioInfo.play.channels = 		tempAudioInfo.play.channels; /* ...device-specific information about this audio device. */
  audioInfo.play.bitsPerSample =	tempAudioInfo.play.precision;
  audioInfo.play.encoding = 		tempAudioInfo.play.encoding;
  audioInfo.play.volume = 		tempAudioInfo.play.gain;
  audioInfo.record.sampleRate = 	tempAudioInfo.record.sample_rate;
  audioInfo.record.channels = 		tempAudioInfo.record.channels;
  audioInfo.record.bitsPerSample = 	tempAudioInfo.record.precision;
  audioInfo.record.encoding = 		tempAudioInfo.record.encoding;
  audioInfo.record.volume = 		tempAudioInfo.record.gain;
  PrintAudioInfo(audioFD, &audioInfo);				    /* Fill in information about the audio device */
  XtVaSetValues(playbackScale,
		XmNvalue,
		(audioInfo.play.volume * 100) / AUDIO_MAX_GAIN,
		NULL);
  XtVaSetValues(recordScale,
		XmNvalue,
		(audioInfo.record.volume * 100) / AUDIO_MAX_GAIN,
		NULL);
  XtVaSetValues(monitorScale,
		XmNvalue,
		(audioInfo.monitorVolume * 100) / AUDIO_MAX_GAIN,
		NULL);
  if ((tempAudioInfo.play.port && AUDIO_SPEAKER) != 0)		    /* Is the speaker enabled? */
  {								    /* Yes, set the correct toggles */
    XmToggleButtonSetState(playbackSpeakerToggle, True, False);
    XmToggleButtonSetState(playbackHeadphonesToggle, False, False);
  }
  else								    /* No, set the toggles to indicate that the headphone... */
  {								    /* ...jack is enabled. */
    XmToggleButtonSetState(playbackSpeakerToggle, False, False);
    XmToggleButtonSetState(playbackHeadphonesToggle, True, False);
  }
  SetRecordDevice(audioFD, AudioMicrophone);
  return(audioFD);
} /* end function InitializeAudio */



void SetAudioPlayVolume(int	audioDevice,
			int	volume)
{
  double	deviceVolume;
  
  deviceVolume = volume / 100.0;
  audio_set_play_gain(audioDevice, &deviceVolume);
  return;
} /* end function SetAudioPlayVolume */



void SetAudioRecordVolume(int	audioDevice,
			  int	volume)
{
  double	deviceVolume;
  
  deviceVolume = volume / 100.0;
  audio_set_record_gain(audioDevice, &deviceVolume);
  return;
} /* end function SetAudioRecordVolume */



void SetAudioMonitorVolume(int	audioDevice,
			   int	volume)
{
  double	deviceVolume;
  
  deviceVolume = volume / 100.0;
  audio_set_monitor_gain(audioDevice, &deviceVolume);
  return;
} /* end function SetAudioMonitorVolume */



void SetPlaybackDevice(int deviceFD,
		       int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioSpeaker)
    localDeviceType = AUDIO_SPEAKER;
  if (deviceType == AudioLineOut)
    localDeviceType = AUDIO_LINE_OUT;
  if (deviceType == AudioHeadphone)
    localDeviceType = AUDIO_HEADPHONE;
  audio_set_play_port(deviceFD, &localDeviceType);
} /* end function SetPlaybackDevice */




void SetRecordDevice(int deviceFD,
		     int deviceType)
{
  int	localDeviceType;
  
  if (deviceType == AudioMicrophone)
    localDeviceType = AUDIO_MICROPHONE;
  else
    localDeviceType = AUDIO_LINE_IN;
  AudioRecordPause(deviceFD);					    /* Pause the record device & clear out old data before... */
  audio_set_record_port(deviceFD, &localDeviceType);		    /* ...changing the audio device parameters. */
} /* end function SetRecordDevice */



int SetAudioPlaybackConfiguration(int		audioDeviceFD,
				  Audio_hdr*	audioInfo)
{
  int		audioResult;
  Audio_hdr	tempInfo;
  
  tempInfo.sample_rate = audioInfo->sample_rate;
  tempInfo.samples_per_unit = audioInfo->samples_per_unit;
  tempInfo.bytes_per_unit = audioInfo->bytes_per_unit;
  tempInfo.channels = audioInfo->channels;
  tempInfo.data_size = audioInfo->data_size;
#ifdef SVR4
  tempInfo.endian = audioInfo->endian;
#endif  
  switch (audioInfo->encoding)					    /* Translate the PortableAudio encoding constant into the... */
  {								    /* ...appropriate Sun-specific constant. */
   case AudioULawEncoding:
    tempInfo.encoding = AUDIO_ENCODING_ULAW;
    break;
   case AudioALawEncoding:
    tempInfo.encoding = AUDIO_ENCODING_ALAW;
    break;
   case AudioLinearEncoding:
    tempInfo.encoding = AUDIO_ENCODING_LINEAR;
    break;
  }
  audioResult = audio_set_play_config(audioDeviceFD,		    /* Try to correctly configure the audio device */
				      &tempInfo);
  if (audioResult == AUDIO_SUCCESS)				    /* Did the attempt succeed? */
    return(0);							    /* Yes, return zero */
  else								    /* Nope, return an error code */
    return(-1);
} /* end function SetAudioPlaybackConfiguration */



void AudioClose(int audioFD)
{
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
  close(audioFD);						    /* Close the audio device properly */
  return;
} /* end function AudioClose */



void SyncAudioPlaybackDevice(int audioFD)
{
  struct timeval	after;
  struct timeval	before;
  int			bytesWritten;
  int			counter = 0;
  int			elapsedTime = 0;
  static char*		syncBuffer = NULL;
  
  return;
  if (syncBuffer == NULL)					    /* Allocate some memory for holding audio to be pushed... */
  {								    /* ...through the audio device. */
    syncBuffer = malloc(48000);					    /* I'm guessing that 48K is the max. sample rate and will... */
    if (syncBuffer == NULL)					    /* ...be an upper-bound on sound size used. */
    {
      sprintf(errorMessage, "Could not allocate enough audio memory to properly sync audio.\n");
      PrintErrorMessage(errorMessage);
      return;
    }
    memset(syncBuffer, '\0', 48000);				    /* Fill the audio buffer with zeros */
  }
  while (elapsedTime < (1000 / playbackSpeed))			    /* Keep pushing audio until time to play 1 frame is close... */
  {								    /* ...to real-time */
    if (printDiagnostics == True)
    {
      printf("Playing %d bytes of audio (iteration %d).\n",
	     movieHeader.audioslice, counter++);
    }
    gettimeofday(&before, (struct timezone*)NULL);
    bytesWritten = write(audioFD, syncBuffer,			    /* Playback 1 frame's worth of audio */
			 movieHeader.audioslice);
    gettimeofday(&after, (struct timezone*)NULL);
    if ((bytesWritten < 0) && (printDiagnostics == True))
    {
      printf("SyncAudioPlaybackDevice(): Error %d (errno is %d) writing audio.\n",
	     bytesWritten, errno);
    }
    elapsedTime = ElapsedTime(&before, &after);			    /* Count time in milliseconds to play 1 frame of audio */
  }
  return;
} /* end function SyncAudioPlaybackDevice */



void SyncAudioRecordDevice(int audioFD)
{
  return;							    /* Currently no need to sync the Sun audio device for record */
} /* end function SyncAudioRecordDevice */


void FlushAudioBuffer(int audioFD)
{
  ioctl(audioFD, I_FLUSH, FLUSHRW);				    /* Clear out all audio data */
}
