/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/utils.c,v 1.26 1994/09/13 08:11:45 drapeau Exp $ */
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Log: utils.c,v $
 * Revision 1.26  1994/09/13  08:11:45  drapeau
 * Fixed potential dangling memory pointer problem (declared character
 * space as static to keep it around).  Also dimmed the "Compact..." menu
 * item when the movie is unmodified.
 *
 * Revision 1.25  1994/08/23  01:35:42  drapeau
 * Added code for an alert dialog box.
 *
 * Revision 1.24  1994/05/31  20:14:40  drapeau
 * Fixed tools for converting and displaying timecode for a movie.
 *
 * Revision 1.23  1994/01/10  22:41:44  drapeau
 * Added functionality to properly display movie status, timecode, etc.
 * Also, fixed minor errors in the translation between timecode and frame
 * number values.
 *
 * Revision 1.22  1993/12/04  01:02:54  drapeau
 * Added the function UpdateTotalTimePaused() to make the pause feature
 * more robust.  Before this version, pausing a movie during playback
 * would throw off the synchronization mechanism; pause now works
 * correctly.
 *
 * Revision 1.21  1993/12/03  02:12:55  drapeau
 * Minor change in RCS info; mostly it was done to allow me to change the
 * RCS numbering scheme for this file.
 *
 * Revision 1.2  1993/12/03  01:37:58  drapeau
 * Added a function to update the timecode display on the main panel;
 * also added a prototype function to disable certain controls during
 * playback and record.
 * */
static char utilsRcsid[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/utils.c,v 1.26 1994/09/13 08:11:45 drapeau Exp $";

#include "MovieTool.h"

/* Convert time to address in frames */
int
ConvertToAddress(int hr, int min, int sec, int frame)
{
  int address;
  
  address = signalFrameRate * (hr * 3600 + min * 60 + sec) + frame;
  return(address);
}								    /* end function ConvertToAddress */



void ConvertToTime(int* hr,					    /* Convert address in frames to time */
		   int* min,
		   int* sec,
		   int*	frame,
		   int	frameNumber,
		   int	framesPerSecond)
{
  int	framesPerHour;
  int	framesPerMinute;
  
  framesPerHour = 3600 * framesPerSecond;			    /* 3600 = seconds per hour */
  framesPerMinute = 60 * framesPerSecond;
  
  *hr = frameNumber / framesPerHour;
  *min = (frameNumber % framesPerHour) / framesPerMinute;
  *sec = ((frameNumber % framesPerHour) % framesPerMinute) / framesPerSecond;
  *frame = ((frameNumber % framesPerHour) % framesPerMinute) % framesPerSecond;
}								    /* end function ConvertToTime */



void UpdateTimecodeDisplay(int	frameRate)
{
  int		hour;
  int		minute;
  int		second;
  int		frame;
  static int	lastHour = 0;
  static int	lastMinute = 0;
  static int	lastSecond = 0;
  static int	lastFrame = 0;
  
  ConvertToTime(&hour, &minute, &second, &frame,		    /* Determine the correct timecode value based on the... */
		currentFrame - firstFrame, frameRate);		    /* ...current frame and the frame rate. */
  if (hour != lastHour)						    /* Did the hour value change?  Avoid unnecessary drawing... */
  {								    /* ...and updating of the text field */
    lastHour = hour;
    sprintf(timecodeString, "%02d", hour);
    XmTextSetString(hoursTextField, timecodeString);
  }
  if (minute != lastMinute)					    /* Did the minute value change?  Avoid unnecessary... */
  {								    /* ... drawing and updating of the text field */
    lastMinute = minute;
    sprintf(timecodeString, "%02d", minute);
    XmTextSetString(minutesTextField, timecodeString);
  }
  if (second != lastSecond)					    /* Did the second value change?  Avoid unnecessary... */
  {								    /* ...drawing and updating of the text field */
    lastSecond = second;
    sprintf(timecodeString, "%02d", second);
    XmTextSetString(secondsTextField, timecodeString);
  }
  if (frame != lastFrame)					    /* Did the frame value change?  Avoid unnecessary drawing... */
  {								    /* ...and updating of the text field */
    lastFrame = frame;
    sprintf(timecodeString, "%02d", frame);
    XmTextSetString(framesTextField, timecodeString);
  }
  if (movieToolStatus != Recording)				    /* If MovieTool is recording, no use updating the slider */
    XtVaSetValues(searchSlider,					    /* Else, update the search slider to show the current... */
		  XmNvalue, (currentFrame - firstFrame),	    /* ...position within the movie. */
		  NULL);
  return;
} /* end function UpdateTimecodeDisplay */



void DeSensitizePlaybackControls(void)
{
  return;							    /* Don't do anything right now; just return */
  XtVaSetValues(hourPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minutePlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(framePlusButton, XmNsensitive, False, NULL);
  
  XtVaSetValues(hourMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minuteMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameMinusButton, XmNsensitive, False, NULL);
  
  XtVaSetValues(searchSlider, XmNsensitive, False, NULL);
  XtVaSetValues(frameAdvanceButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameReverseButton, XmNsensitive, False, NULL);
  XtVaSetValues(endOfMovieButton, XmNsensitive, False, NULL);
  XtVaSetValues(startOfMovieButton, XmNsensitive, False, NULL);
  return;
} /* end function DeSensitizePlaybackControls */



void UpdateTotalTimePaused(Boolean startPausing)
{
  static struct timeval		pauseStartTime;
  struct timeval		pauseStopTime;
  
  if (startPausing == True)
  {
    gettimeofday(&pauseStartTime, (struct timezone*)NULL);	    /* Record the time that the pause was turned on*/
  }
  else
  {
    gettimeofday(&pauseStopTime, (struct timezone*)NULL);	    /* Record the time that the pause was turned off */
    totalTimePaused += ElapsedTime(&pauseStartTime, &pauseStopTime); /* Calculate the total time paused and update the variable */
  }
  return;
} /* end function UpdateTotalTimePaused */



void UpdateTitleBar()
{
  static char	titleBarString[MAXPATHLEN + 64];
  
  if (movieStatus == MovieModified)
  {
    sprintf(titleBarString, "MovieTool: %s (modified)", currentMovieFileName);
  }
  if (movieStatus == MovieNew)
  {
    sprintf(titleBarString, "MovieTool: %s (new)", currentMovieFileName);
  }
  if (movieStatus == MovieOK)
  {
    sprintf(titleBarString, "MovieTool: %s", currentMovieFileName);
  }
  XtVaSetValues(mainWindow, XmNtitle, titleBarString, NULL);	    /* Reset the title string for MovieTool's main window */
  return;
} /* end function UpdateTitleBar */



void SetMovieStatus(int	status)
{
  movieStatus = status;
  if (movieStatus == MovieModified)				    /* Should the "Save" button be enabled? */
  {
    XtVaSetValues(saveMenuButton, XmNsensitive, True, NULL);	    /* Yes, this movie needs to be saved */
    XtVaSetValues(compactMovieButton, XmNsensitive, True, NULL);
  }
  else
  {
    XtVaSetValues(saveMenuButton, XmNsensitive, False, NULL);
    XtVaSetValues(compactMovieButton, XmNsensitive, False, NULL);
  }
  UpdateTitleBar();
  return;
} /* end function SetMovieStatus */



void UpdateMovieLengthIndicators(void)
{
  int					hour;
  int					minute;
  int					second;
  int					frame;
  char					tempString[32];
  XmString				labelString;
  
  XtVaSetValues(searchSlider,					    /* Update the search slider to constrain itself to the... */
		XmNminimum, 0,					    /* ...number of frames in this movie. */
		XmNmaximum, lastFrame - firstFrame,
		NULL);
  ConvertToTime(&hour, &minute, &second, &frame,		    /* Determine duration of this movie in timecode terms */
		lastFrame - firstFrame,
		movieHeader.fps);
  sprintf(tempString, "%02d:%02d:%02d:%02d", hour, minute, second, frame);
  labelString = XmStringCreateSimple(tempString);		    /* Create a string to put on the end-of-movie timecode label */
  XtVaSetValues(endTimeLabel, XmNlabelString, labelString, NULL);   /* Update the label showing the end timecode of this movie */
  XmStringFree(labelString);
  return;
} /* end function UpdateMovieLengthIndicators */



void InitializeInterface(void)
{
  XmScaleCallbackStruct	callData;
  
  XtVaSetValues(anyWindowMenuButton, XmNsensitive, False, NULL);
  XtVaSetValues(practiceRunButton, XmNsensitive, False, NULL);
  XtVaSetValues(hourMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minuteMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameMinusButton, XmNsensitive, False, NULL);
  XtVaSetValues(hourPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(minutePlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(secondPlusButton, XmNsensitive, False, NULL);
  XtVaSetValues(framePlusButton, XmNsensitive, False, NULL);
  
  XtVaSetValues(searchSlider, XmNsensitive, False, NULL);
  XtVaSetValues(speedSlider, XmNsensitive, False, NULL);
  XtVaSetValues(playButton, XmNsensitive, False, NULL);
  XtVaSetValues(pauseButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameAdvanceButton, XmNsensitive, False, NULL);
  XtVaSetValues(frameReverseButton, XmNsensitive, False, NULL);
  XtVaSetValues(endOfMovieButton, XmNsensitive, False, NULL);
  XtVaSetValues(startOfMovieButton, XmNsensitive, False, NULL);
  callData.value = recordFrameRate;
  ChangeRecordSpeed((Widget)NULL,				    /* Make sure the recording speed is okay for the audio... */
		    (XtPointer)NULL,				    /* record rate being used. */
		    (XtPointer)&callData);
  return;
} /* end function InitializeInterface */



void EnableTransportButtons(void)
{
  XtVaSetValues(hourMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(minuteMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(secondMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(frameMinusButton, XmNsensitive, True, NULL);
  XtVaSetValues(hourPlusButton, XmNsensitive, True, NULL);
  XtVaSetValues(minutePlusButton, XmNsensitive, True, NULL);
  XtVaSetValues(secondPlusButton, XmNsensitive, True, NULL);
  XtVaSetValues(framePlusButton, XmNsensitive, True, NULL);
  
  XtVaSetValues(searchSlider, XmNsensitive, True, NULL);
  XtVaSetValues(speedSlider, XmNsensitive, True, NULL);
  XtVaSetValues(playButton, XmNsensitive, True, NULL);
  XtVaSetValues(pauseButton, XmNsensitive, True, NULL);
  XtVaSetValues(frameAdvanceButton, XmNsensitive, True, NULL);
  XtVaSetValues(frameReverseButton, XmNsensitive, True, NULL);
  XtVaSetValues(endOfMovieButton, XmNsensitive, True, NULL);
  XtVaSetValues(startOfMovieButton, XmNsensitive, True, NULL);
  return;
} /* end function EnableTransportButtons */



void PrintErrorMessage(char*	message)
{
  XmString		messageString;
  
  messageString = XmStringCreateLocalized(message);
  XtVaSetValues(alertLabel, XmNlabelString, messageString, NULL);
  XmStringFree(messageString);
  if (XtIsManaged(alertDialog) == False)
  {
    XtManageChild(alertDialog);
  }
} /* end function PrintErrorMessage */
