/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/VideoTool.c,v 1.24 1994/09/14 06:02:02 drapeau Exp $ */
/* $Log: VideoTool.c,v $
 * Revision 1.24  1994/09/14  06:02:02  drapeau
 * Significant changes to the implementation of the rubberband code.  It
 * works cleanly now.
 *
 * Revision 1.23  1994/09/13  08:03:24  drapeau
 * Minor change, in width of rubberband line to be drawn when changing
 * the video output rectangle.
 *
 * Revision 1.22  1994/09/04  07:12:09  drapeau
 * Two changes:
 *
 * 1) Video input shells now respond correctly when the window manager
 *    tries to delete them; they unmap and then turn off the
 *    corresponding toggle on the main panel.
 *
 * 2) Removed the force of shared memory off on the JPEG widget, since
 *    the JPEG shared memory error has now been fixed.
 *
 * Revision 1.21  1994/09/03  01:57:21  drapeau
 * Added a variable "autoSearch" telling the application whether to
 * enable or disable tho "autoSearch" feature of the VideoIn widgets.  If
 * autoSearch is off, the widget will look for the requested video signal
 * but fail if not immediately found.  If autoSearch is on, the widget
 * will look at all possible video input configurations until a valid
 * sync signal is found or until exhausting all possible input types.
 *
 * Revision 1.20  1994/08/27  05:13:19  drapeau
 * Simple cosmetic changes, and disabled output by default (the user must
 * explicitly select the output toggle to turn on output).
 *
 * Revision 1.19  1994/08/24  02:02:33  drapeau
 * Added support for a command-line argument, for the HP VUE environment.
 * If an argument is passed in, VideoTool will interpret it as the name
 * of a file to display (a JFIF or JPEGMovie file).
 *
 * Revision 1.18  1994/08/19  02:36:57  andyg
 * Added a stmt to disable the "Select Window" feature of video
 * output generation.
 *
 * Revision 1.17  1994/08/14  01:42:23  drapeau
 * Three minor changes:
 *
 * 1) Moved the dimming of the "Save As TIFF" button to the
 *    OsSpecificInit() function.
 *
 * 2) Disabled shared memory in the JPEG widget.
 *
 * 3) Added a function call for OS-specific initialization.
 *
 * Revision 1.16  1994/08/10  03:12:52  drapeau
 * Removed the creation of the output widget and placed it in a module
 * that can detect whether actually to create the widget.  The widget
 * should not be created if no output function is present on the board.
 *
 * Revision 1.15  1994/08/01  18:33:23  drapeau
 * Moved the PlaceWindows() function into a different module, one that
 * contains general utility functions.
 *
 * Also made minor modifications to the code to facilitate better
 * selection of inputs.
 *
 * Revision 1.14  1994/06/09  21:27:34  drapeau
 * Disabled several features to support the first release of the HP
 * software product.
 *
 * Revision 1.13  1994/04/23  03:51:54  drapeau
 * Added an alert panel to display error messages.
 *
 * Revision 1.12  1994/04/16  00:36:28  drapeau
 * Restored the "input2Widget" video input widget to display channel
 * "INPUT_1".  It had previously been changed to display channel
 * "INPUT_0" to accommodate the HP hardware, but this change was always
 * meant to be short-term.
 *
 * Revision 1.11  1994/03/22  04:50:30  drapeau
 * Removed lots of code as part of a restructuring of the application.
 * The interface-only portion of the code moved to a new module called
 * "interface.c".
 *
 * Revision 1.1  1993/12/01  03:02:40  drapeau
 * Initial revision
 * */
static char videoToolRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/VideoTool.c,v 1.24 1994/09/14 06:02:02 drapeau Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/*
** Generated by X-Designer 
*/
/*
**LIBS: -lXm -lXt -lX11
*/
#include "VideoTool.h"

Widget		input1Widget = (Widget)NULL;
Widget		input2Widget = (Widget)NULL;
Widget		outputWidget = (Widget)NULL;
Widget		jpegWidget = (Widget)NULL;
Widget		selectedSettingsWidget = (Widget)NULL;
Widget		selectedVideoWidget = (Widget)NULL;
Widget		rootWindowWidget = (Widget)NULL;
char*		fileFormat = (char*)NULL;
XRectangle	outputSourceRect;     
Cursor		handCursor;
GC		rubberBandGC;
int		rubberBandMode;					    /* Tells whether resizing or moving the output widget */
XtAppContext	app_context;
Display*	display;					    /*  Display             */
char*		errorMessage;
Boolean		autoSearch;
Atom		deleteWindowAtom;


int main (int argc, char **argv)
{
  XSetWindowAttributes			attributes;
  XmFileSelectionBoxCallbackStruct	imageName;
  XmString				onString = XmStringCreateSimple("On");
  
  XtSetLanguageProc ( (XtAppContext) NULL, (XtLanguageProc) NULL, (XtPointer) NULL );
  XtToolkitInitialize ();
  app_context = XtCreateApplicationContext ();
  display = XtOpenDisplay (app_context, NULL, argv[0], "VideoTool",
			   NULL, 0, &argc, argv);
  if (!display)
  {
    exit (-1);
  }
  create_mainWindow ( display, argv[0], argc, argv );
  create_outputConfigShell ( mainWindow );
  create_imageShell ( mainWindow );
  create_fileDialog ( mainWindow );
  create_outputShell ( mainWindow );
  create_input2Shell ( mainWindow );
  create_input1Shell ( mainWindow );
  create_configShell ( mainWindow );
  create_alertShell ( mainWindow );
  errorMessage = malloc(1024);					    /* Allocate some memory to store error messages */
  deleteWindowAtom = XmInternAtom(display,			    /* Create an Atom for communicating with the window mgr... */
				  "WM_DELETE_WINDOW",		    /* ...that the app. needs to be informed about shells... */
				  False);			    /* ...being deleted. */
  XmAddWMProtocolCallback(input1Shell, deleteWindowAtom,	    /* Intercept window manager Delete Window calls in order... */
			  (XtCallbackProc)WMDeleteWindowCallback,   /* ...to update the interface components correctly. */
			  (XtPointer)NULL);
  XmAddWMProtocolCallback(input2Shell, deleteWindowAtom,	    /* Do this for the 2nd video input shell, too */
			  (XtCallbackProc)WMDeleteWindowCallback,
			  (XtPointer)NULL);
  OsSpecificInit();						    /* Do platform-specific initialization */
  input1Widget = XtVaCreateManagedWidget("input1Widget",	    /* Create a video input widget for the first digitizer */
					 xVideoInWidgetClass,
					 input1Shell,
					 XtNchannel, PLX_INPUT_0,
					 XtNautoSearch, autoSearch,
					 NULL);
  BuildInputMenu();						    /* Determine how many digitizers & other h/w configuration */
  rootWindowWidget = XtCreateWidget("root",
				    rootWindowWidgetClass,
				    mainWindow,
				    NULL, 0);
  jpegWidget = XtVaCreateManagedWidget("jpegWidget",
				       jpegWidgetClass,
				       imageShell,
				       NULL);
  selectedSettingsWidget = input1ChannelButton;
  selectedVideoWidget = input1Widget;
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  SetImageSaveFormat(saveAsJFIFButton, "JFIF", (XtPointer)NULL);
  XtRealizeWidget (mainWindow);
  PlaceWindows();
  XtVaSetValues(selectWindowButton, XmNsensitive, False, NULL);	    /* The "Select Window" output feature does not yet work */
  XtPopup(input1Shell, XtGrabNone);				    /* Bring up the 1st input window on application launch */
  XtVaSetValues(videoIn1Toggle, XmNlabelString, onString, NULL);    /* Make the Input1 toggle reflect apperance of Input 1 */
  XmStringFree(onString);					    /* Return memory no longer needed */
  XmToggleButtonSetState(videoIn1Toggle, True, False);		    /* Manually turn on the videoIn1 toggle */
  rubberBandGC = XCreateGC(XtDisplay(mainWindow),		    /* Initialize resources for drawing rubberBandGC boxes */
			   XtWindow(mainWindow), 0, NULL);
  XSetSubwindowMode(XtDisplay(mainWindow),
		    rubberBandGC,
		    IncludeInferiors); 
  XSetForeground(XtDisplay(mainWindow),
		 rubberBandGC,
		 BlackPixel(XtDisplay(mainWindow),
			    XScreenNumberOfScreen(XtScreen(mainWindow))));
  
  XSetFunction(XtDisplay(mainWindow), rubberBandGC, GXxor);
  XSetLineAttributes(XtDisplay(mainWindow),
		     rubberBandGC,
		     3,
		     LineSolid, CapButt, JoinMiter);
  handCursor = XCreateFontCursor(XtDisplay(mainWindow), XC_hand2);
  XtVaSetValues(outputShell, XtNborderWidth, 0, NULL);
  if (argc > 1)							    /* Was an image name specified on the command line? */
  {								    /* Yes, try to open it as a compressed image (JFIF or... */
    imageName.value = XmStringCreateSimple(argv[1]);		    /* ...JPEG Movie still). */
    OpenImageFile((Widget)NULL,
		  (XtPointer)NULL,
		  (XtPointer)&imageName);
    XmStringFree(imageName.value);				    /* Return memory no longer needed */
  }
  XtAppMainLoop (app_context);
  exit (0);
}
