/* $Header: callbacks.c,v 1.41.1.1 95/03/21 23:13:18 drapeau Exp $ */
/* $Log:	callbacks.c,v $
 * Revision 1.41.1.1  95/03/21  23:13:18  23:13:18  drapeau (George Drapeau)
 * Fixed the QuitWorkProc() function so that it only destroys non-NULL
 * widgets.
 * 
 * Revision 1.41  1994/09/14  05:58:24  drapeau
 * Significant changes to the implementation of the rubberband code.  It
 * works cleanly now.
 *
 * Revision 1.40  1994/09/14  03:20:19  drapeau
 * Added functionality to ShowInputSettings() to properly display the
 * current input signal format and standard.  Moved this functionality
 * from other functions where it does not really belong.
 *
 * Revision 1.39  1994/09/13  08:02:47  drapeau
 * Several changes:
 *
 * * Fixed the Quit() function so that it correctly destroys video
 *   widgets.
 *
 * * Added function "ResizeVideoWidgetIfNecessary()" to resize input
 *   widgets if the input signal characteristics have changed in such a
 *   way as to necessitate resizing of the widget (e.g., if signal std
 *   was changes from NTSC to PAL).
 *
 * * Implemented callbacks to handle output aspect ratio changes.
 *
 * Revision 1.38  1994/09/04  07:15:17  drapeau
 * Added WMDeleteWindowCallback(), a function that is called when the
 * window manager tries to close one of the video input shells.  The
 * function is responsible for updating the rest of the interface
 * correctly (namely, the video input toggles on the main panel).
 *
 * Revision 1.37  1994/09/03  06:01:55  drapeau
 * Modified code to better reflect the current state of video input
 * channels.  Now when trying to select a new input source, the
 * application now queries the video input widget to determine whether
 * the new input selection actually took place.
 *
 * Revision 1.36  1994/08/29  22:56:37  drapeau
 * Removed problem code in setting 4:3 and any aspect ratios: the code
 * used to set the widthInc and heightInc resources for the parent shell,
 * but this caused problems with the VUE window manager (it might've with
 * MWM, too; didn't check) and solved nothing.  In short, the code was
 * extraneous and troublesome, and so was removed.
 *
 * Revision 1.35  1994/08/27  05:19:33  drapeau
 * Enhanced Input1Callback() so that it explicitly turns on video when
 * set.  This is to handle the case when there was previously no sync
 * signal; now, the user only need toggle Input1 off then on again to try
 * to look again for sync on that channel.
 *
 * Also, added support for displaying error messages in a dialog box
 * instead of to stdout.
 *
 * Revision 1.34  1994/08/23  05:39:57  drapeau
 * Removed extraneous printf's.
 *
 * Revision 1.33  1994/08/10  03:10:47  drapeau
 * Moved Input2Callback() and OutputCallback() to the os-specific files,
 * since these two functions sensitive to platform-specific rules.
 *
 * Revision 1.32  1994/08/01  18:34:17  drapeau
 * Added functionality to better facilitate input selection, to conform
 * with the new model that accounts for multiple instances of the same
 * type of video signal.
 *
 * Revision 1.31  1994/06/23  20:59:11  drapeau
 * Added two functions to cleanly handle shell popdowns when a video input
 * shell is popped down from the window manager menu instead of from the main
 * panel.
 *
 * Revision 1.3  1994/06/14  00:39:30  drapeau
 * Fixed an error in the way JFIF files are saved.  When saving a
 * PAL-sized image, the source compression rectangle was previously not
 * correctly adjusted from NTSC size.
 *
 * Revision 1.2  1994/03/22  04:53:07  drapeau
 * Cosmetic changes to clean up the code, reflecting a restructuring of the
 * application with the addition of a "VideoTool.h" module for all major
 * includes and data structure definitions.  The "globals.h" module is now
 * obsolete.
 *
 * Revision 1.1  1993/12/01  03:03:18  drapeau
 * Initial revision
 * */
static char callbacksRcsId[] = "$Header: callbacks.c,v 1.41.1.1 95/03/21 23:13:18 drapeau Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
#include "VideoTool.h"

XPlxCImage*	ReadJPEGFile(char*, XPlxCImage*);
static void	DisplayVideoEffectsOptionSetting(Widget);
void		OutputShellEventHandler(Widget, XtPointer, XConfigureEvent*);



void XDmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
  XtManageChild ( *(Widget *)client_data );
}

void XDunmanage_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmanageChild ( *(Widget *)client_data );
}

void XDpopup_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtPopup ( *(Widget *)client_data, XtGrabNone );
}

void XDpopdown_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtPopdown ( *(Widget *)client_data );
}

void XDmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtMapWidget (*(Widget *)client_data);
}

void XDunmap_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtUnmapWidget (*(Widget *)client_data);
}

void XDenable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtSetSensitive (*(Widget *)client_data, TRUE);
}

void XDdisable_link (Widget w, XtPointer client_data, XtPointer call_data)
{
    XtSetSensitive (*(Widget *)client_data, FALSE);
}


void 
  SetLiveVideo(Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNlive, True, NULL);
  return;
}								    /* end function SetLiveVideo */



void 
  SetStillVideo (Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNlive, False, NULL);
  return;
}								    /* end function SetStillVideo */



void 
  SqueezeVideo (Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNcropped, False, NULL);
  return;
}								    /* end function SqueezeVideo */



void 
  CropVideo (Widget	w,
	     XtPointer	client_data,
	     XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNcropped, True, NULL);
  return;
}								    /* end function CropVideo */



void 
  SetAnyAspectRatio (Widget	w,
		     XtPointer	client_data,
		     XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNnormalAspectRatio, False, NULL);
  return;
}								    /* end function SetAnyAspectRatio */



void 
  Set4To3AspectRatio (Widget	w,
		      XtPointer client_data,
		      XtPointer	xt_call_data)
{
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNnormalAspectRatio, True, NULL);
  return;
}								    /* end function Set4To3AspectRatio */



void 
  SetBrightness(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNbrightness, callData->value, NULL);
  return;
}								    /* end function SetBrightness */



void 
  SetContrast(Widget	w,
	      XtPointer client_data,
	      XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNcontrast, callData->value, NULL);
  return;
}								    /* end function SetContrast */


void 
  SetHue(Widget		w,
	 XtPointer	client_data,
	 XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNhue, callData->value, NULL);
  return;
}								    /* end function SetHue */



void 
  SetSaturation(Widget		w,
		XtPointer	client_data,
		XtPointer	xt_call_data)
{
  XmScaleCallbackStruct*	callData =
    (XmScaleCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNsaturation, callData->value, NULL);
  return;
}								    /* end function SetSaturation */



Boolean QuitWorkProc(XtPointer unusedClientData)
{
  XtDestroyWidget(input1Widget);
  if ((Widget)NULL != input2Widget)				    /* Was a 2nd input widget created (i.e., VIO board used)? */
    XtDestroyWidget(input2Widget);				    /* Yes, destroy it */
  if ((Widget)NULL != outputWidget)				    /* Was an output widget created (i.e., VIO board used)? */
    XtDestroyWidget(outputWidget);				    /* Yes, destroy it */
  if ((Widget)NULL != jpegWidget)				    /* Was a JPEG widget created? */
    XtDestroyWidget(jpegWidget);				    /* Yes, destroy it */
  XtDestroyWidget(mainWindow);					    /* Free resources taken by all widgets in this application */
  exit(0);
} /* end function QuitWorkProc */



void Quit(Widget	w,
	  XtPointer	client_data,
	  XtPointer	xt_call_data)
{
  XtWorkProcId	quitProcID;
  
  quitProcID = XtAppAddWorkProc(app_context,			    /* Start a work-proc to quit the application; Xt is... */
				QuitWorkProc,			    /* ...poorly designed and this is the only way to assure... */
				(XtPointer)NULL);		    /* ...that shared memory will be freed from the JPEG... */
  return;							    /* ...widgets. */
} /* end function Quit */



void Input1Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data)
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)
  {
    XtPopup(input1Shell, XtGrabNone);
    XtVaSetValues(videoIn1Toggle, XmNlabelString, onString, NULL);
    XtVaSetValues(input1Widget, XtNchannel, PLX_INPUT_0, NULL);	    /* Restart video input if possible */
    ResizeVideoWidgetIfNecessary(input1Widget);			    /* If signal std. changed, widget might need resizing */
  }
  else
  {
    XtPopdown(input1Shell);
    XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(input1Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off video input 1 */
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input1Callback */



void 
  PopupConfigPanel(Widget		w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  static Boolean		widgetIsManaged = False;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (widgetIsManaged == False)					    /* Check if the shell has been managed yet; if not, do so... */
  {								    /* ...so that it will properly display. */
    XtManageChild(configShell);
    widgetIsManaged = True;
  }
  if (XtIsRealized(configShell) == False)
    XtRealizeWidget(configShell);
  XtPopup(configShell, XtGrabNone);
  return;
}								    /* end function PopupConfigPanel */



void 
  ShowInputSettings(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data )
{
  int		brightness, saturation, contrast, hue;
  int		instance;
  XmString	label;
  int		signalFormat;
  int		signalStandard;
  Boolean	live;
  Boolean	cropped;
  Boolean	normalAspectRatio;
  
  if (w == input1ChannelButton)					    /* Determine which video widget's settings are to be shown */
  {
    SelectVideoWidget(input1Widget);
    selectedSettingsWidget = input1ChannelButton;
  }
  else
  {
    SelectVideoWidget(input2Widget);
    selectedSettingsWidget = input2ChannelButton;
  }
/*
  XtVaSetValues(jpegWidget,
		XtNsourceWin, XtWindow(selectedVideoWidget),
		NULL);
*/
  XtVaGetValues(selectedVideoWidget,				    /* Retrieve current state of the appropriate video widget */
		XtNbrightness, &brightness,
		XtNsaturation, &saturation,
		XtNcontrast, &contrast,
		XtNhue, &hue,
		XtNlive, &live,
		XtNcropped, &cropped,
		XtNnormalAspectRatio, &normalAspectRatio,
		XtNinstance, &instance,
		XtNsignalFormat, &signalFormat,
		XtNsignalStandard, &signalStandard,
		NULL);
  DisplayVideoEffectsOptionSetting(selectedVideoWidget);	    /* Make the OptionMenu display the correct video effect */
  XmScaleSetValue(brightnessSlider, brightness);		    /* Set interface elements to reflect the state of the widget */
  XmScaleSetValue(contrastSlider, contrast);
  XmScaleSetValue(hueSlider, hue);
  XmScaleSetValue(saturationSlider, saturation);
  if (live == True)						    /* Is the widget set to display live video? */
  {								    /* Yes, update interface components appropriately */
    XmToggleButtonSetState(liveVideoToggle, True, False);
    XmToggleButtonSetState(stillVideoToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(liveVideoToggle, False, False);
    XmToggleButtonSetState(stillVideoToggle, True, False);
  }
  if (cropped == True)						    /* Is the widget set to crop video? */
  {								    /* Yes, update interface components appropriately */
    XmToggleButtonSetState(cropVideoToggle, True, False);
    XmToggleButtonSetState(squeezeVideoToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(cropVideoToggle, False, False);
    XmToggleButtonSetState(squeezeVideoToggle, True, False);
  }
  if (normalAspectRatio == True)				    /* Is the widget set to display video at a 4:3 aspect ratio? */
  {								    /* Yes, update interface components appropriately */
    XmToggleButtonSetState(fourToThreeAspectRatioToggle, True, False);
    XmToggleButtonSetState(anyAspectRatioToggle, False, False);
  }
  else
  {
    XmToggleButtonSetState(fourToThreeAspectRatioToggle, False, False);
    XmToggleButtonSetState(anyAspectRatioToggle, True, False);
  }
  switch (instance)						    /* Determine which input cable is currently being used */
  {
   case 0:
    break;
   case 1:
    break;
   case 2:
    break;
  }
  switch (signalFormat)						    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to signal format in effect. */
   case PLX_COMP:						    /* One wire: composite video */
    label = XmStringCreateLocalized("Composite");
    XtVaSetValues(inputTypeCascade, XmNlabelString, label, NULL);
    break;
   case PLX_YC:							    /* Two wires: S-Video */
    label = XmStringCreateLocalized("S-VHS");
    XtVaSetValues(inputTypeCascade, XmNlabelString, label, NULL);
    break;
   case PLX_RGB:						    /* Three wires: Red-Green-Blue video */
    label = XmStringCreateLocalized("RGB");
    XtVaSetValues(inputTypeCascade, XmNlabelString, label, NULL);
    break;
  }
  XmStringFree(label);
  switch (signalStandard)					    /* Print the appropriate label on the popup menu cascade... */
  {								    /* ...button, according to the actual signal standard... */
   case PLX_NTSC:						    /* ...in effect. */
    label = XmStringCreateLocalized("NTSC");
    XtVaSetValues(signalStandardCascade, XmNlabelString, label, NULL);
    break;
   case PLX_PAL:
    label = XmStringCreateLocalized("PAL");
    XtVaSetValues(signalStandardCascade, XmNlabelString, label, NULL);
    break;
   case PLX_SECAM:
    label = XmStringCreateLocalized("SECAM");
    XtVaSetValues(signalStandardCascade, XmNlabelString, label, NULL);
    break;
  }
  XmStringFree(label);
  return;
}								    /* end function ShowInputSettings */



void 
  RestoreFromUser1(Widget	w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNrestoreColorSettings, USER_0, NULL);
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  return;
}								    /* end function RestoreFromUser1 */



void 
  RestoreFromUser2(Widget	w,
		   XtPointer	client_data,
		   XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNrestoreColorSettings, USER_1, NULL);
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  return;
}								    /* end function RestoreFromUser2 */



void 
  RestoreFromFactoryDefaults(Widget	w,
			     XtPointer	client_data,
			     XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNrestoreColorSettings, FACTORY, NULL);
  ShowInputSettings(selectedSettingsWidget, (XtPointer)NULL, (XtPointer)NULL);
  return;
}								    /* end function RestoreFromFactoryDefaults */



void 
  SaveToUser1(Widget	w,
	      XtPointer	client_data,
	      XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNsaveColorSettings, USER_0, NULL);
  return;
}								    /* end function SaveToUser1 */



void 
  SaveToUser2(Widget	w,
	      XtPointer	client_data,
	      XtPointer xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget, XtNsaveColorSettings, USER_1, NULL);
  return;
}								    /* end function SaveToUser2 */



void 
  NormalSize(Widget	w,
	     XtPointer	client_data,
	     XtPointer	xt_call_data)
{
  Dimension			width, height;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaGetValues(selectedVideoWidget,				    /* Determine the full size of this video signal */
		XtNsignalWidth, &width,
		XtNsignalHeight, &height,
		NULL);
  XtVaSetValues(selectedVideoWidget,				    /* Reset the video widget to the full size of the signal... */
		XtNwidth, width,				    /* ...this is done as a side effect of resizing the video... */
		XtNheight, height,				    /* ...widget's parent shell. */
		NULL);
  return;
}								    /* end function NormalSize */



void 
  QuarterSize (Widget		w,
	       XtPointer	client_data,
	       XtPointer	xt_call_data)
{
  Dimension			width, height;
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaGetValues(selectedVideoWidget,				    /* Get the current size of the video widget */
		XtNsignalWidth, &width,
		XtNsignalHeight, &height,
		NULL);
  width = width / 2;
  height = height /2;
  XtVaSetValues(selectedVideoWidget,
		XmNwidth, width,
		XmNheight, height,
		NULL);
  return;
}								    /* end function QuarterSize */


void SelectVideoWidget(Widget theWidget)
{
  selectedVideoWidget = theWidget;
  return;
}								    /* end function SelectVideoWidget */



void 
  EffectsCallback(Widget	w,
		  XtPointer	client_data,
		  XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct *callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(selectedVideoWidget,
		XtNvideoParameterString, client_data,
		NULL);
  return;
}								    /* end function EffectsCallback */



static void DisplayVideoEffectsOptionSetting(Widget videoWidget)
{
  Widget	theWidget = (Widget)NULL;
  String	theString;
  
  XtVaGetValues(videoWidget,					    /* Determine the video effect set for this widget */
		XtNvideoParameterString, &theString,
		NULL);
  if (strcmp(theString, "COLOR") == 0)				    /* Determine which widget corresponds to the effect... */
    theWidget = effects1Button;					    /* ...currently being used by this widget */
  if (strcmp(theString, "BW") == 0)
    theWidget = effects2Button;
  if (strcmp(theString, "INVERTED") == 0)
    theWidget = effects3Button;
  if (strcmp(theString, "NONINVERTED") == 0)
    theWidget = effects4Button;
  if (strcmp(theString, "GENLOCK") == 0)
    theWidget = effects5Button;
  if (strcmp(theString, "FREERUN") == 0)
    theWidget = effects6Button;
  XtVaSetValues(effectsOptionMenu,				    /* Update the OptionMenu to display the correct effect */
		XmNmenuHistory, theWidget,
		NULL);
  return;
}								    /* end function DisplayVideoEffectsOptionSetting */


void 
  OpenImageFile (Widget w,
		 XtPointer client_data,
		 XtPointer xt_call_data )
{
  char*					fileName = NULL;
  XPlxCImage*				jpegData = NULL;
  XPlxCImage*				sharedImageData = NULL;
  FILE*					theFile;
  int					fileType;
  Boolean				result;
  Boolean				success = False;
  Boolean				usingSharedMemory = False;
  static Boolean			widgetIsManaged = False;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			   XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			   &fileName);
  if (result == False)						    /* Could not determine the filename that was typed */
  {
    XtUnmanageChild(fileDialogSelectionBox);
    PrintErrorMessage("The filename you typed included unprintable characters.  Please try again.");
    return;
  }
  theFile = fopen(fileName, "r");				    /* Attempt to open the requested file */
  if (theFile == (FILE*)NULL)					    /* Did the attempt fail? */
  {
    XtUnmanageChild(fileDialogSelectionBox);
    PrintErrorMessage("Could not open the file you specified.");
    return;
  }
  fileType = getc(theFile);					    /* Get the first four bytes of the file, for file typing */
  fclose(theFile);						    /* Done with the file for our purposes for now */
  if (fileType == (int)0xff)					    /* Is this file a JFIF file? */
  {								    /* Yes, load it into the JPEG widget */
    XtVaSetValues(jpegWidget,
		  XtNjfifInputFile, fileName,
		  NULL);
    XtVaGetValues(jpegWidget,
		  XtNjpegData, &jpegData,
		  NULL);
    XtVaSetValues(imageShell,
		  XtNwidth, (Dimension)jpegData->width,
		  XtNheight, (Dimension)jpegData->height,
		  NULL);
    success = True;
  }
  else								    /* No, is this file a JPEG file? */
    if (fileType == (int) 'j')
    {								    /* Yes, load the JPEG file data into the widget */
      XtVaGetValues(jpegWidget,
		    XtNuseSharedMemory, &usingSharedMemory,
		    NULL);
      if (usingSharedMemory == True)				    /* Is the JPEG widget using shared memory? */
      {
	XtVaGetValues(jpegWidget,				    /* Yes, don't try to allocate new memory for the JPEG... */
		      XtNjpegData, &sharedImageData,		    /* ...image to be read in from disk; instead, use the... */
		      NULL);					    /* ...shared memory segment already allocated. */
      }
      else
      {
	sharedImageData = (XPlxCImage*)NULL;
      }
      jpegData = ReadJPEGFile(fileName, sharedImageData);
      if (jpegData != NULL)					    /* Was the read successful? */
      {								    /* Yes, load the new data into the JPEG widget */
	if (usingSharedMemory == True)				    /* Is the JPEG widget using shared memory? */
	  XtVaSetValues(jpegWidget,				    /* Yes, don't try to tell the widget to use a different... */
			XtNputNewImage,	True,			    /* ...memory segment; instead, just inform it that the... */
			NULL);					    /* ...shared memory segment has new data to display. */
	else							    /* No, the JPEG widget isn't using shared memory; give... */
	  XtVaSetValues(jpegWidget,				    /* ...the widget new JPEG data to display. */
			XtNjpegData, jpegData,
			XtNwidth, jpegData->width,
			XtNheight, jpegData->height,
			NULL);
	XtVaSetValues(jpegWidget,
		      XtNwidth, jpegData->width,
		      XtNheight, jpegData->height,
		      NULL);
	success = True;
      }
    }
  if (success == True)						    /* Is it okay to show the widget? */
  {
    if (widgetIsManaged == False)				    /* Yes, display the widget and its image */
    {								    /* Has the shell been managed already? */
      XtManageChild(imageShell);				    /* No, manage it to show it on-screen for the 1st time */
      widgetIsManaged = True;					    /* Set a variable saying that the shell has been managed */
    }
    else							    /* Yes, the shell has already been managed; simply... */
      XtPopup(imageShell, XtGrabNone);				    /* ...popup the shell. */
  }
  XtUnmanageChild(fileDialogSelectionBox);
  return;
}								    /* end function OpenImageFile */



void 
  SaveImageFile (Widget w,
		 XtPointer client_data,
		 XtPointer xt_call_data )
{
  char*					fileName = NULL;
  XPlxCImage*				jpegData = NULL;
  FILE*					theFile;
  int					fileType;
  Boolean				result;
  Boolean				success = False;
  Dimension				sourceWindowWidth;
  Dimension				sourceWindowHeight;
  XRectangle*				compressRect;
  XRectangle*				compressDestRect;
  XmFileSelectionBoxCallbackStruct*	callData =
    (XmFileSelectionBoxCallbackStruct *) xt_call_data;
  
  result = XmStringGetLtoR(callData->value,			    /* Try to determine the filename typed by into the... */
			   XmSTRING_DEFAULT_CHARSET,		    /* ...File Selection Box. */
			   &fileName);
  if (result == False)						    /* Could not determine the filename that was typed */
  {
    XtUnmanageChild(fileDialogSelectionBox);
    PrintErrorMessage("The filename you typed included unprintable characters.  Please try again.");
    return;
  }
  XtVaGetValues(selectedVideoWidget,				    /* Determine the current size of the video window from... */
		XtNwidth, &sourceWindowWidth,			    /* ...which JPEG data will be digitized. */
		XtNheight, &sourceWindowHeight,
		NULL);
  if (strcmp(fileFormat, "JFIF") == 0)				    /* Is this file a JFIF file? */
  {								    /* Yes, load it into the JPEG widget */
    XtVaGetValues(jpegWidget,					    /* Get the current rectangle used to determine how large... */
		  XtNcompressSourceRect, &compressRect,		    /* ...a region to compress. */
		  XtNcompressDestRect, &compressDestRect,
		  NULL);
    compressRect->width = (unsigned short) sourceWindowWidth;	    /* Make sure that only the window is compressed, no more,... */
    compressRect->height = (unsigned short) sourceWindowHeight;	    /* ...no less. */
    compressDestRect->width = (unsigned short) sourceWindowWidth;
    compressDestRect->height = (unsigned short) sourceWindowHeight;
    XtVaSetValues(jpegWidget,					    /* Compress the contents of the video still image window */
		  XtNsourceWin, XtWindow(selectedVideoWidget),
		  XtNgetNewImage, True,
		  NULL);
    XtVaSetValues(jpegWidget,					    /* Save the newly-created JPEG data into a JFIF file */
		  XtNjfifOutputFile, fileName,
		  NULL);
    success = True;
  }
  else								    /* No, is this file a JPEG file? */
    if (strcmp(fileFormat, "JPEG") == 0)
    {								    /* Yes, load the JPEG file data into the widget */
      XtVaGetValues(jpegWidget,					    /* Get the current rectangle used to determine how large... */
		    XtNcompressSourceRect, &compressRect,	    /* ...a region to compress. */
		    XtNcompressDestRect, &compressDestRect,
		    NULL);
      compressRect->width = (unsigned short) sourceWindowWidth;	    /* Make sure that only the window is compressed, no more,... */
      compressRect->height = (unsigned short) sourceWindowHeight;   /* ...no less. */
      compressDestRect->width = (unsigned short) sourceWindowWidth;
      compressDestRect->height = (unsigned short) sourceWindowHeight;
      XtVaSetValues(jpegWidget,					    /* Compress the contents of the video still image window */
		    XtNsourceWin, XtWindow(selectedVideoWidget),
		    XtNgetNewImage, True,
		    NULL);
      XtVaGetValues(jpegWidget,					    /* Point to the newly-created JPEG data */
		    XtNjpegData, &jpegData,
		    NULL);
      if (jpegData != (XPlxCImage*)NULL)			    /* Was the compression step successful? */
      {
	SaveJPEGMovieStill(jpegData, fileName);			    /* Yes, save the JPEG data to a JPEG Movie file */
	success = True;
      }
    }
    else							    /* No, assume the file is a Sun raster file */
    {
      SaveImageFileInFormat(XtDisplay(w),
			    XtWindow(selectedVideoWidget),
			    fileFormat,
			    fileName);
    }
  XtUnmanageChild(fileDialogSelectionBox);
  return;
}								    /* end function SaveImageFile */



void 
  ShowOpenFileDialog (Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(fileDialog,
		XmNtitle, "Open",
		NULL);
  XtVaSetValues(saveAsCascadeButton,				    /* This function auto-detects image format; it makes no... */
		XmNsensitive, False, NULL);			    /* ...sense to allow a choice to be made manually. */
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtAddCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtManageChild(fileDialogSelectionBox);
  return;
}								    /* end function ShowOpenFileDialog */



void 
  ShowSaveFileDialog (Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(saveAsCascadeButton, XmNsensitive, True, NULL);	    /* Allow manual choice of image format for saving files. */
  XtVaSetValues(fileDialog,
		XmNtitle, "Save",
		NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtAddCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtManageChild(fileDialogSelectionBox);
  return;
}								    /* end function ShowSaveFileDialog */



void 
  ShowSaveAsFileDialog (Widget w,
			XtPointer client_data,
			XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(saveAsCascadeButton, XmNsensitive, True, NULL);	    /* Allow manual choice of image format for saving files. */
  XtVaSetValues(fileDialog,
		XmNtitle, "Save As",
		NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtRemoveCallback(fileDialogSelectionBox, XmNokCallback, OpenImageFile,NULL);
  XtAddCallback(fileDialogSelectionBox, XmNokCallback, SaveImageFile,NULL);
  XtManageChild(fileDialogSelectionBox);
  return;
}								    /* end function ShowSaveAsFileDialog */



void SetImageSaveFormat(Widget		w,
			XtPointer	clientData,
			XtPointer	xt_call_data )
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  if (fileFormat != (char*)NULL)				    /* Free up any space previously pointed to by this variable */
    free(fileFormat);
  fileFormat = strdup(clientData);				    /* Set a global determining file format for still images */
  return;
}								    /* end function SetImageSaveFormat */



void 
  SetOutputAnyAspectRatio (Widget w,
			   XtPointer client_data,
			   XtPointer xt_call_data)
{
  XtVaSetValues(outputWidget, XtNnormalAspectRatio, False, NULL);
  return;
}								    /* end function SetOutputAnyAspectRatio */



void 
  SetOutput4To3AspectRatio (Widget w,
			    XtPointer client_data,
			    XtPointer xt_call_data)
{
  XtVaSetValues(outputWidget, XtNnormalAspectRatio, True, NULL);
  return;
}								    /* end function SetOutput4To3AspectRatio */



void ShowOutputRectangle(Widget		w,
			 XtPointer	client_data,
			 XtPointer	xt_call_data)
{
  XmPushButtonCallbackStruct*	callData =
    (XmPushButtonCallbackStruct *) xt_call_data;
  static XRectangle		widgetRect;
  int				originalWindowX;
  int				originalWindowY;
  Window			rootWindow;
  Window			child;
  Dimension			originalWindowWidth;
  Dimension			originalWindowHeight;
  Dimension			borderWidth;
  
  rootWindow = RootWindowOfScreen(XtScreen(w));
  XtVaGetValues(outputWidget,					    /* Get the current size of the video output widget */
		XtNwidth, &originalWindowWidth,
		XtNheight, &originalWindowHeight,
		XtNborderWidth, &borderWidth,
		NULL);
  XTranslateCoordinates(XtDisplay(w), XtWindow(outputWidget),	    /* Get the absolute X & Y coordinates of the video... */
			rootWindow,				    /* ...input widget. */
			(int)-borderWidth, (int)-borderWidth,
			&originalWindowX, &originalWindowY,
			&child);
  widgetRect.x = (short)originalWindowX;
  widgetRect.y = (short)originalWindowY;
  widgetRect.width = (unsigned short)originalWindowWidth;
  widgetRect.height = (unsigned short)originalWindowHeight;
  rubberBandMode = MoveMode;					    /* Tell the app to move the output rectangle by default */
  DrawBox(w, rubberBandGC, widgetRect.x, widgetRect.y,		    /* Draw the new rubberband rectangle */
	  widgetRect.width, widgetRect.height);
  XWarpPointer(XtDisplay(w), None, rootWindow, 0, 0, 0, 0,	    /* Move the pointer to the origin of the video output widget */
	       (int)widgetRect.x, (int)widgetRect.y);
  XtGrabPointer(rootWindowWidget, False,			    /* Grab control of the pointer so that rubberbanding can... */
		(PointerMotionMask |				    /* ...be done. */
		 ButtonPressMask |
		 ButtonReleaseMask),
		GrabModeAsync, GrabModeAsync,
		None, handCursor, CurrentTime);
  XtAddRawEventHandler(rootWindowWidget,
		       (ButtonPressMask),
		       True, StartRubberBandAction, (XtPointer)&widgetRect);
  XtAddRawEventHandler(rootWindowWidget,
		       (PointerMotionMask),
		       True, MoveResize, (XtPointer)&widgetRect);
  return;
}								    /* end function ShowOutputRectangle */



void 
  SelectOutputWindow (Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data)
{
  XmPushButtonCallbackStruct *call_data =
    (XmPushButtonCallbackStruct *) xt_call_data;
  return;
}								    /* end function SelectOutputWindow */



void 
  MaximumSizeOutputWindow(Widget w, XtPointer client_data, XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *call_data = (XmPushButtonCallbackStruct *) xt_call_data;
  int	width;
  int	height;
  
  XtVaGetValues(outputWidget,
		XtNsignalWidth, &width,
		XtNsignalHeight, &height,
		NULL);
  XtVaSetValues(outputWidget,
		XtNwidth, (Dimension)width,
		XtNheight, (Dimension)height,
		NULL);
  return;
}								    /* end function MaximumSizeOutputWindow */



void 
SetOutputStandardNTSC(Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *call_data =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(outputWidget,
		XtNsignalStandard, PLX_NTSC,
		NULL);
  return;
} /* end function SetOutputStandardNTSC */



void 
SetOutputStandardPAL (Widget w,
		      XtPointer client_data,
		      XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *call_data =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(outputWidget,
		XtNsignalStandard, PLX_PAL,
		NULL);
  return;
} /* end function SetOutputStandardPAL */



void 
SetOutputStandardSECAM (Widget w,
			XtPointer client_data,
			XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *call_data =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(outputWidget,
		XtNsignalStandard, PLX_SECAM,
		NULL);
  return;
} /* end function SetOutputStandardSECAM */



void 
SetOutputFormatComposite (Widget w,
			  XtPointer client_data,
			  XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *call_data =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(outputWidget,
		XtNsignalFormat, PLX_COMP,
		NULL);
  return;
} /* end function SetOutputFormatComposite */



void 
SetOutputFormatSVHS (Widget w,
		     XtPointer client_data,
		     XtPointer xt_call_data )
{
  XmPushButtonCallbackStruct *call_data =
    (XmPushButtonCallbackStruct *) xt_call_data;
  
  XtVaSetValues(outputWidget,
		XtNsignalFormat, PLX_YC,
		NULL);
  return;
} /* end function SetOutputFormatSVHS */



void PopdownInput1Shell(Widget w,
			XtPointer client_data,
			XtPointer xt_call_data)
{
  XmString	offString = XmStringCreateSimple("Off");
  
  XmToggleButtonSetState(videoIn1Toggle, False, False);		    /* Manually turn off the videoIn1 toggle */
  XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);   /* Set the On/Off string beside the toggle to "Off" */
  XmStringFree(offString);
  return;
} /* end function PopdownInput1Shell */



void PopdownInput2Shell(Widget w,
			XtPointer client_data,
			XtPointer xt_call_data)
{
  XmString	offString = XmStringCreateSimple("Off");
  
  XmToggleButtonSetState(videoIn2Toggle, False, False);		    /* Manually turn off the videoIn2 toggle */
  XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);   /* Set the On/Off string beside the toggle to "Off" */
  XmStringFree(offString);
  return;
} /* end function PopdownInput2Shell */



void SetInputInstance(Widget	w,
		      XtPointer client_data,
		      XtPointer	xt_call_data )
{
  int		channel;
  int		instance;
  XmString	label;
  int		signalFormat;
  
  XtVaGetValues(selectedVideoWidget,
		XtNchannel, &channel,
		NULL);
  if ((w == compositeInstanceButton1) ||
      (w == ycInstanceButton1) ||
      (w == rgbInstanceButton1))
  {
    instance = 0;
  }
  if (w == compositeInstanceButton2)
    instance = 1;
  if (w == compositeInstanceButton3)
    instance = 2;
  
  if ((w == compositeInstanceButton1) ||
      (w == compositeInstanceButton2) ||
      (w == compositeInstanceButton3))
  {
    signalFormat = PLX_COMP;
  }
  if (w == ycInstanceButton1)
    signalFormat = PLX_YC;
  if (w == rgbInstanceButton1)
    signalFormat = PLX_RGB;
  
  XtVaSetValues(selectedVideoWidget,				    /* Attempt to look at another input cable ("instance") */
		XtNsignalFormat, signalFormat,
		XtNinstance, instance,
		NULL);
  ShowInputSettings(selectedSettingsWidget,			    /* Update labels on the input settings panel */
		    (XtPointer)NULL, (XtPointer)NULL);
  ResizeVideoWidgetIfNecessary(selectedVideoWidget);		    /* If signal std. changed, widget might need resizing */
  return;
} /* end function SetInputInstance */



void SetSignalStandard(Widget		w,
		       XtPointer	client_data,
		       XtPointer	xt_call_data)
{
  int		signalStandard;
  
  if (w == ntscButton)
    signalStandard = PLX_NTSC;
  if (w == palButton)
    signalStandard = PLX_PAL;
  if (w == secamButton)
    signalStandard = PLX_SECAM;
  
  XtVaSetValues(selectedVideoWidget,				    /* Try to set signal standard to the newly-requested value */
		XtNsignalStandard, signalStandard,
		NULL);
  XtVaGetValues(selectedVideoWidget,				    /* Find out if the signal standard really changed */
		XtNsignalStandard, &signalStandard,
		NULL);
  ResizeVideoWidgetIfNecessary(selectedVideoWidget);		    /* If signal std. changed, widget might need resizing */
  ShowInputSettings(selectedSettingsWidget,			    /* Update labels on the input settings panel */
		    (XtPointer)NULL, (XtPointer)NULL);
  return;
} /* end function SetSignalStandard */



void WMDeleteWindowCallback(Widget	widget,
			    XtPointer	clientData,
			    XtPointer	callData)
{
  XmString	offString = XmStringCreateLocalized("Off");
  
  if (widget == input1Shell)
  {
    XmToggleButtonSetState(videoIn1Toggle, False, False);	    /* Turn off the 1st video input's toggle */
    XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);
  }
  if (widget == input2Shell)
  {
    XmToggleButtonSetState(videoIn1Toggle, False, False);	    /* Turn off the 1st video input's toggle */
    XtVaSetValues(videoIn1Toggle, XmNlabelString, offString, NULL);
  }
  XmStringFree(offString);
  return;
} /* end function WMDeleteWindowCallback */



void ResizeVideoWidgetIfNecessary(Widget widget)
{
  int		signalStandard;
  Dimension	signalHeight;
  Dimension	signalWidth;
  Dimension	height;
  Dimension	width;
  
  XtVaGetValues(widget, XtNwidth, &width,			    /* Find out what the current widget's width and height are */
		XtNheight, &height,
		NULL);
  XtVaGetValues(widget, XtNsignalWidth, &signalWidth,		    /* Find out what the widget's width & height *should* be... */
		XtNsignalHeight, &signalHeight,			    /* ...if they fit their signal standards */
		NULL);
  if ((width != signalWidth) || (height != signalHeight))	    /* If the two values are in disagreement, resize the... */
  {								    /* ...video widget to fit the signal standard. */
    XtVaSetValues(widget, XtNwidth, signalWidth,
		  XtNheight, signalHeight,
		  NULL);
  }
} /* end function ResizeWidgetIfNecessary */
