/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/hpSpecific.c,v 0.95 1994/09/04 07:16:11 drapeau Exp $ */
/* $Log: hpSpecific.c,v $
 * Revision 0.95  1994/09/04  07:16:11  drapeau
 * Disabled auto-search of video inputs.
 *
 * Revision 0.94  1994/08/27  05:22:39  drapeau
 * Modified Input2Callback(), explicitly setting the input channel on
 * when the toggle is set.  This allows the user to re-check for a sync
 * signal if one isn't found the first time VideoTool is started.
 *
 * Also added support for displaying error messages in a dialog box
 * instead of on stdout.
 *
 * Revision 0.93  1994/08/19  07:43:02  drapeau
 * Removed code that does not work well on the HP platform.
 *
 * Revision 0.92  1994/08/19  02:33:13  andyg
 * Added HP-specific code for producing Sun raster files.
 *
 * Revision 0.91  1994/08/14  01:43:37  drapeau
 * Added the function OsSpecificInit() for platform-specific
 * initialization code.
 *
 * Revision 0.90  1994/08/10  03:09:00  drapeau
 * First working version.
 * */
static char hpSpecificRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/hpSpecific.c,v 0.95 1994/09/04 07:16:11 drapeau Exp $";

#include "VideoTool.h"



void Input2Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data )
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)					    /* Should input 2 be activated? */
  {								    /* Yes, do the appropriate book work */
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(input2Widget, XtNchannel, PLX_INPUT_1, NULL);	    /* Restart video on the 2nd input */
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
    XmToggleButtonSetState(videoOutToggle, False, False);	    /* Turn off the video output's toggle */
    XtPopup(input2Shell, XtGrabNone);				    /* Popup the 2nd video input's shell */
    XtVaSetValues(videoIn2Toggle, XmNlabelString, onString, NULL);
  }
  else								    /* Nope, disable input 2 for now */
  {
    XtPopdown(input2Shell);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input2Callback */



void 
  OutputCallback(Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)
  {
    XtPopdown(input2Shell);					    /* Remove the 2nd input from the screen for now */
    XmToggleButtonSetState(videoIn2Toggle, False, False);	    /* Turn off the 2nd video input's toggle */
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(outputWidget, XtNchannel, PLX_OUTPUT_0, NULL);    /* Turn on generation of a video signal, if not already on */
    XtVaSetValues(input2Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off the 2nd input so as not to conflict with output */
    XtVaSetValues(videoOutToggle, XmNlabelString, onString, NULL);
  }
  else
  {
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function OutputCallback */



void OsSpecificInit(void)
{
  autoSearch = False;						    /* Don't auto-search for a valid video input signal */
  XtVaSetValues(saveAsTIFFButton, XmNsensitive, False, NULL);	    /* Temporary!!!!! Dim the TIFF option for saving files */
  return;
} /* end function OsSpecificInit */



void SaveImageFileInFormat(Display*	display,
			   Window	imageWindow,
			   char*	fileFormat,
			   char*	fileName)
{
  char		windowName[256];
  XImage*	localImage;
  FILE*		imageFile;
  
  if (strcmp(fileFormat, "SUN") == 0)				    /* Hack!!!!!: if the image type to be saved is Sunraster,... */
  {								    /* ...go around normal means on the HP platform to save... */
    localImage = XGetImage(display, imageWindow, 0, 0, 640, 480,
			   AllPlanes, ZPixmap);
    imageFile = fopen(fileName, "w");
    if (imageFile == (FILE*)NULL)
    {
      sprintf(errorMessage, "Could not open the file %s for writing.\n", fileName);
      PrintErrorMessage(errorMessage);
      return;
    }
    pr_dump_ximage(imageFile, localImage, 0, NULL, NULL, NULL);
    return;
  }
  return;
}								    /* end function SaveImageFileInFormat */
