/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/imageIO.c,v 1.16 1994/09/14 03:24:07 drapeau Exp $ */
/* $Log: imageIO.c,v $
 * Revision 1.16  1994/09/14  03:24:07  drapeau
 * Slight code cleanups in the ReadJPEGFile() function to make code more
 * readable and portable.
 *
 * Revision 1.15  1994/08/27  05:52:34  drapeau
 * Fixed errors in function prototypes, and added support for displaying
 * error messages in a dialog box instead of printing to stdout.
 *
 * Revision 1.14  1994/08/19  02:33:50  andyg
 * Removed the function SaveImageFileInFormat(), moving it into the
 * platform-specific modules, since saving images works differently
 * according to the different hardware.
 *
 * Revision 1.13  1994/06/09  01:35:30  drapeau
 * Put in several changes to fix errors and to workaround the HP
 * platform:
 *
 * * Fixed errors in the way JFIF images are saved.  When reading a JFIF
 * file, the previous version neglected to read the four bytes containing
 * the size of the image.  This version now does so.  Also, CImage's are
 * now correctly allocated, using XPlxCreateCImage() instead of manually
 * allocating storage.
 *
 * * Modified SaveImageFileInFormat() mainly for the HP platform
 * (although changes occur on all platforms at present).  The change was
 * to allow storage of Sun raster files.  The previous version did not
 * work since the ImageMagick code to read and save XImages got
 * conflicting information about the image.
 *
 * Revision 1.12  1994/03/22  04:55:00  drapeau
 * Cosmetic changes to reflect new "VideoTool.h" module that includes
 * all necessary include files and data structure definitions.
 *
 * Revision 1.11  1994/03/11  19:40:26  drapeau
 * Fixed errors in saving JPEG Movie stills; previous versions of this program
 * did not save a syntactically-correct file.
 *
 * Revision 1.1  1993/12/01  03:03:48  drapeau
 * Initial revision
 * */
static char imageIORcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/imageIO.c,v 1.16 1994/09/14 03:24:07 drapeau Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
#include "VideoTool.h"

#include <display.h>						    /* Include files from the ImageMagick package */
#include <image.h>
#include <alien.h>
#include <X.h>

#define True 1
#define False 0

char*	client_name;



/*
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                                                                             %
%                                                                             %
%   E r r o r                                                                 %
%                                                                             %
%                                                                             %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Function Error displays an error message and then terminates the program.
%
%  The format of the Error routine is:
%
%      Error(message,qualifier)
%
%  A description of each parameter follows:
%
%    o message: Specifies the message to display before terminating the
%      program.
%
%    o qualifier: Specifies any qualifier to the message.
%
%
*/
void Error(message,qualifier)
char
  *message,
  *qualifier;
{
  (void) fprintf(stderr,"%s: %s",client_name,message);
  if (qualifier != (char *) NULL)
    (void) fprintf(stderr," (%s)",qualifier);
  (void) fprintf(stderr,".\n");
  exit(1);
}



XPlxCImage* ReadJPEGFile(char* fileName, XPlxCImage* allocatedImage)
{
  char*		imageData = (char*)NULL;
  jpheader	jpegFileHeader;
  XPlxCImage*	jpegImage = (XPlxCImage*)NULL;
  unsigned char	opcode;
  int		result;
  int		theFile;
  
  theFile = open(fileName, O_RDONLY);
  if (theFile == -1)						    /* Try to open the file for reading; did it fail? */
  {
    PrintErrorMessage("Couldn't open jpeg file for input.");	    /* Yes, print an error message and return a NULL pointer */
    return((XPlxCImage*)NULL);
  }
  if (read(theFile, &jpegFileHeader, sizeof(jpheader)) < 0)	    /* Attempt to read in the header information */
  {
    PrintErrorMessage("Could not read the JPEG file header.");
    close(theFile);
    return((XPlxCImage*) NULL);
  }
  if (strcmp(jpegFileHeader.magic, "j_movie") != 0)		    /* Is this really a JPEG Movie file? */
  {								    /* No, report the error and return immediately */
    sprintf(errorMessage, "The file \"%s\" is not a valid JPEG compressed still image.\n",
	   fileName);
    PrintErrorMessage(errorMessage);
    close(theFile);
    return((XPlxCImage*) NULL);
  }
  if (jpegFileHeader.frames != 1)				    /* Does this JPEG Movie have more than one image in it? */
  {								    /* Yes, that will not do; report the error and return. */
    sprintf(errorMessage, "The file \"%s\" is not a valid JPEG compressed still image",
	   fileName);
    PrintErrorMessage(errorMessage);
    close(theFile);
    return((XPlxCImage*) NULL);
  }
  if (allocatedImage == (XPlxCImage*)NULL)			    /* Was the image passed in already allocated? */
  {
    jpegImage = XPlxCreateCImage((Display*)NULL,		    /* No, allocate space for a new image, asking the... */
				 (char*)NULL,			    /* ...CreateCImage() routine to allocate data to hold the... */
				 jpegFileHeader.bandwidth,	    /* ...image itself, instead of this function passing in... */
				 jpegFileHeader.width,		    /* ...already-allocated data. */
				 jpegFileHeader.height);
    imageData = jpegImage->data;				    /* Point to the newly-allocated data */
  }
  else								    /* Yes, no need to allocate space; simply point to the... */
    imageData = allocatedImage->data;				    /* ...existing image data. */
  read(theFile, &opcode, 1);					    /* Read the 1st opcode for the single-frame movie */
  if (opcode != LOAD_JPEG)					    /* It should've been a LOAD_JPEG instruction; was it? */
  {								    /* Nope, don't bother going any further */
    PrintErrorMessage("There was a problem reading in the JPEG Movie still.");
    if (allocatedImage == (XPlxCImage*)NULL)			    /* Was the image passed in already allocated? */
      XPlxDestroyCImage(jpegImage);				    /* No; free space allocated by this function  */
    close(theFile);
    return((XPlxCImage*) NULL);
  }
  read(theFile, imageData, sizeof(int));			    /* Consume the integer containing size of this JPEG image */
  if (read(theFile, imageData, jpegFileHeader.bandwidth) < 0)	    /* Get the compressed frame out */
  {
    PrintErrorMessage("There was a problem reading in the JPEG Movie data.");
    if (allocatedImage == (XPlxCImage*)NULL)			    /* Was the image passed in already allocated? */
      XPlxDestroyCImage(jpegImage);				    /* No; free space allocated by this function  */
    close(theFile);
    return((XPlxCImage*) NULL);
  }
  close(theFile);						    /* Done reading the still image, successfully. */
  if (allocatedImage != (XPlxCImage*)NULL)			    /* Was the image passed in already allocated? */
  {								    /* Yes, it is now safe to modify the existing JPEG... */
    jpegImage = allocatedImage;					    /* ...image's data structures, updating size & dimensions... */
    jpegImage->width = jpegFileHeader.width;			    /* ...of the image. */
    jpegImage->height = jpegFileHeader.height;
    jpegImage->size = jpegFileHeader.bandwidth;
  }
  return(jpegImage);						    /* Return the JPEG image data to the caller */
}								    /* end function ReadJPEGFile */



int SaveJPEGMovieStill(XPlxCImage*	image,			    /* Taken from "jpeg_save_still" as written by... */
		       char*		file)			    /* ...John Malleo-Roach */
{
  u_char    c;
  int       i;
  int      fd;
  jpheader  h;
  
  if ((fd = open(file,O_CREAT|O_WRONLY|O_TRUNC,0644)) == -1)
  {
    sprintf(errorMessage, "Couldn't open JPEG Movie file \"%s\" for output.\n", file);
    PrintErrorMessage(errorMessage);
    return(-1);
  }
  strcpy(h.magic,"j_movie");
  h.version    = JPEG_VERSION_2;
  h.fps        = 0;
  h.frames     = 1;
  h.width      = image->width;
  h.height     = image->height;
  h.bandwidth  = image->size;
  h.qfactor	 = 0;
  h.mapsize	 = 256 * 256 * 256;
  h.tracks     = 0;
  h.volbase    = 0;
  h.audioslice = 0;
  
  for (i=0; i<FILLER; i++)
    h.filler[i] = 0;
  if (write(fd, (void*)&h,sizeof(jpheader)) < 0)		    /* Put out the header info */
  {
    sprintf(errorMessage, "Could not save the JPEG Movie file \"%s\".\n", file);
    PrintErrorMessage(errorMessage);
    close(fd);
    unlink(file);
    return(-1);
  }
  c = LOAD_JPEG;						    /* Write the load jpeg frame instruction */
  write(fd,&c,1);
  write(fd, &(image->size), sizeof(int));			    /* Write the number of bytes to follow for this JPEG image */
  if (write(fd,image->data,image->size) < 0)			    /* Save the compressed image data */
  {
    sprintf(errorMessage, "Could not save the JPEG Movie file \"%s\".\n", file);
    PrintErrorMessage(errorMessage);
    close(fd);
    unlink(file);
    return(-1);
  }
  c = END_FRAME;						    /* Write the end-of-this-frame instruction */
  write(fd,&c,1);
  close(fd);
  return(NULL);
}								    /* end function SaveJPEGMovieStill */
