/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/rubberBand.c,v 1.13 1994/09/14 05:57:50 drapeau Exp $ */
/* $Log: rubberBand.c,v $
 * Revision 1.13  1994/09/14  05:57:50  drapeau
 * Significant changes to the rubberband code; it works cleanly now.
 *
 * Revision 1.12  1994/08/06  01:07:06  drapeau
 * Minor cosmetic change: removed an extraneous diagnostic message.
 *
 * Revision 1.11  1994/03/22  04:56:33  drapeau
 * Cosmetic changes to reflect new "VideoTool.h" module that includes
 * all necessary include files and data structure definitions.
 *
 * Revision 1.1  1993/12/01  03:03:52  drapeau
 * Initial revision
 * */
static char rubberBandRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/rubberBand.c,v 1.13 1994/09/14 05:57:50 drapeau Exp $";
/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */

#include "VideoTool.h"


void StartRubberBandAction(Widget	widget,
			   XtPointer	rectData,
			   XEvent*	event,
			   Boolean*	unused) 
{
  XRectangle*	theRect = (XRectangle*)rectData;
  
  if (event->xbutton.button == Button2)				    /* Was the "resize" button pressed? */
  {
    rubberBandMode = ResizeMode;				    /* Yes, call the resize function */
    XWarpPointer(XtDisplay(widget), None,			    /* Move the pointer to the lower-right corner of the... */
		 RootWindowOfScreen(XtScreen(widget)),		    /* ...video output widget. */
		 0, 0, 0, 0,
		 (theRect->x + theRect->width),
		 (theRect->y + theRect->height));
  }
  else
  {
    rubberBandMode = MoveMode;
  }
  XtRemoveRawEventHandler(rootWindowWidget,			    /* Now that StartRubberBandAction has done its job,... */
			  (ButtonPressMask),			    /* ...tell the app that it is done. */
			  True, StartRubberBandAction,
			  rectData);
  XtAddRawEventHandler(rootWindowWidget,
		       (ButtonReleaseMask),
		       True, EndRubberBandAction, rectData);
  return;
} /* end function StartRubberBandAction */



void EndRubberBandAction(Widget		widget,
			 XtPointer	rectData,
			 XEvent*	event,
			 Boolean*	unused) 
{
  XRectangle*	theRect = (XRectangle*)rectData;
  
  UndrawBox(widget, rubberBandGC,				    /* Erase the old box one final time */
	    theRect->x, theRect->y,
	    theRect->width, theRect->height);
  XtUngrabPointer(rootWindowWidget, CurrentTime);		    /* Allow normal operation of the X server again */
  XtRemoveRawEventHandler(rootWindowWidget,			    /* Remove the special event handlers for drawing the... */
			  (PointerMotionMask),			    /* ...rubberband rectangles. */
			  True, MoveResize,
			  rectData);
  XtRemoveRawEventHandler(rootWindowWidget,
			  (ButtonReleaseMask),
			  True, EndRubberBandAction,
			  rectData);
  XtVaSetValues(outputShell,					    /* Resize the video output rectangle */
		XtNx, (Position)theRect->x,
		XtNy, (Position)theRect->y,
		XtNwidth, (Dimension)theRect->width,
		XtNheight, (Dimension)theRect->height,
		NULL);
  XtVaSetValues(outputWidget,
		XtNwidth, (Dimension)theRect->width,
		XtNheight, (Dimension)theRect->height,
		NULL);
  rubberBandMode = MoveMode;					    /* Next time MoveResize is called, enter move mode */
  return;
} /* end function EndRubberBandAction */



void MoveResize(Widget		widget,
		XtPointer	rectData,
		XEvent*		event,
		Boolean*	unused) 
{
  XRectangle*	theRect = (XRectangle*)rectData;
  
  if (((event->xbutton.x_root < theRect->x) ||			    /* If resizing the output widget, check if the mouse... */
       (event->xbutton.y_root < theRect->y)) &&			    /* ...position would create a rectangle smaller then 0,0. */
      (rubberBandMode == ResizeMode))
  {
    return;
  }
  UndrawBox(widget, rubberBandGC,				    /* Erase the old box */
	    theRect->x, theRect->y,
	    theRect->width, theRect->height);
  if (rubberBandMode == ResizeMode)				    /* Is the output widget being resized?  If so, update the... */
  {								    /* ...width and height coordinates of the output rectangle. */
    theRect->width = event->xbutton.x_root - theRect->x;
    theRect->height = event->xbutton.y_root - theRect->y;
  }
  else								    /* No, the output widget is being moved; update the x & y... */
  {								    /* ...starting coords of the output rectangle. */
    theRect->x = event->xbutton.x_root;
    theRect->y = event->xbutton.y_root;
  }
  DrawBox(widget, rubberBandGC, theRect->x, theRect->y,		    /* Redraw the new rubberband rectangle */
	  theRect->width, theRect->height);
  return;
} /* end function MoveResize */



UndrawBox(Widget	widget,
	  GC		gc,
	  int		x,
	  int		y,
	  int		width,
	  int		height)
{
  DrawBox(widget, gc, x, y, width, height);
  return;
} /* end function UndrawBox */



DrawBox(Widget	widget,
	GC	gc,
	int	x,
	int	y,
	int	width,
	int	height)
{
  static Display*	display;
  static Boolean	firstTime = True;
  static Window		rootWindow;
  static int		screenNumber;
  
  if (firstTime == True)
  {
    firstTime = False;
    display = XtDisplay(widget);
    rootWindow = RootWindowOfScreen(XtScreen(widget));
    screenNumber = XScreenNumberOfScreen(XtScreen(widget));
    XSetForeground(display, gc, WhitePixel(display, screenNumber) ^ BlackPixel(display, screenNumber));
  }
  XDrawRectangle(display, rootWindow, gc,
		 x, y, width, height);
  return;
} /* end function DrawBox */
