/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/sunSpecific.c,v 0.95 1994/09/13 08:04:59 drapeau Exp $ */
/* $Log: sunSpecific.c,v $
 * Revision 0.95  1994/09/13  08:04:59  drapeau
 * Fixed an error in Input2Callback(), which was incorrectly setting the
 * 2nd input's input to the 1st digitizer.  Also, added a call to
 * ResizeVideoWidgetIfNecessary() to handle widget adjustments when
 * finding a new signal standard.
 *
 * Revision 0.94  1994/09/04  07:15:45  drapeau
 * Enabled auto-search on video input widgets.
 *
 * Revision 0.93  1994/08/24  22:32:02  drapeau
 * Removed a memory leak in SaveImageFileInFormat(); now properly
 * destroying image data after saving to disk and no longer needed.
 *
 * Revision 0.92  1994/08/19  02:36:06  andyg
 * Moved function SaveImageFileInFormat() into this platform-
 * specific file.
 *
 * Revision 0.91  1994/08/14  01:43:45  drapeau
 * Added the function OsSpecificInit() for platform-specific
 * initialization code.
 *
 * Revision 0.90  1994/08/10  03:09:00  drapeau
 * First working version.
 * */
static char sunSpecificRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/sunSpecific.c,v 0.95 1994/09/13 08:04:59 drapeau Exp $";

#include "VideoTool.h"

#include <display.h>						    /* Include files from the ImageMagick package */
#include <image.h>
#include <alien.h>
#include <X.h>

#define True 1
#define False 0


void Input2Callback(Widget	w,
		    XtPointer	client_data,
		    XtPointer	xt_call_data )
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)
  {
    XtPopup(input2Shell, XtGrabNone);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, onString, NULL);
    XtVaSetValues(input2Widget, XtNchannel, PLX_INPUT_1, NULL);	    /* Restart video input if possible */
    ResizeVideoWidgetIfNecessary(input2Widget);			    /* If signal std. changed, widget might need resizing */
  }
  else
  {
    XtPopdown(input2Shell);
    XtVaSetValues(videoIn2Toggle, XmNlabelString, offString, NULL);
    XtVaSetValues(input2Widget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off video input 2 */
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function Input2Callback */



void 
  OutputCallback(Widget		w,
		 XtPointer	client_data,
		 XtPointer	xt_call_data)
{
  XmString			onString = XmStringCreateSimple("On");
  XmString			offString = XmStringCreateSimple("Off");
  XmToggleButtonCallbackStruct*	callData =
    (XmToggleButtonCallbackStruct *) xt_call_data;
  
  if (callData->set == True)
  {
    XtVaSetValues(outputWidget, XtNchannel, PLX_OUTPUT_0, NULL);    /* Turn on generation of a video signal, if not already on */
    XtVaSetValues(videoOutToggle, XmNlabelString, onString, NULL);
  }
  else
  {
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(videoOutToggle, XmNlabelString, offString, NULL);
  }
  XmStringFree(onString);
  XmStringFree(offString);
  return;
}								    /* end function OutputCallback */



void OsSpecificInit(void)
{
  autoSearch = True;						    /* Allow auto-searching for a valid video input signal */
  return;							    /* For now, don't do anything special for the Sun platform */
} /* end function OsSpecificInit */



void SaveImageFileInFormat(Display*	display,
			   Window	imageWindow,
			   char*	fileFormat,
			   char*	fileName)
{
  char		windowName[256];
  Image*	theImage = (Image*)NULL;
  
  sprintf(windowName, "%dx", imageWindow);
  theImage = ReadXImage(windowName,				    /* Use ImageMagick routine to digitize the contents of an... */
			XDisplayName(NULL),			    /* ...X window, the one that will be written to disk. */
			False, False, False);
  if (theImage == (Image*)NULL)					    /* Did the image grab fail? */
    return;							    /* Yes, return without doing anything more */
  sprintf(theImage->filename, "%s:%s", fileFormat, fileName);
  WriteAlienImage(theImage);					    /* Write the Sun Raster file to disk */
  DestroyImage(theImage);					    /* Done with the space needed to store the image */
  return;
}								    /* end function SaveImageFileInFormat */
