/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/utils.c,v 1.4 1994/09/03 01:58:32 drapeau Exp $ */
/* $Log: utils.c,v $
 * Revision 1.4  1994/09/03  01:58:32  drapeau
 * Minor modifications to PlaceWindows(), using XmN strings instead of
 * XtN strings in the hope that Motif widgets would do a better job of
 * suggesting their preferred positions to the HP VUE window manager.  So
 * far the attempt has failed.
 *
 * Revision 1.3  1994/08/27  05:12:11  drapeau
 * Fixed the application so that it does not enable output by default
 * (the user must select the toggle in order to enable output).  Also
 * added the function PrintErrorMessage(), a convenience function to put
 * up a simple error dialog box with a message.
 *
 * Revision 1.2  1994/08/10  03:12:08  drapeau
 * Made the application's sensitivity to various board configurations
 * more complete.  The application should now be fully aware of whether
 * the underlying hardware has two digitizers or only one.
 *
 * Revision 1.1  1994/08/01  18:55:57  drapeau
 * Initial revision
 * */
static char utilsRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/utils.c,v 1.4 1994/09/03 01:58:32 drapeau Exp $";

#include "VideoTool.h"


void PlaceWindows()						    /* Try to place the main windows in a nice position */
{
  int		screenWidth;
  Dimension	width;
  Dimension	height;
  
  screenWidth = XWidthOfScreen(XtScreen(mainWindow));		    /* How wide is the screen?  Use for centering windows */
  XtVaGetValues(mainWindow,					    /* How large is the main window? */
		XtNwidth, &width,
		XtNheight, &height,
		NULL);
  XtVaSetValues(mainWindow,					    /* Place the main window at the top of the screen, centered */
		XmNx, (Position)((screenWidth / 2) - (width / 2)),
		XmNy, 0,
		NULL);
  XtVaSetValues(input1Shell,					    /* Place the 1st video input window at the left side of... */
		XmNx, 0,					    /* ...the screen, just below the main window. */
		XmNy, (Position)height,
		NULL);
  XtVaGetValues(input2Shell,					    /* How wide is video input window 2?  Use for centering */
		XtNwidth, &width,
		NULL);
  XtVaSetValues(input2Shell,					    /* Place the 2nd video input window against the right... */
		XmNx, (Position)(screenWidth - width),		    /* ...side of the screen, also just below the main window. */
		XmNy, (Position)height,
		NULL);
  XtVaGetValues(imageShell,					    /* How wide is the still image shell?  Use for centering */
		XtNwidth, &width,
		NULL);
  XtVaSetValues(imageShell,					    /* Place the still image shell window centered, just... */
		XmNx, (Position)((screenWidth / 2) - (width / 2)),  /* ...below the main window. */
		XmNy, (Position)height,
		NULL);
  return;
} /* end function PlaceWindows */



void BuildInputMenu(void)
{
  plx_IO*		hardwareInfo;				    /* Info about the Parallax board being used */
  plx_IO_list*		inputList = (plx_IO_list*) NULL;
  int			numDigitizers;
  Bool			result;
  
  XtVaGetValues(input1Widget,					    /* Retrieve information about the hardware being used */
		XtNhardwareParams, &hardwareInfo,
		NULL);
  numDigitizers = 1;						    /* Unless otherwise known, the board being used only has... */
								    /* ...1 digitizer (i.e., no VIO option). */
  XtVaSetValues(videoIn2Toggle, XmNsensitive, False, NULL);	    /* Dim the second input, assuming only 1 digitizer on board */
  XtVaSetValues(videoOutToggle, XmNsensitive, False, NULL);	    /* Dim the output, too, assuming only 1 digitizer on board */
  XtVaSetValues(outputConfigButton, XmNsensitive, False, NULL);	    /* Don't let the output configuration panel be selected */
  XtVaSetValues(input2ChannelButton, XmNsensitive, False, NULL);    /* Don't let 2nd input settings to be used */
  for (inputList = hardwareInfo->inputs;			    /* Search through all available inputs for this hardware,... */
       inputList != (plx_IO_list*) NULL;			    /* ...to determine if 2 digitizers are on board. */
       inputList = inputList->next_IO)
  {
    if (inputList->channel == PLX_INPUT_1)			    /* Was a 2nd digitizer found? */
    {
      numDigitizers = 2;					    /* Yes, set a variable indicating this */
    }
  }
  if (numDigitizers == 2)
  {
    XtVaSetValues(videoIn2Toggle, XmNsensitive, True, NULL);	    /* Allow the 2nd input to be used */
    input2Widget = XtVaCreateManagedWidget("input2Widget",
					   xVideoInWidgetClass,
					   input2Shell,
					   XtNchannel, PLX_INPUT_1,
					   XtNinstance, 1,	    /* Choose the 2nd video cable for this digitizer by default */
					   XtNautoSearch, autoSearch,
					   NULL);
    outputWidget = XtVaCreateManagedWidget("outputWidget",	    /* Enable video output (as a side effect, video... */
					   xVideoOutWidgetClass,    /* ...output will be turned on immediately). */
					   outputShell,
					   NULL);
    XtVaSetValues(outputWidget, XtNchannel, PLX_IO_OFF, NULL);	    /* Turn off generation of a video signal */
    XtVaSetValues(videoOutToggle, XmNsensitive, True, NULL);	    /* Allow output to be used */
    XtVaSetValues(outputConfigButton, XmNsensitive, True, NULL);    /* Allow the output configuration panel to be used */
    XtVaSetValues(input2ChannelButton, XmNsensitive, True, NULL);   /* Allow 2nd input settings to be used */
  }
  if ((XPlxFindMatchingSignal(hardwareInfo,			    /* Look for an RGB signal */
			     PLX_INPUT_0,
			     PLX_NTSC,
			     PLX_RGB,
			     0) == False) &&
      (XPlxFindMatchingSignal(hardwareInfo,
			      PLX_INPUT_1,
			      PLX_NTSC,
			      PLX_RGB,
			      0) == False))
  {
    XtVaSetValues(rgbCascade, XmNsensitive, False, NULL);	    /* Turn off the appropriate interface component */
  }
  if (XPlxFindMatchingSignal(hardwareInfo,			    /* Look for a 3rd composite input (genlock, most likely) */
			     PLX_INPUT_0,
			     PLX_NTSC,
			     PLX_COMP,
			     2) == False)
  {
    XtVaSetValues(compositeInstanceButton3,			    /* Turn off the appropriate interface component */
		  XmNsensitive, False,
		  NULL);
  }
  if (XPlxFindMatchingSignal(hardwareInfo,			    /* Look for an S-VHS input (likely on 2nd digitizer) */
			     PLX_INPUT_1,
			     PLX_NTSC,
			     PLX_YC,
			     0) == False)
  {
    XtVaSetValues(ycCascade, XmNsensitive, False, NULL);	    /* Turn off the appropriate interface component */
  }
  return;
} /* end function BuildInputMenu */



void PrintErrorMessage(char*	message)
{
  XmString		messageString;
  
  messageString = XmStringCreateLocalized(message);
  XtVaSetValues(alertShellMessageLabel, XmNlabelString, messageString, NULL);
  XmStringFree(messageString);
  if (XtIsManaged(alertWidget) == False)
  {
    XtManageChild(alertWidget);
  }
} /* end function PrintErrorMessage */
