/*
 *   Copyright (c) 1990 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Murali. V. Srinivasan
 */


#ifndef lint
static char *sid_ccomp = "@(#)ccomp.c	1.14 9/7/90 	Parallax Graphics Inc";
#endif

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <XPlxExt.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include <Xm/BulletinB.h>

/* Menus */
#include <Xm/RowColumn.h>
#include <Xm/PushBG.h>
#include <Xm/Scale.h>


XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display);


static Widget SourcePopup, swidget, dwidget, slwidget, form, capt, val;
static Widget decompressionPopup, sliderPopup;
static Atom exit_atom;

int videowidth = 320;
int videoheight = 240;
static int sourcewidth, sourceheight, sourcestart;

static Arg args[25];
static unsigned int n;

static GC gc, decompressionGC;
static Display *display;
static Window sourceWindow, destinationWindow;
static int first=1, first1 = 1;
static int usingSharedMemory;
static unsigned char *qTable;
static XPlxCImage *shmimage = NULL;
static XShmSegmentInfo shminfo;
static short QFactor;
static int holdit = 1 ;

/*
 * To clean up when program exit.
 */
static void
exit_func(sig, code, scp)
    int         code;
    int         sig;
    struct sigcontext *scp;
{
    if (usingSharedMemory)
    {
	XShmDetach(display, &shminfo);
	shmdt(shminfo.shmid);
	shmctl(shminfo.shmid, IPC_RMID, 0);
    }
    exit(0);
}

static void ehandler(w, cldata, event)
Widget w;
caddr_t cldata;
XClientMessageEvent *event;

{
    int data = (int) cldata;

    if (event->type == ClientMessage && (w == swidget || w == dwidget))
    {
	if (event->data.l[0] == exit_atom)
	    exit_func();
    }
}

static void 
dragcall(w, data, call_data)
Widget w;
caddr_t data, call_data;
{
    int size;
    int i = (int)data;

    XmScaleCallbackStruct *p = (XmScaleCallbackStruct *)call_data;

    QFactor = p->value;
    size = MakeQTables(QFactor, &qTable);
    XPlxPutTable(display, sourceWindow, gc, qTable, size, 0);
    XPlxPutTable(display, destinationWindow, decompressionGC, qTable, size, 1);
}

/*
 * this function figures the size of a window since sometimes the
 * the xt functions don't work properly.
 */
static void get_size(display, win, w, h, bw)
Display *display;
Window win;
int *w, *h, *bw;
{
    XWindowAttributes attrib;

    XGetWindowAttributes(display, win, &attrib);
    *w = attrib.width;
    *h = attrib.height;
    *bw = attrib.border_width;
}

/*
 *    function to make the destination window the same size as the source
 *    when the source size changes.
 */

static int flag = 0;
static void configit(w, data, event)
Widget w;
caddr_t data;
XEvent *event;
{
    if ((first == 0) || flag)
    {
	int bw;
	flag = 1;
	get_size(display, sourceWindow, &videowidth, &videoheight, &bw);
	if (((videowidth * 3) / 4) > ((videoheight * 4) / 3))
	    videoheight = (videowidth * 3) / 4;
	else
	    videowidth = (videoheight * 4) / 3;

	XtResizeWidget(SourcePopup, videowidth, videoheight, bw);
	XtResizeWidget(decompressionPopup, videowidth, videoheight, bw);
	XtResizeWidget(swidget, videowidth, videoheight, bw);
	XtResizeWidget(dwidget, videowidth, videoheight, bw);
    }
}

/*
 * Since we tag the windows such that the tag is not drawn
 * we need to tag as video so the next video/decompress
 * will set the tags properly
 */

static void exposit(w, data, event)
Widget w;
caddr_t data;
XExposeEvent *event;
{
    if (w == swidget)
	first = 1;
    else
	first1 = 1;
}

static int loopflag = 0;

/*
 *    Start things going on first button press
 */

static void buttonp(w, data, event)
Widget w;
caddr_t data;
XButtonEvent *event;
{
    holdit = 0 ;
}

/*
 *    Main work process.
 *    capture a video image, compress it, then decompress it
 */
Boolean worker_shm(XtPointer data)
{
    if (holdit) return FALSE; /* wait untill button is pressed */

    if (first)
	XPlxVideoTag(display, sourceWindow, gc, PLX_VIDEO);
    if (first1)
	XPlxVideoTag(display, destinationWindow, decompressionGC, PLX_VIDEO);

    /* capture a video frame into the source */
    XPlxVideoSqueezeStill(display, sourceWindow, gc, 0, sourcestart, sourcewidth,
			sourceheight, 0, 0, videowidth, videoheight);

    /* Get a compressed image */
    shmimage->width = videowidth;
    shmimage->height = videoheight;

    XPlxShmGetCImage(display, sourceWindow, gc, shmimage, 0, 0,
		     videowidth, videoheight);

    /* decompress the image into the destination window */
    XPlxShmPutCImage(display, destinationWindow, decompressionGC, shmimage, 0, 0, videowidth, videoheight,
			0, 0, videowidth, videoheight, 0, 0);

    /*
     * set the tag such that subsequent video/decompress will not
     * set the tag bits, saves time.
     */
    if (first) /* only do this when necessary */
    {
	XPlxVideoTag(display, sourceWindow, gc, PLX_VIDEO_OVR);
	first = 0;
    }
    if (first1) /* only do this when necessary */
    {
	XPlxVideoTag(display, destinationWindow, decompressionGC, PLX_VIDEO_OVR);
	first1 = 0;
    }
    return(FALSE); /* work processes must return FALSE */
}

Boolean worker_nonshm(XtPointer data)
{
    XPlxCImage *image;
    int count;

    if (holdit) return FALSE; /* wait untill button is pressed */

    if (first)
	XPlxVideoTag(display, sourceWindow, gc, PLX_VIDEO);
    if (first1)
	XPlxVideoTag(display, destinationWindow, decompressionGC, PLX_VIDEO);

    /* capture a video frame into the source */
    XPlxVideoSqueezeStill(display, sourceWindow, gc, 0, sourcestart, sourcewidth,
			sourceheight, 0, 0, videowidth, videoheight);

    /* Get a compressed image */
    image = (XPlxCImage *) XPlxGetCImage(display, sourceWindow, gc, 0, 0,
			videowidth, videoheight, videowidth, videoheight);

    /* decompress the image into the destination window */
    XPlxPutCImage(display, destinationWindow, decompressionGC, image, 0, 0, videowidth, videoheight,
			0, 0,  videowidth, videoheight, 1);

    /* free up storage */
    XPlxDestroyCImage(image);

    /*
     * set the tag such that subsequent video/decompress will not
     * set the tag bits, saves time.
     */
    if (first) /* only do this when necessary */
    {
	XPlxVideoTag(display, sourceWindow, gc, PLX_VIDEO_OVR);
	first = 0;
    }
    if (first1) /* only do this when necessary */
    {
	XPlxVideoTag(display, destinationWindow, decompressionGC, PLX_VIDEO_OVR);
	first1 = 0;
    }
    return(FALSE); /* work processes must return FALSE */
}

static void printUsage(c)
char *c;
{
    printf("usage: %s [-n]\n", c);
    exit(1);
}

void main(argc, argv)
int argc;
char **argv;
{
    XtAppContext app_context;
    plx_IO *plxio;
    plx_IO_list *iolist;
    plx_signal_list *siglist;
    plx_signal *sig;
    int i, tmp, size, badsync;
    char *name1 = ":0.0";
    char *name2 = "unix:0.0";
    char *msg = "Fail to create shared memory, it is running on non-shared memory\n";
    static char *filename = "/tmp/theimage";
    static int mask = ButtonPressMask | ExposureMask;
    static int mask1 = StructureNotifyMask;
    int screen;
    XtWorkProc	worker = worker_nonshm;
    Colormap colormap;
    XVisualInfo * visualInfo;
    int copy_of_argc;
    char ** copy_of_argv;
    int argv_size;

    usingSharedMemory = True;
    QFactor = 100; /* default valuse */

    /*
     * install signal handler for kill and crahses.
     */
    signal (SIGINT, exit_func);
    signal (SIGSEGV, exit_func);
    signal (SIGBUS, exit_func);

    /*
     * The -n option is for non-shared memory.
     */
    for (i = 1; i < argc; ++i)
    {
	if(strcmp("-n", argv[i]) == 0)
	    usingSharedMemory = False;
	else
	    printUsage(argv[0]);
    }

    /*
     * make the q tables. if you change the qfactor you must call
     * MakeQtables
     */
    size = MakeQTables(QFactor, &qTable);

    XtToolkitInitialize();
    app_context = XtCreateApplicationContext();

    copy_of_argc = argc;
    argv_size = argc * (sizeof(char*));
    copy_of_argv = (char**) XtMalloc(argv_size);
    memcpy(copy_of_argv, argv, argv_size);

    display = XtOpenDisplay(app_context,
			    NULL,
			    NULL,
			    "scomp",
			    (XrmOptionDescList) NULL,
			    0,
			    &argc, argv);

    if((Display *) NULL == display)
      {
      printf("Unable to open local display check permissionts\n");
      exit(1);
      }

    screen = DefaultScreen(display);

    visualInfo = GetTrueColorVisual(screen, display);
    if((XVisualInfo *) NULL == visualInfo)
      {
      XtWarning("Could not find the TrueColor visual being requested.");
      exit(-1);
      }

    colormap = XCreateColormap(display,
			       XRootWindow(display, screen),
			       visualInfo->visual,
			       AllocNone);

    n = 0;
    XtSetArg(args[n], XmNvisual, visualInfo->visual); n++;
    XtSetArg(args[n], XmNdepth, visualInfo->depth); n++;
    XtSetArg(args[n], XmNcolormap, colormap); n++;
    XtSetArg(args[n], XmNargc, copy_of_argc); n++;
    XtSetArg(args[n], XmNargv, copy_of_argv); n++;

    SourcePopup = XtAppCreateShell("scomp",
			   "scomp",
			   applicationShellWidgetClass,
			   display,
			   args, n);

    n = 0;
    XtSetArg(args[n], XmNvisual, visualInfo->visual); n++;
    XtSetArg(args[n], XmNdepth, visualInfo->depth); n++;
    XtSetArg(args[n], XmNcolormap, colormap); n++;

    decompressionPopup = XtAppCreateShell("Decompressed",
				  "Compress",
				  applicationShellWidgetClass, 
				  display, 
				  args, n);

    n = 0;
    XtSetArg(args[n], XtNwidth, videowidth); n++;
    XtSetArg(args[n], XtNheight, 50); n++;
    sliderPopup = XtAppCreateShell("Q Factor","Qfactor",
			applicationShellWidgetClass, XtDisplay(SourcePopup), args, n);

    n = 0;
    XtSetArg(args[n], XtNborderWidth, 0); n++;
    XtSetArg(args[n], XtNwidth, videowidth); n++;

    form = XtCreateManagedWidget("QFactorAdjust", xmFormWidgetClass,
                                     sliderPopup, args, n);

    n = 0;

    XtSetArg(args[n], XmNborderWidth, 0); n ++;
    XtSetArg(args[n], XmNwidth, videowidth); n ++;
    XtSetArg(args[n], XmNheight, videoheight); n++;

    swidget = XtCreateManagedWidget("src", xmBulletinBoardWidgetClass, SourcePopup,
			args, n);
    dwidget = XtCreateManagedWidget("dest", xmBulletinBoardWidgetClass,
			decompressionPopup, args, n);

    n = 0;
    XtSetArg(args[n], XmNx, 20); n++;
    XtSetArg(args[n], XmNwidth, 255); n++;
    XtSetArg(args[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg(args[n], XmNprocessingDirection, XmMAX_ON_RIGHT); n++;
    XtSetArg(args[n], XmNhighlightOnEnter, TRUE); n++;
    XtSetArg(args[n], XmNscaleWidth, 250); n++;
    XtSetArg(args[n], XmNmaximum, 1000); n++;
    XtSetArg(args[n], XmNminimum, 25); n++;
    XtSetArg(args[n], XmNvalue, 100); n++;
    XtSetArg(args[n], XmNshowValue, TRUE); n++;
    XtSetArg(args[n], XmNborderWidth, 0); n++;
    XtSetArg(args[n], XmNtitleString,
     		XmStringCreate("Qfactor", XmSTRING_DEFAULT_CHARSET)); n++;
/*
    slwidget = XmCreateScale (form, "value", args, n);
*/
    slwidget = XtCreateManagedWidget("qfactor", xmScaleWidgetClass,
                                        form, args, n);

    XtAddCallback(slwidget, XmNdragCallback, dragcall, 0);
    XtManageChild(slwidget);

    XtRealizeWidget(SourcePopup);
    XtRealizeWidget(decompressionPopup);
    XtRealizeWidget(sliderPopup);

    display = XtDisplay(swidget);
    sourceWindow = XtWindow(swidget);
    destinationWindow = XtWindow(dwidget);
    gc = XCreateGC(display, sourceWindow, 0, 0);
    decompressionGC = XCreateGC(display, destinationWindow, 0, 0);
    XSetForeground(display, gc, 0);
    XSetForeground(display, decompressionGC, 0);

    /* store qtables for compress and decompress in the GCs */
    XPlxPutTable(display, sourceWindow, gc, qTable, size, 0);
    XPlxPutTable(display, destinationWindow, decompressionGC, qTable, size, 1);

    if (usingSharedMemory &&
	((strncmp (name1, display->display_name, strlen(name1)) == 0) ||
	(strncmp (name2, display->display_name, strlen(name2)) == 0)))
    {
	tmp = XShmGetEventBase(display) + ShmCompletion;
	shminfo.shmid = shmget(IPC_PRIVATE, 256 * 1024, IPC_CREAT|0777);
	if (shminfo.shmid < 0)
	{
	    perror(msg);
	    usingSharedMemory = False;
	}
	else
	{
	    shminfo.shmaddr = (char *)shmat(shminfo.shmid, 0, 0);
	    if (shminfo.shmaddr == ((char *)-1))
	    {
		perror(msg);
		usingSharedMemory = False;
	    }
	    else
	    {
		shmimage = (XPlxCImage *)XPlxShmCreateCImage(display, shminfo.shmaddr,
						&shminfo, 0, 0, 0);
		shminfo.readOnly = False;
		XShmAttach(display, &shminfo);
		worker = worker_shm;
	    }
	}
    }

    XtAddEventHandler(swidget, ButtonPressMask, False, buttonp, NULL);
    XtAddEventHandler(dwidget, ButtonPressMask, False, buttonp, NULL);
    XtAddEventHandler(swidget, StructureNotifyMask, False, configit, NULL);
    XtAddEventHandler(swidget, ExposureMask, False, exposit, NULL);
    XtAddEventHandler(dwidget, ExposureMask, False, exposit, NULL);

    XtAddEventHandler(swidget, 0, False, ehandler, NULL);
    XtAddEventHandler(dwidget, 0, False, ehandler, NULL);
    XtAddEventHandler(slwidget, 0, False, ehandler, NULL);

    XtAppAddWorkProc(app_context, worker, NULL);
    exit_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);

    XSetWMProtocols(display, sourceWindow, &exit_atom, 1);
    XSetWMProtocols(display, destinationWindow, &exit_atom, 1);

    /*
     * query the server to get hardware/signal information.
     */
    plxio = (plx_IO *) XPlxQueryConfig(display, sourceWindow, gc);

    /* if the hardware type is not XVideo, can't work */
    if (plxio->hardware != PLX_XVIDEO_24)
    {
	printf("%s can only run on Parallax XVideo hardware\n", argv[0]);
	exit(1);
    }

    /* find a pointer to channel one input */
    for (iolist = plxio->inputs; iolist && (iolist->channel != PLX_INPUT_0);
	 iolist = iolist->next_IO); /* has to work, can't fail */

    /* go through the list of signals and find an active signal */
    badsync = 1;

    for (siglist = iolist->signal_list; siglist;
	siglist = siglist->next_signal)
    {
	/*
	 * do an input select
	 * we are only using input 0 so it, and PLX_COMP are hard coded
	 * we also know that all the XVideo hardware can do is PLX_RGB24
	 */
	XPlxVideoInputSelect(display, sourceWindow, gc, PLX_INPUT_0,
				 siglist->signal->standard,
				 PLX_COMP, PLX_RGB24);

	/* now do a query video to find out if there is sync */
	sig = (plx_signal *) XPlxQueryVideo(display, sourceWindow, gc);
	if (sig->sync_ok)
	{
	    badsync = 0;
	    break;
	}
    }

    if (badsync)
    {
	printf("no video connected to input 0\n");
	exit(1);
    }

    sourcewidth = sig->w;
    tmp = (sig->w * 3) / 4;
    sourceheight = tmp;
    tmp = sig->h - tmp;
    sourcestart = tmp + 1;

    XtAppMainLoop(app_context);
}



XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display)
{
  int			i;
  long			visualInfoMask;
  int			visualsMatched;
  XVisualInfo		visualHints;
  static XVisualInfo*	visualList = (XVisualInfo*)NULL;
  XVisualInfo*		theVisual;
  
  if (visualList != (XVisualInfo*)NULL)				    /* Was data saved by a previous call to this function? */
    XFree((void*)visualList);					    /* Yes, release the data since it is no longer needed */
  visualHints.class = TrueColor;				    /* Set up hints to determine the type of visual needed... */
  visualHints.screen = screenNumber;				    /* ...to create a window that can display video */
#ifdef __hpux
  visualInfoMask = (VisualScreenMask);				    /* For the HP product, don't actually need a TrueColor class */
#else
  visualInfoMask = (VisualClassMask | VisualScreenMask);	    /* For Sun, need to get real TrueColor class */
#endif
  visualList = XGetVisualInfo(display,				    /* Try to find a visual that matches what is needed for... */
			      visualInfoMask,			    /* ... a window that will display video or TrueColor... */
			      &visualHints,			    /* ...graphics. */
			      &visualsMatched);
  if (visualsMatched == 0)					    /* Did the search for a TrueColor visual fail? */
    return((XVisualInfo*)NULL);					    /* Yes, return a NULL structure to represent failure */
  for (theVisual = visualList, i = 0;				    /* Look at all the visuals returned by XGetVisualInfo... */
       i < visualsMatched;					    /* ...to find the best match.  The idea match would be... */
       i++, theVisual++)					    /* ...a 24-bit depth Visual. */
  {
    if (theVisual->depth == 24)					    /* Was an ideal match found? */
      return(theVisual);					    /* Yes, return this Visual */
  }
  for (theVisual = visualList, i = 0;				    /* If this statement is reached, no 24-bit TrueColor... */
       i < visualsMatched;					    /* ...Visual was found; in this case, fallback and try... */
       i++, theVisual++)					    /* ...to find the next best thing: any TrueColor visual... */
  {								    /* ...of depth >= 24 bits (probably a 32-bit is what will... */
    if (theVisual->depth >= 24)					    /* ...be found). */
      return(theVisual);					    /* Was a good fallback Visual found?  If so, return it. */
  }
  return(visualList);						    /* If this statement was reached, no good fallback was... */
								    /* ...found.  In this case, simply return the first... */
								    /* ...TrueColor visual found. */
}								    /* end function GetTrueColorVisual */



