/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */
/*   $Log: makemovie.c,v $
 * Revision 0.92  1994/10/22  04:40:12  drapeau
 * Made the code more ANSI-compatible.
 *
 * Revision 0.91  1994/09/14  02:04:57  drapeau
 * Checking into the master tree changes made by Paul Lovvik.
 *
 * Revision 0.91  1994/09/13  21:23:00  lovvik
 * Fixed the parameters-with-no-filename problem.
 *
 * Revision 0.90  1994/08/10  16:57:02  lovvik
 * Synchronization added.
 * User options implemented.
 * Window title added.
 * */
/*   $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/makemovie.c,v 0.92 1994/10/22 04:40:12 drapeau Exp $ */
static char rcsid[]="$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/makemovie.c,v 0.92 1994/10/22 04:40:12 drapeau Exp $";
/*
	Program:	makemovie
	Usage:		makemovie [options] moviefile
                           [options]:
                                  -fFrames Per Second
                                  -wWidth (in pixels)  Height is automatic
                                  -sSeconds to record

        Example:        makemovie -f25 -w320 -s15

                        This will record at 25 frames per second with a width of 320 pixels
                        (height is automatic), for a length of 15 seconds.

	Description:	This demo illustrate the usage of XVideo Movie API to make a JPEG movie
			from a live source.  It captures and compresses 30 frames of video and
			puts it in the JPEG movie format.  Audio is not implemented in this demo.
			Although libPlxApi.a is xview dependent, the movie API used here does
			not depend on xview.  Thus, you can use other toolkits for your user's
			interface along with the following functions.
*/

#include <stdio.h> 
#include <math.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <XPlxExt.h>
#include <malloc.h>
#include <fcntl.h>
#include <multimedia/audio_hdr.h>
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <vaa.h>
#include <audio.h>
#include <jpeg.h>
#include <stdlib.h>
#include <sys/time.h>                           /* required for gettimeofday */
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>


/* function prototypes */

void print_help(void);
XVisualInfo* GetTrueColorVisual();
static void MyUSleep();
long ElapsedTime();
int timefornextframe();

main(argc,argv)
int argc;
char **argv;
{
    int depth;                                          
    Display *disp;
    Window win;
    int screen, width=640, height=480, i=1, 
        winMask;					/* Window attribute mask. 		*/
    XPlxCImage *cImage;
    GC gc;
    static unsigned char *qTable;			/* Q Table to load to CCube chip 	*/
    static int qTableSize;				/* Size of Q Table. 			*/
    XVisualInfo *vInfo;
    XSetWindowAttributes attr;
    static int mask=ExposureMask;			/* Window event mask.			*/
    XEvent winEvent;
    plx_signal *plxSignal=NULL;				/* Video signal structure. 		*/

    int playbackRate=20;				/* Movie playback rate, default to 20 fps, */
							/* this is conservative.              	*/
    int QFactor=100;
    char jpegFile[30]; 					/* Movie file name. 			*/
    int end = 30;						/* Number of frames to capture.		*/
    int seconds = 10;                                   /* Default to 10 seconds of recording   */
    long int us_per_frame;                              /* micro seconds per frame */
    long int ms_per_frame;
    int frame = 0;
    struct timeval frametime, starttime, now;
    char *window_name="makemovie";
    XTextProperty windowname;

    if (argc<2)                                         /* Forgot to specify a filename?        */
    {                                                   /* Print help message                   */
      print_help();
      exit(1);
    }
    i = 1;
    while(i < argc && argv[i][0] == '-')               /* Decode option list (options begin with */
                                                        /* a '-'                                  */
    {
      switch(argv[i][1])
      {
        case 'f':  /* set frames per second */
        {
          playbackRate = atoi(argv[i]+2);
          printf("Record at %d fps.\n", playbackRate);
        }  /* end of case 'f' */
        break;
        case 's':   /* set seconds to record */
	{
          seconds = atoi(argv[i]+2);
          printf("Recording %d seconds.\n", seconds);
        }  /* end of case 's' */
        break;
      case 'w':     /* set the width (height is automatic) */
	{
          width = atoi(argv[i]+2);
          height = (int)((double)(480.0/640.0) * (double) width);
          printf("width = %d, height = %d (pixels).\n", width, height);
	}  /* end of case 'w' */
        break;
      }   /* end of switch statement */
      i++;   /* increment the argument pointer */
    }     /* end of while */

    if(argv[argc - 1][0] == '-')      /* The last parameter should not have a '-' */
    {
      printf("parameter error\n");
      printf("\n\n");
      print_help();
      return;
    } 

    strcpy(jpegFile, argv[i]);

    us_per_frame = (long int)((1.0 / (float)playbackRate) * 1000000);
    ms_per_frame = us_per_frame / 1000;
    disp=XOpenDisplay(getenv("DISPLAY"));
    screen=(int)XDefaultScreen(disp);

    vInfo = GetTrueColorVisual(screen, disp);
    depth = vInfo->depth;
    winMask=CWBackPixel|CWColormap|CWBorderPixel;	/* Setup window attribute mask.		*/
    attr.colormap=XCreateColormap(disp, 		/* Create colormap according to visual.	*/
	XRootWindow(disp,screen),
	vInfo->visual,AllocNone);
    attr.background_pixel=BlackPixel(disp,screen);	/* Set background color to black. 	*/
    attr.border_pixel=WhitePixel(disp,screen);		/* Set boarder color to white. 		*/

 
    win=XCreateWindow(disp,RootWindow(disp,screen),	/* Create window for frame grab.	*/
	0,0,width,height,0,depth,InputOutput,		/* Window size is set to 640 x 480.	*/ 
	vInfo->visual,winMask,&attr);

    XStringListToTextProperty(&window_name, 1, &windowname);
    XSetWMProperties(disp, win, &windowname, &windowname, argv, argc, NULL, NULL, NULL);

    gc=XCreateGC(disp, win, 0, 0);			/* Create window gc.			*/
    qTableSize=MakeQTables(QFactor, &qTable);		/* Create Q Table according to QFactor.	*/
    XPlxPutTable(disp,win,gc,qTable,qTableSize,0);	/* Load QTable to chip for compression.	*/

    XPlxVideoInputSelect(disp,win,gc,			
	PLX_INPUT_0, PLX_NTSC, PLX_COMP, 
	PLX_RGB24);					/* Select video input type & channel.	*/

    plxSignal=(plx_signal*)XPlxQueryVideo(disp,		/* Detect for video signal at input. 	*/
	win,gc);
    while (!plxSignal->sync_ok) 
	{
        printf("Sync Absent - Hook up an input source\n");
        MyUSleep(500000);
        plxSignal=(plx_signal*)XPlxQueryVideo(disp, win, gc);
    	}

    XPlxVideoTag(disp,win,gc,PLX_VIDEO);		/* Setup up window tag to display video	*/
    XSelectInput(disp,win,mask);			/* Setup up window event mask.		*/
    XMapWindow(disp,win);				/* Map window to display. 		*/

    XNextEvent(disp,&winEvent);

    jpeg_open_movie(disp,jpegFile,playbackRate, 
	QFactor, NULL);					/* Initialize and open JPEG movie file.	*/

    XPlxVideoSqueezeLive(disp,win,gc,0,plxSignal->b, 
	   640, 480,  0,0,width,height);		/* Display live video.			*/

  gettimeofday(&frametime, NULL);
  gettimeofday(&starttime, NULL);
  while(1)
  {
    gettimeofday(&now, NULL);
    MyUSleep(us_per_frame - (ElapsedTime(frametime, now) * 1000));
    gettimeofday(&frametime, NULL);
    cImage=XPlxGetCImage(disp,win,
  	 gc,0,0, width, height, width, height);			    /* Grab and compress image from window.	*/
    jpeg_add_still(disp,(JPEGImage*)cImage);			    /* add still frame to movie file 	*/
    jpeg_end_frame(disp);
    frame++;
    
  
    if(ElapsedTime(starttime, frametime)/1000 >= seconds)
    {
      XPlxVideoStop(disp, win, gc);                     /* Stop the live video.                 */
      XPlxDestroyCImage(cImage);
      jpeg_stop_movie(disp);				/* End of making this movie. 		*/
      XFreeColormap(disp,attr.colormap);		/* Free resources when done. 		*/
      XFreeGC(disp,gc);
      XDestroyWindow(disp,win);
      XFlush(disp);
      printf("\nRecorded %d frames.\n\n", frame);
      exit(0);
    }
  } 
}


void print_help(void)
{
  
  printf("usage: makemovie [-options] moviefile\n");
  printf("    -options:\n");
  printf("         -fFrames Per Second\n");
  printf("         -wWidth (Height is automatic)\n");
  printf("         -sSeconds\n");

  return;
}

long ElapsedTime(before, after)
     struct timeval* before;
     struct timeval* after;
{
  return(((after->tv_sec - before->tv_sec) * 1000) +        /* Convert the seconds part to milliseconds */
         ((after->tv_usec - before->tv_usec) /1000));       /* Convert the microseconds part to milliseconds */
} /* end function ElapsedTime */

int timefornextframe(before, after, uspf)
    struct timeval* before;
    struct timeval* after;
    long uspf;
{
  return(((after->tv_sec - before->tv_sec) * 1000000 +
        (after->tv_usec - before->tv_usec)) >= uspf);
}

static void MyUSleep(microseconds)
unsigned int microseconds;
{
  int			descriptorTableSize = 0;
  int			result;
  struct timeval	waitTime;				    /* Used to tell select() how long to wait until it returns */
  
  descriptorTableSize = sysconf(_SC_OPEN_MAX);			    /* Get size of descriptor table for later use by select() */
  waitTime.tv_sec = 0;						    /* Set up a timeval to wait for the time passed in as... */
  waitTime.tv_usec = microseconds;				    /* ...argument. */
  result = select(descriptorTableSize,				    /* Use select to sleep by not asking to be informed of... */
		      (fd_set*)0, (fd_set*)0,(fd_set*)0,	    /* ...input on any file descriptors, and filling in the... */
		      &waitTime);				    /* ...timeval for the wait period. */
  return;
} /* end function MyUSleep */

XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display)
{
  int			i;
  int			visualsMatched;
  XVisualInfo		visualHints;
  static XVisualInfo*	visualList = (XVisualInfo*)NULL;
  XVisualInfo*		theVisual;
  
  if (visualList != (XVisualInfo*)NULL)				    /* Was data saved by a previous call to this function? */
    XFree((void*)visualList);					    /* Yes, release the data since it is no longer needed */
  visualHints.class = TrueColor;				    /* Set up hints to determine the type of visual needed... */
  visualHints.screen = screenNumber;				    /* ...to create a window that can display video */
  visualList = XGetVisualInfo(display,				    /* Try to find a visual that matches what is needed for... */
			      VisualClassMask |			    /* ...a window that will display video or TrueColor.... */
			      VisualScreenMask,			    /* ...graphics. */
			      &visualHints,
			      &visualsMatched);
  if (visualsMatched == 0)					    /* Did the search for a TrueColor visual fail? */
    return((XVisualInfo*)NULL);					    /* Yes, return a NULL structure to represent failure */
  for (theVisual = visualList, i = 0;				    /* Look at all the visuals returned by XGetVisualInfo... */
       i < visualsMatched;					    /* ...to find the best match.  The idea match would be... */
       i++, theVisual++)					    /* ...a 24-bit depth Visual. */
  {
    if (theVisual->depth == 24)					    /* Was an ideal match found? */
      return(theVisual);					    /* Yes, return this Visual */
  }
  for (theVisual = visualList, i = 0;				    /* If this statement is reached, no 24-bit TrueColor... */
       i < visualsMatched;					    /* ...Visual was found; in this case, fallback and try... */
       i++, theVisual++)					    /* ...to find the next best thing: any TrueColor visual... */
  {								    /* ...of depth >= 24 bits (probably a 32-bit is what will... */
    if (theVisual->depth >= 24)					    /* ...be found). */
      return(theVisual);					    /* Was a good fallback Visual found?  If so, return it. */
  }
  return(visualList);						    /* If this statement was reached, no good fallback was... */
								    /* ...found.  In this case, simply return the first... */
								    /* ...TrueColor visual found. */
}								    /* end function GetTrueColorVisual */
