/*
 *   Copyright (c) 1994 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: Paul Lovvik
 */

/*   $Log: playmovie.c,v $
 * Revision 0.91  1994/10/22  04:40:23  drapeau
 * Made the code more ANSI-compatible.
 *
 * Revision 0.90  1994/08/10  17:42:22  lovvik
 * Synchronization added.
 * Window Title added.
 * XFlush removed.
 * */
/*   $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/playmovie.c,v 0.91 1994/10/22 04:40:23 drapeau Exp $ */
static char rscid[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/playmovie.c,v 0.91 1994/10/22 04:40:23 drapeau Exp $";
 /*
	 Program:	playback
	 Usage:		playback jpeg_movie_file
	 Description:	This demo illustrates the usage of XPlxPutCImage() to
			 display JPEG movie images.  Parallax JPEG movie file
			 has the following format:

			 ---------------------------------------------
			 |   jpheader + filler = sizeof(jpheader)    |   jpheader defined in movie.h
			 |-------------------------------------------|
			 |   frame #1 starts at offset[0]            |
			 |-------------------------------------------|
			 |   frame #2 starts at offset[1]            |
			 |-------------------------------------------|
			 :                      :                    :
			 :                      :                    :
			 |-------------------------------------------|
			 |   frame #N starts at offset[N-1]          |
			 |-------------------------------------------|
			 |   offset[0 to (N-1)] = N * sizeof(int)    |
			 |   starts at jpheader.indexbuf             |
			 ---------------------------------------------

			 Each frame can be of video only or video+audio:

			 AUDIO:	--------------------------------------------
				 | LOAD_AUDIO instr |       audio data      |
				 --------------------------------------------
				      1 byte          size defined in jpheader.audioslice

			 VIDEO:  ---------------------------------------------------------
				 | LOAD_JPEG instr | frame data size |  XPlxCImage data  |
				 ---------------------------------------------------------
				       1 byte          sizeof(int)      frame data size

			 Instructions (instr) are defined in movie.h.

 */

 #include <stdio.h>
 #include <malloc.h>
 #include <X11/Xlib.h>
 #include <XPlxExt.h>
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <sys/time.h>                           /* required for gettimeofday */
 #include <sys/types.h>
 #include <sys/param.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <multimedia/libaudio.h>		/* required for movie.h      */ 
 #include "movie.h"


 long ElapsedTime();

 static void MyUSleep();

 XVisualInfo* GetTrueColorVisual();

 void main(argc, argv)
 int argc;
 char **argv;
 {
   Window win;				/* movie playback window */
   Display *disp;
   char *window_name = "playmovie";
   XTextProperty windowname;             /* used to set up the window name */
   int scr, winMask;			/* scr= screen, winMask= valuemask of window attributes */
   GC gc;
   XEvent event;
   FILE *jpegFile;			/* JPEG movie file to play back                         */
   int width, height, depth;		/* movie window width, height, and depth                */
   jpheader header;			/* JPEG movie header sturcture                          */
  char instruction;			/* movie file instruction byte                          */
  int *indexBuffer;			/* frame offset index buffer                            */
  int currentFrame;			/* current movie frame                                  */
  int qTableSize;			/* Q table size                                         */
  unsigned char*	qTable;         /* decompression Q table                                */
  int cImageSize, maximumSize;		/* compressed image size & maximum image size           */
  XPlxCImage *cImage;			/* compressed image sturcture                           */
  char *cImageData;			/* compressed image data                                */
  XVisualInfo *visualInfo;	       	/* visual info of movie window                          */
  XSetWindowAttributes winAttributes;	/* window attrubutes of movie window                    */
  char *audioslice=NULL;
  int devaudio;
  int first=1;				/* flag to keep track of first XPlxPutCImage            */
  int FramesPerSecond;                  /* number of frames per second                          */
  long int USecondsPerFrame;            /* number of micro seconds per frame                    */
  struct timeval before, after;         /* variable to calculate event-loop time                */

  if (argc<2)			        /* requires movie filename as argument                  */
  {
    printf("usage: playmovie moviefile\n");
    exit(1);
  }
  jpegFile=fopen(argv[1],"r");

  if (jpegFile==NULL)
  {
    printf("Can't open input file\n");
    exit(1);
  }
  fread((char*)&header,sizeof(jpheader),1,jpegFile);  /* read in jpheader info of movie         */

  FramesPerSecond = header.fps;                       /* Get frames per second                  */
  USecondsPerFrame = (long int)(1 / (float)(FramesPerSecond * 0.000001));

  if (header.tracks>0)				      /* check whether there's audio recorded in movie */
  {
    audioslice=(char*)malloc(header.audioslice);      /* initialize audio device                */
    devaudio=open("/dev/audio",O_WRONLY,0);
    if (devaudio<0)
      printf("Can't open /dev/audio\n");
  }

  width=header.width;					/* set window width to that of movie    */
  height=header.height;				        /* set window height to that of movie   */
  fseek(jpegFile,header.indexbuf,0);			/* skip file to read frame offset locations */
  indexBuffer=(int*)malloc(header.frames*sizeof(int));	/* create index buffer according to number of movie frames */
  fread((char*)indexBuffer,sizeof(int),header.frames,jpegFile);	
							/* read in frame offsets to buffer      */
  currentFrame=0;					/* frame index starts at 0              */

  disp=XOpenDisplay("");
  scr=(int)XDefaultScreen(disp);
  visualInfo = GetTrueColorVisual(scr, disp);           /* Get a proper Visual for using 24-bit video in a window */
  depth = visualInfo->depth;
 
  winMask=CWBackPixel|CWColormap|CWBorderPixel;
  winAttributes.colormap=XCreateColormap(disp,
        XRootWindow(disp, scr),
        visualInfo->visual,AllocNone);
  winAttributes.background_pixel=BlackPixel(disp,scr);
  winAttributes.border_pixel=WhitePixel(disp,scr);
 
  win=XCreateWindow(disp,XRootWindow(disp,scr),
        0,0,width,height,				/* x, y, width and height               */
        0, 						/* border width */
        depth, 						/* window depth */
        InputOutput,visualInfo->visual,
        winMask,&winAttributes);

           /* Set up the window title */
  XStringListToTextProperty(&window_name, 1, &windowname);
  XSetWMProperties(disp, win, &windowname, &windowname, argv, argc, NULL, NULL, NULL);

  gc=XCreateGC(disp,win,0,0);				/* create GC for movie window */

  XSelectInput(disp,win,ExposureMask | ButtonPressMask);
  XMapWindow(disp,win); 

  qTableSize=MakeQTables(header.qfactor,&qTable);	/* generate Q table according to Q factor from header */
  XPlxPutTable(disp,win,gc,qTable,qTableSize,1);	/* load Q table to CCube chip for decompression */
    
  maximumSize=width*height*3/5;			/* calculate maximum compressed image data size,
							   for one frame of movie */
							/* assume minimum compression of 5:1 */
  cImageData=malloc(maximumSize);			/* allocate data area in memory */
  cImage=(XPlxCImage*)XPlxCreateCImage(disp,cImageData,
       maximumSize,width,height);			/* assign allocated memory for XPlxCImage */

  while (1) 
  {
    gettimeofday(&before, NULL);                    /* This is used to calculate the time spent in the 
                                                           event loop for video synchronization */
    if (XPending(disp)) 
    {
      XNextEvent(disp,&event);
      XPlxVideoTag(disp,win,gc,PLX_VIDEO); 	/* video tag remains PLX_VIDEO until ready for playback */
      first=1;
    } 
        
    fseek(jpegFile,indexBuffer[currentFrame],0);	/* skip to start of frame pointed to by index offset */

    fread(&instruction,1,1,jpegFile);		/* this is to skip the opcode instruction */

    if ((instruction&0x00ff)==LOAD_AUDIO_0)		/* if instruction is to load audio */
    {
      int i;

      if (audioslice==NULL)
        abort();
      i=fread(audioslice,1,header.audioslice,jpegFile);	/* read audio data from file */
      write(devaudio,audioslice,header.audioslice);	/* send data to device */

      fread(&instruction,1,1,jpegFile);		/* read the opcode instruction. */

    }

    if ((instruction&0x00ff)==LOAD_JPEG) 		/* if instruction is to load jpeg data */
    {
      fread(&cImageSize,4,1,jpegFile);		/* read in the frame data size */

      fread(cImageData,cImageSize,1,jpegFile);	/* read in the jpeg data of the frame */

      cImage->size=cImageSize; 
      XPlxPutCImage(disp,win,gc,cImage, 
                0,0,width,height,
		0,0,width,height,
		0); 					/* set to crop image if necessary */
      if (first)
      {
        XPlxVideoTag(disp,win,gc,PLX_VIDEO_OVR);/* set tag to PLX_VIDEO_OVR so window doesn't get */
	first=0;				/* refresh with background color before drawing */
      }					        /* the next image */
    }
    if (++currentFrame>=header.frames-1)		/* allows loopback when it's end of movie */
      currentFrame=0;

    if (event.type == ButtonPress)
    {
      XFreeGC(disp, gc);
      XCloseDisplay(disp);
      exit(1);
    }
    
    gettimeofday(&after, NULL);                     /* Get the time after the event loop.  Now calculate
                                                           the difference and subtract that from the time
                                                           per frame */

    /* NOTE:  Do not try to use usleep instead of MyUSleep. */
    MyUSleep((unsigned int)(USecondsPerFrame - (ElapsedTime(before, after))));
  }
        
}


long ElapsedTime(before, after)
     struct timeval* before;
     struct timeval* after;
{
  return(((after->tv_sec - before->tv_sec) * 1000) +        /* Convert the seconds part to milliseconds */
         ((after->tv_usec - before->tv_usec) /1000));       /* Convert the microseconds part to milliseconds */
} /* end function ElapsedTime */


static void MyUSleep(microseconds)
unsigned int microseconds;
{
  int			descriptorTableSize = 0;
  int			result;
  struct timeval	waitTime;				    /* Used to tell select() how long to wait until it returns */
  
  descriptorTableSize = sysconf(_SC_OPEN_MAX);			    /* Get size of descriptor table for later use by select() */
  waitTime.tv_sec = 0;						    /* Set up a timeval to wait for the time passed in as... */
  waitTime.tv_usec = microseconds;				    /* ...argument. */
  result = select(descriptorTableSize,				    /* Use select to sleep by not asking to be informed of... */
		      (fd_set*)0, (fd_set*)0,(fd_set*)0,	    /* ...input on any file descriptors, and filling in the... */
		      &waitTime);				    /* ...timeval for the wait period. */
  return;
} /* end function MyUSleep */




XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display)
{
  int			i;
  int			visualsMatched;
  XVisualInfo		visualHints;
  static XVisualInfo*	visualList = (XVisualInfo*)NULL;
  XVisualInfo*		theVisual;
  
  if (visualList != (XVisualInfo*)NULL)				    /* Was data saved by a previous call to this function? */
    XFree((void*)visualList);					    /* Yes, release the data since it is no longer needed */
  visualHints.class = TrueColor;				    /* Set up hints to determine the type of visual needed... */
  visualHints.screen = screenNumber;				    /* ...to create a window that can display video */
  visualList = XGetVisualInfo(display,				    /* Try to find a visual that matches what is needed for... */
			      VisualClassMask |			    /* ...a window that will display video or TrueColor.... */
			      VisualScreenMask,			    /* ...graphics. */
			      &visualHints,
			      &visualsMatched);
  if (visualsMatched == 0)					    /* Did the search for a TrueColor visual fail? */
    return((XVisualInfo*)NULL);					    /* Yes, return a NULL structure to represent failure */
  for (theVisual = visualList, i = 0;				    /* Look at all the visuals returned by XGetVisualInfo... */
       i < visualsMatched;					    /* ...to find the best match.  The idea match would be... */
       i++, theVisual++)					    /* ...a 24-bit depth Visual. */
  {
    if (theVisual->depth == 24)					    /* Was an ideal match found? */
      return(theVisual);					    /* Yes, return this Visual */
  }
  for (theVisual = visualList, i = 0;				    /* If this statement is reached, no 24-bit TrueColor... */
       i < visualsMatched;					    /* ...Visual was found; in this case, fallback and try... */
       i++, theVisual++)					    /* ...to find the next best thing: any TrueColor visual... */
  {								    /* ...of depth >= 24 bits (probably a 32-bit is what will... */
    if (theVisual->depth >= 24)					    /* ...be found). */
      return(theVisual);					    /* Was a good fallback Visual found?  If so, return it. */
  }
  return(visualList);						    /* If this statement was reached, no good fallback was... */
								    /* ...found.  In this case, simply return the first... */
								    /* ...TrueColor visual found. */
}								    /* end function GetTrueColorVisual */








