/* (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved */

/* This file is $OPENWINHOME/src/client/xlib/example/xlib_test.c */

static char *rcsid = "@(#)$Id: xlib_test.c,v 1.87 1994/10/22 23:36:14 drapeau Exp $ Parallax Graphics Inc";

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <XPlxExt.h>
#include <XPlxProtos.h>

plx_IO *XPlxQueryConfig();
plx_signal *XPlxQueryVideo();
static void MyUSleep(unsigned int);


Widget top, video_widget;

plx_IO *plxio = NULL;
plx_signal *plxsignal = NULL;

static Arg args[25];
static unsigned int n;

int videowidth = 640;
int videoheight = 512;



XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display)
{
  int			i;
  long			visualInfoMask;
  int			visualsMatched;
  XVisualInfo		visualHints;
  static XVisualInfo*	visualList = (XVisualInfo*)NULL;
  XVisualInfo*		theVisual;
  
  if (visualList != (XVisualInfo*)NULL)				    /* Was data saved by a previous call to this function? */
    XFree((void*)visualList);					    /* Yes, release the data since it is no longer needed */
  visualHints.class = TrueColor;				    /* Set up hints to determine the type of visual needed... */
  visualHints.screen = screenNumber;				    /* ...to create a window that can display video */
#ifdef __hpux
  visualInfoMask = (VisualScreenMask);				    /* For the HP product, don't actually need a TrueColor class */
#else
  visualInfoMask = (VisualClassMask | VisualScreenMask);	    /* For Sun, need to get real TrueColor class */
#endif
  visualList = XGetVisualInfo(display,				    /* Try to find a visual that matches what is needed for... */
			      visualInfoMask,			    /* ... a window that will display video or TrueColor... */
			      &visualHints,			    /* ...graphics. */
			      &visualsMatched);
  if (visualsMatched == 0)					    /* Did the search for a TrueColor visual fail? */
    return((XVisualInfo*)NULL);					    /* Yes, return a NULL structure to represent failure */
  for (theVisual = visualList, i = 0;				    /* Look at all the visuals returned by XGetVisualInfo... */
       i < visualsMatched;					    /* ...to find the best match.  The idea match would be... */
       i++, theVisual++)					    /* ...a 24-bit depth Visual. */
  {
    if (theVisual->depth == 24)					    /* Was an ideal match found? */
      return(theVisual);					    /* Yes, return this Visual */
  }
  for (theVisual = visualList, i = 0;				    /* If this statement is reached, no 24-bit TrueColor... */
       i < visualsMatched;					    /* ...Visual was found; in this case, fallback and try... */
       i++, theVisual++)					    /* ...to find the next best thing: any TrueColor visual... */
  {								    /* ...of depth >= 24 bits (probably a 32-bit is what will... */
    if (theVisual->depth >= 24)					    /* ...be found). */
      return(theVisual);					    /* Was a good fallback Visual found?  If so, return it. */
  }
  return(visualList);						    /* If this statement was reached, no good fallback was... */
								    /* ...found.  In this case, simply return the first... */
								    /* ...TrueColor visual found. */
}								    /* end function GetTrueColorVisual */

void main(argc, argv)
int argc;
char **argv;
{
  Window win;
  Display *disp;
  int scr, mask;
  GC gc; 
  XEvent event;
  XSetWindowAttributes attr;
  XVisualInfo vinfo;
  int depth = 32;
  XVisualInfo*		visualInfo;
  
  disp = XOpenDisplay(getenv("DISPLAY"));			    /* 	Connect your client program with the X server	*/
  scr = (int) XDefaultScreen(disp);				    /*	Get a default screen				*/
  visualInfo = GetTrueColorVisual(scr, disp);			    /* Get a proper Visual for using 24-bit video in a window */
  depth = visualInfo->depth;
  mask = CWBackPixel |						    /*	WARNING!!! Must set up the colormap, the... */
    CWColormap |						    /* ...background_pixel and the border_pixel to make it... */
      CWBorderPixel;						    /* ...work when the window depth is diff. from root depth.  */
  attr.colormap = XCreateColormap(disp, XRootWindow(disp, scr),
				  visualInfo->visual, AllocNone);
  attr.background_pixel = BlackPixel(disp, DefaultScreen(disp));
  attr.border_pixel = WhitePixel(disp, DefaultScreen(disp));
  win = XCreateWindow(disp, RootWindow(disp, scr),		    /* 	Create the drawable for the XPlx calls		*/
		      0, 0, videowidth, videoheight, 1, depth,
		      InputOutput, visualInfo->visual, mask, &attr);
  gc = XCreateGC(disp, win, 0, 0);				    /* 	WARNING!!! Must create the gc with the window */
  XFlush(disp);
  plxio = XPlxQueryConfig(disp, win, gc);			    /*	Query the hardware for configuration parameters */
  XPlxVideoInputSelect(disp, win, gc,				    /* Ask for input on channel 0, with the following... */
		       PLX_INPUT_0,				    /* ...characteristics: */
		       PLX_NTSC,				    /* Standard = NTSC */
		       PLX_COMP,				    /* Format = Composite */
		       PLX_RGB24);				    /* Type = RGB24 */
  
  plxsignal = XPlxQueryVideo(disp, win, gc);			    /*	Query the Video to see if sync is there		*/
  while ( !plxsignal->sync_ok )					    /*	Wait until a signal is input into the board	*/
  {
    printf(" Sync Absent - Hook up an input source\n");
    MyUSleep(500000);
    plxsignal = XPlxQueryVideo(disp, win, gc);                      /*  Query the Video again to check sync             */
  }
  videowidth = plxsignal->w;
  videoheight = plxsignal->h - plxsignal->b;
  XPlxVideoTag(disp, win, gc, PLX_VIDEO);
  XSelectInput(disp, win, ExposureMask);			    /*	Catch exposure events				*/
  XResizeWindow(disp, win, videowidth, videoheight);		    /*	Map windows					*/
  XMapWindow(disp, win); 
  while (1)							    /*	Continuously show live video with the standard signal width and height */
  {
    XNextEvent(disp, &event);
    XPlxVideoLive(disp, win, gc, 0, 0, 0, 0, videowidth, videoheight);
  }
} /* end program */



static void MyUSleep(unsigned int microseconds)
{
  int			descriptorTableSize = 0;
  int			result;
  struct timeval	waitTime;				    /* Used to tell select() how long to wait until it returns */
  
  descriptorTableSize = sysconf(_SC_OPEN_MAX);			    /* Get size of descriptor table for later use by select() */
  waitTime.tv_sec = 0;						    /* Set up a timeval to wait for the time passed in as... */
  waitTime.tv_usec = microseconds;				    /* ...argument. */
  result = select(descriptorTableSize,				    /* Use select to sleep by not asking to be informed of... */
		      0, 0, 0,					    /* ...input on any file descriptors, and filling in the... */
		      &waitTime);				    /* ...timeval for the wait period. */
  return;
} /* end function MyUSleep */
