/*
 *   Copyright (c) 1993 by
 *   PARALLAX GRAPHICS, INCORPORATED, Santa Clara, California.
 *   All rights reserved
 *
 *   This software is furnished on an as-is basis, and may be used and copied
 *   only with the inclusion of the above copyright notice.
 *
 *   The information in this software is subject to change without notice.
 *   No committment is made as to the usability or reliability of this
 *   software.
 *
 *   Parallax Graphics, Inc.
 *   2500 Condensa Street
 *   Santa Clara, California  95051
 *   Author: George Drapeau
 */
/* This program is used to show the ability to read raw JPEG images from disk,
 * and to perform some simple performance tests on the JPEG widget.
 * The program takes one argument, the name of the disk file from which to read
 * JPEG raw data images.  The program calls a function which opens the file,
 * then begins reading JPEG image data and displaying it in the widget's window
 * as quickly as possible.
 * When the program is finished, it displays some simple statistics to indicate
 * how quickly it was able to display images.
 * This program assumes that all the images in a single file have the same
 * dimensions (e.g., 640x480).
 * Usage: jpegPlaybackTest fileName
 */
/* $Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/jpegPlaybackTest.c,v 1.13 1994/10/22 04:41:12 drapeau Exp $ */
/* $Log: jpegPlaybackTest.c,v $
 * Revision 1.13  1994/10/22  04:41:12  drapeau
 * Made the code more ANSI-compatible.
 *
 * Revision 1.12  1993/11/17  02:32:04  drapeau
 * No real changes; only small cosmetic adjustments here.
 *
 * Revision 1.11  93/09/10  18:04:12  drapeau
 * Added support for JPEG widgets that don't use shared memory.
 * 
 * Revision 1.10  93/09/10  12:37:31  drapeau
 * First public version of this test program.
 *  */
static char jpegPlaybackTestRcsId[] = "$Header: /plx/S/source/Xclients/solaris2.4/3.4.new/RCS/jpegPlaybackTest.c,v 1.13 1994/10/22 04:41:12 drapeau Exp $";

#include <stdio.h>
#include <sys/param.h>
#include <sys/time.h>
#include <signal.h>
#include <fcntl.h>
#include <malloc.h>
#include <X11/Xlib.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <XPlxExt.h>
#include <XVideoIn.h>
#include <JPEG.h>

Widget top, jpegWidget;
Boolean	WorkProc(XtPointer);

int		inputFile;
char*		fileName;
int		totalNumberOfGrabs = 0;
struct timeval	before;
XPlxCImage*	jpegDataPtr;
XPlxCImage	jpegImage;
XPlxCImage*	allocatedImage;
char*		imageData;

 
void main(int argc, char** argv)
{
  XtAppContext	appContext;
  int		result;
  XtWorkProcId	timerID;
  Boolean	useSharedMemory;
  
  top = XtAppInitialize(&appContext, "JPEGTest", NULL, 0,
			&argc, argv,
			NULL, (ArgList)NULL, 0);
  if (argc < 2)							    /* Was an input file specified? */
  {								    /* No, report the error and exit this program */
    printf("No input file was specified.  Exiting.\n");
    exit(-1);
  }
  fileName = argv[1];						    /* Yes, point to the file name specified on the command line */
  inputFile = open(fileName, O_RDONLY);				    /* Try to open the specified file for reading JPEG images */
  if (inputFile == -1)
  {
    printf("Could not open the file %s for reading.  Exiting.\n",
	   fileName);
    exit(inputFile);
  }
  jpegWidget = XtVaCreateManagedWidget("jpeg", jpegWidgetClass,	    /* Create a JPEG widget used for displaying the raw JPEG... */
				       top,			    /* ...images that will be read in from disk. */
				       NULL);
  XtVaGetValues(jpegWidget,					    /* Determine if the JPEG widget is using shared memory... */
		XtNuseSharedMemory, &useSharedMemory,		    /* ...to store JPEG data. */
		NULL);
  if (useSharedMemory == False)
  {
    imageData = malloc(1024 * 256);				    /* Allocate enough data to hold a big image */
    allocatedImage = XPlxCreateCImage(XtDisplay(jpegWidget),	    /* Create a XPlxCImage structure to hold JPEG images */
				      imageData, 1024 * 256,
				      640, 480);
    if (allocatedImage == NULL)
    {
      printf("Could not create a CImage for storing JPEG images.  Exiting.\n");
      exit(-1);
    }
    XtVaSetValues(jpegWidget, XtNjpegData, allocatedImage, NULL);   /* Point the JPEG widget at the newly-created JPEG image */
  }
  XtVaGetValues(jpegWidget,					    /* Point to the JPEG data structure used by this widget;... */
		XtNjpegData,					    /* ...if the widget is using shared memory, then it's... */
		&jpegDataPtr,					    /* ...better to read data directly into the shared memory... */
		NULL);						    /* ...memory segment used by the widget. */
  result = read(inputFile, (char*)&jpegImage,			    /* Try to read the header for one JPEG image */
		sizeof(XPlxCImage));
  result = read(inputFile, jpegDataPtr->data,			    /* Try to read one JPEG image */
		jpegImage.size);
  XtVaSetValues(top,						    /* Resize the top-level widget to correspond to the... */
		XtNwidth, (Dimension)jpegImage.width,		    /* ...dimensions of the JPEG images contained in this file. */
		XtNheight, (Dimension)jpegImage.height,
		NULL);
  XtVaSetValues(jpegWidget,					    /* Force the widget to re-examine its JPEG image and... */
		XtNputNewImage, True,				    /* ...determine whether to redraw itself. */
		NULL);
  XtRealizeWidget(top);
  result = gettimeofday(&before, (struct timezone*)NULL);	    /* Store the current time of day for later stats reporting */
  timerID = XtAppAddWorkProc(appContext, WorkProc,		    /* Set up to read JPEG images from disk and display them... */
			     (XtPointer)NULL);			    /* ...whenever there's free time. */
  XtAppMainLoop(appContext);
  exit(0);
}								    /* end function main */


Boolean WorkProc(XtPointer clientData)
{
  int			result;
  int			grabsPerSecond;
  Boolean		moreDataToRead = True;
  struct timeval	after;
  struct timeval	totalTime;
  struct timeval	timeForOneGrab;
  
  printf("Entered WorkProc.\n");
  if (moreDataToRead == True)					    /* Is there more data in the file to read? */
  {								    /* Yes, continue processing */
    result = read(inputFile, (char*)&jpegImage,			    /* Try to read the header for one JPEG image */
		  sizeof(XPlxCImage));
    if (result < sizeof(XPlxCImage))				    /* Did the read fail? */
    {								    /* Yes, report the error and take appropriate action */
      printf("Could not read the JPEG header from the file.  Must be done.\n");
      moreDataToRead = False;					    /* Set variable saying not to read anymore data */
    }
    if (moreDataToRead == True)					    /* Is it okay to try to read the JPEG image? */
    {								    /* Yes, go for it */
      printf("Total number of grabs so far is %d.\n",
	     totalNumberOfGrabs);
      result = read(inputFile, jpegDataPtr->data,		    /* Try to read one JPEG image */
		    jpegImage.size);
      if (result < jpegImage.size)				    /* Did the read fail? */
      {								    /* Yes, report the error and take appropriate action */
	printf("Could not read the JPEG image from the file.  Must be done.  Exiting.\n");
	moreDataToRead = False;					    /* Set variable saying not to read anymore data */
      }
      else							    /* Nope, the read was successful; show the new image */
      {
	totalNumberOfGrabs++;					    /* Increment the number of images read from disk */
	jpegDataPtr->size = jpegImage.size;			    /* Update the widget's notion of the size of the JPEG image. */
	jpegDataPtr->width = jpegImage.width;
	jpegDataPtr->height = jpegImage.height;
	XtVaSetValues(jpegWidget,				    /* Force the widget to re-examine its JPEG image and... */
		      XtNputNewImage, True,			    /* ...determine whether to redraw itself. */
		      NULL);
      }
    }
  }
  if (moreDataToRead == False)
  {
    close(inputFile);						    /* Done reading JPEG data from disk */
    result = gettimeofday(&after, (struct timezone*)NULL);
    before.tv_usec += (before.tv_sec * 1000000);		    /* Convert this timeval to all microseconds */
    after.tv_usec += (after.tv_sec * 1000000);			    /* Do the same for this timeval */
    totalTime.tv_usec = after.tv_usec - before.tv_usec;		    /* Determine the total time taken for the above compressions */
    timeForOneGrab.tv_usec = (totalTime.tv_usec / totalNumberOfGrabs);
    grabsPerSecond = (1000000 / timeForOneGrab.tv_usec);
    timeForOneGrab.tv_sec = (timeForOneGrab.tv_usec / 1000000);	    /* Convert this timeval back into seconds and microseconds */
    timeForOneGrab.tv_usec = (timeForOneGrab.tv_usec % 1000000);
    totalTime.tv_sec = (totalTime.tv_usec / 1000000);		    /* Convert this timeval back into seconds and microseconds */
    totalTime.tv_usec = (totalTime.tv_usec % 1000000);
    printf("Total time taken for %d frames is %d.%d seconds.\n",
	   totalNumberOfGrabs,
	   totalTime.tv_sec,
	   totalTime.tv_usec);
    printf("Time taken for 1 frame is %d.%d seconds.\n",
	   timeForOneGrab.tv_sec,
	   timeForOneGrab.tv_usec);
    printf("Number of frames per second: %d.\n", grabsPerSecond);
  }
  if (moreDataToRead)						    /* Should this function be called again? */
    return(False);						    /* Yes, there is more data to read from disk */
  else								    /* No, done reading all data from disk */
  {
    XtDestroyWidget(top);					    /* Destroy all widgets, especially the JPEG widget, to... */
    exit(0);							    /* ...free up shared memory resources taken by it. */
  }
} /* end function WorkProc */
