/*
 * (c) Copyright 1991 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log: file.c,v $
 * Revision 1.7  1992/05/27  16:23:06  schen
 * Moved from file.c to /plx/S/source/Xclients/sun4/schen/src/lib/api//file.c
 *
 * Revision 1.6  92/05/21  15:09:08  schen
 * checkin demo19
 * 
 * Revision 1.2  91/08/19  13:09:15  eric
 * added rcs header
 * 
 *
 */

#ifndef lint
static char *rcsid = "@(#)$Id: file.c,v 1.7 1992/05/27 16:23:06 schen Deleted $ Parallax Graphics Inc";
#endif

/*
 *	file.c - low level routines for driving a JPEG movie file as a virtual videodisc
 *
 *	Written by John Malleo-Roach
 *
 */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<unistd.h>
#include	<math.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>
#include	<sys/time.h>
#include	<sys/asynch.h>
#include	<sys/ipc.h>
#include	<sys/shm.h>
#include	<strings.h>
#include	"vaa.h"
#include	<X11/extensions/XShm.h>
#include	"movie.h"
#include	"alert.h"

/* CONSTANTS */

#define V_FORWARD	 1
#define V_REVERSE	 2
#define V_STILL		 4
#define V_SLOW		 8
#define V_NORMAL	16
#define V_FAST		32

/* Player status codes, these are the same in all device driver files */

#define PLAYER_STATUS_UNKNOWN	0
#define PLAYER_STATUS_STOP		1
#define PLAYER_STATUS_STILL		2
#define PLAYER_STATUS_PLAY		3
#define PLAYER_STATUS_EMPTY		4

#define JBUFSIZE	8192 * 8
#define BUFFERSIZE	8192 * 8
#define ABUFSIZE	8000
#define MAXFRAMES	16384

/* STATIC VARIABLES */

static u_char *bufp   = NULL;
static u_char *bufend = NULL;

static unsigned char Buffer[BUFFERSIZE];	/* Disk buffer A for reads/writes		*/

static unsigned int VStatus  = 0;			/* Current virtual videodisc status     */
static unsigned int EndPoint = -1;			/* If positive, valid stopping point    */
static int		CurrentFrame = 0;			/* Number of current frame				*/
static jpheader JPHeader;					/* JPEG header info						*/

static unsigned int  audiofd;				/* File descriptor for audio device		*/
static unsigned char audiobuf[ABUFSIZE];	/* Buffer for loading audio data		*/

static unsigned char  jpegbuf[JBUFSIZE];	/* Buffer for loading jpeg data 	    */
static unsigned int  indexbuf[MAXFRAMES];	/* Buffer for storing frame offsets		*/

static struct itimerval timeval;			/* Timer parameters						*/

static VideoIn  VideoWin;					/* Pointer to XView JPEG output object	*/
static Display *OurDisplay;					/* Cached X Display pointer				*/

static JPEGImage       *VideoImage;			/* The XPlxCImage structure				*/
static XShmSegmentInfo  shminfo;			/* Info for any shared memory			*/

static BOOLEAN          UsingSharedMemory=FALSE;

/* EXTERNAL DEFINITIONS */

extern int PlayerId;			/* File descriptor for the virtual videodisc player	*/

/* FUNCTIONS */

/* Create a shared memory version of JPEGImage structure */

static JPEGImage *CreateJPEGImage(display, width, height)
Display			*display;
int				 width;
int				 height;
{
	int			majorv;
	int			minorv;
	int			i;
	JPEGImage  *image;
	Bool		sharedPixmaps;

	if(!XShmQueryVersion(display,&majorv,&minorv,&sharedPixmaps)) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	shminfo.shmid = shmget(IPC_PRIVATE, 128 * 1024, IPC_CREAT|0777);
	if (shminfo.shmid < 0) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	shminfo.shmaddr = (char *)shmat(shminfo.shmid, 0, 0);
	if (shminfo.shmaddr == ((char *) -1)) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	image = (JPEGImage *)
		XPlxShmCreateCImage(display,shminfo.shmaddr,&shminfo,0,width,height);
	shminfo.readOnly  = False;
	XShmAttach(display, &shminfo);
	UsingSharedMemory = TRUE;
	return(image);
}

/* Free up a XVideo jpeg image data structure */

static FreeImage(dpy,i)
Display   *dpy;
JPEGImage *i;
{
	XPlxDestroyCImage(i);
	return(NULL);
}

/* Destroy the X11 shared memory JPEGImage structure */

static int DestroyJPEGImage(display,image)
Display *display;
JPEGImage *image;
{
	int rc;

	if (UsingSharedMemory) {
		UsingSharedMemory = FALSE;
		XShmDetach(display, image->info);
		rc = shmdt(image->info->shmaddr);
		if (rc < 0) SystemError(errno,"shmdt failed");
		rc = shmctl(image->info->shmid, IPC_RMID, 0);
		if (rc < 0) SystemError(errno,"shmctl failed");
	}
	else FreeImage(display,image);
}

static int LoadAudio(fd,audio,play)
int fd;
int audio;
int play;
{
	int i,j;
	u_char *f;

	i = JPHeader.audioslice;

	bcopy(bufp,audiobuf,i); bufp += i;

/* Load up the audio associated with this frame */

	if (play && audio) write(audio,audiobuf,JPHeader.audioslice);
}

/* Decode a frame into a equivalend area of memory */

static int LoadFrame(dpy,fd,video)
Display *dpy;
int     fd;
VideoIn video;
{
	int i,j;
	int size;
	u_char *f;

/* Get the JPEG frame size out */

	i = sizeof(int);

	bcopy(bufp,&size,i); bufp += i;	i = size;

/* Get the JPEG frame data out */

	bcopy(bufp,VideoImage->data,i); bufp += i;

/* Set the size field of the JPEGImage structure */

	VideoImage->size = size;

/* Pull in the decoded frame now

	if (CurrentFrame && UsingSharedMemory) while (xv_get(video,VIDEO_SHM_DONE) == FALSE);
*/
	xv_set(video,VIDEO_JPEG_IMAGE,VideoImage,NULL);

/* Flush the X server */

	XSync(dpy,False);
}

static int DecodeFrame(dpy,fd,audio,video,track)
Display *dpy;
int     fd;
int		audio;
VideoIn video;
int     track;
{
	unsigned char c;

	while (1) {

/* Get next byte stream command until we get a frame loaded */

		c = *bufp++;

		switch (c) {
		case LOAD_JPEG:
			LoadFrame(dpy,fd,video);
			break;
		case END_FRAME:
			return;
			break;
		case LOAD_AUDIO_0:
			LoadAudio(fd,audio,(track == 0));
			break;
		case LOAD_AUDIO_1:
			LoadAudio(fd,audio,(track == 1));
			break;
		case LOAD_AUDIO_2:
			LoadAudio(fd,audio,(track == 2));
			break;
		case LOAD_AUDIO_3:
			LoadAudio(fd,audio,(track == 3));
			break;
		default:
			printf("Undefined j_movie byte stream command: %d\n",c);
			printf("Current frame number is %d\n",CurrentFrame);
			return(-1);
		}
	}
}

/* Initialize the virtual videodisc player to a known state */

int InitializeFile(filename,video)
char    *filename;
VideoIn  video;
{
	int offset;
	double vol;

/* Open the compressed video file */

	if ((PlayerId = open(filename,O_RDONLY)) < 0) {
		SystemError(errno,"Couldn't open j_movie file");
		return(-1);
	}

/* Load the Uniflix header information */

	if ((offset = read(PlayerId,Buffer,BUFFERSIZE)) < 0) {
		SystemError(errno,"Load of j_movie header information failed");
		return(-1);
	}
	bcopy(Buffer,&JPHeader,sizeof(jpheader));

	bufp   = Buffer + sizeof(jpheader);
	bufend = Buffer + offset;

/* Verify the file type and version */

	if (strcmp(JPHeader.magic,"j_movie")) {
		ShowAlert("Requested file is not a j_movie file!","",filename,NULL);
		close(PlayerId); return(-1);
	}
	
/* Load up the frame index table */

	if (JPHeader.indexbuf) {
		if (lseek(PlayerId,JPHeader.indexbuf,SEEK_SET) < 0) {
			SystemError(errno,"SEEK in movie file failed for frame indexes",NULL);
			close(PlayerId); return(-1);
		}
		if (read(PlayerId,indexbuf,(sizeof(int) * JPHeader.frames)) < 0) {
			SystemError(errno,"Read in movie file failed for frame indexes",NULL);
			close(PlayerId); return(-1);
		}
	}

/* Open up port for communicating with the audio driver, if needed */

	if (JPHeader.audio.data_size != 0) {
		if ((audiofd = open("/dev/audio",O_WRONLY|O_NDELAY)) < 0) {
			if (errno == EBUSY) {
				ShowAlert("Audio device is currently busy, try again later...",NULL);
				close(PlayerId); return(-1);
			}
			else {
				SystemError(errno,"Audio device open failed");
				close(PlayerId); return(-1);
			}
		}

/* Make sure audio device matches audio file requirements */

		if (audio_set_play_config(audiofd,&(JPHeader.audio)) != AUDIO_SUCCESS) {
			ShowAlert("Couldn't reset audio device encoding",NULL);
			close(audiofd); close(PlayerId); return(-1);
		}

/* Set playback volume (currently disabled)

		vol =  60.0 / 100.0;

		if (audio_set_play_gain(audiofd,&vol) != AUDIO_SUCCESS) {
			ShowAlert("Could not set audio gain!",NULL);
			close(audiofd); close(PlayerId);
			return(-1);
		}
*/
	}
	else audiofd = 0;

/* Initialize some important globals */

	VideoWin   = video;
	OurDisplay = (Display *) xv_get(VideoWin,XV_DISPLAY);

/* Set the video object to still frames */

	xv_set(VideoWin,VIDEO_LIVE,FALSE,NULL);

/* Display the first frame in the file */

	CurrentFrame = 0;
	VStatus      = V_STILL;
	EndPoint     = -1;

/* Initialize the XPlxCImage structure */

	VideoImage = (JPEGImage *) CreateJPEGImage(OurDisplay,JPHeader.width,JPHeader.height);

	if (UsingSharedMemory) xv_set(VideoWin,VIDEO_USE_SHM,TRUE,NULL);

/* Show first frame to user */

	DecodeFrame(OurDisplay,PlayerId,audiofd,VideoWin,0);

/* return open file pointer */

	return(FALSE);
}

/* Return a simple status code for the player */

int FileStatus()
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	return(PLAYER_STATUS_UNKNOWN);
}

/* Return current frame number from player */

int FileFrameNumber()
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	return(CurrentFrame);
}

/* Set Audio parameters */

int SetFileAudio(mode)
int mode;
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	return(NULL);
}

/* Set whether the video is visible or not */

int SetFileVideo(mode)
int mode;
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	return(NULL);
}

/* Seek and show a single frame */

int ShowFileFrame(frame)
int frame;
{
	int offset,size;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Check for valid frame address */

	if (frame < 0) {
		ShowAlert("Invalid frame address requested in ShowFileFrame",NULL);
		return(-1);
	}
	if (frame >= JPHeader.frames) {
		ShowAlert("Invalid frame address requested in ShowFileFrame",NULL);
		return(-1);
	}

/* Index into frame offset table for file offset */

	offset = indexbuf[frame];
	if (lseek(PlayerId,offset,SEEK_SET) < 0) {
		SystemError(errno,"SEEK into movie file failed for frame index",NULL);
		return(-1);
	}
	size = indexbuf[frame+1] - offset;
	if ((offset = read(PlayerId,Buffer,size)) < 0) {
		SystemError(errno,"Load of jpeg movie information failed");
		return(-1);
	}
	if (offset != size) {
		ShowAlert("Read didn`t return requested number of bytes",NULL);
		return(-1);
	}

/* Initialize pointers for frame display */

	CurrentFrame = frame;
	EndPoint     = -1;
	bufp         = Buffer;
	bufend       = Buffer + offset;

	DecodeFrame(OurDisplay,PlayerId,audiofd,VideoWin,0);

	return(NULL);
}

/* Seek and show a complete video segment */

int ShowFileSegment(start,end)
int start,end;
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Check validity */

	if ((start > end) || (start < 0) || (end > JPHeader.frames)) {
		ShowAlert("Invalid video segment endpoints",NULL);
		return(-1);
	}

/* Interrupt and locate starting frame */

	FileStop(); if (ShowFileFrame(start)) return(-1);

/* Play till end of segment */

	EndPoint = end; FilePlay();

	return(NULL);
}

/* Service routine for virtual videodisc */

Notify_value service()
{
	if (VStatus & V_FORWARD) {
		if (VStatus & V_FAST) {
			if ((CurrentFrame + 3) < (JPHeader.frames - 1)) CurrentFrame += 2;
		}
		FileStepForward();
	}
	else if (VStatus & V_REVERSE) {
		if (VStatus & V_FAST) {
			if ((CurrentFrame - 2) > 0) CurrentFrame -= 2;
		}
		FileStepReverse();
	}
	return(NOTIFY_DONE);
}

/* Command : play forward */

int FilePlay()
{
	int value;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Initialize notifer itimer signal handler */

	value = (1000000 / JPHeader.fps);
	timeval.it_value.tv_usec    = value;
	timeval.it_value.tv_sec     = 0;
	timeval.it_interval.tv_usec = value;
	timeval.it_interval.tv_sec  = 0;
	notify_set_itimer_func(VideoWin,service,ITIMER_REAL,&timeval,NULL);

	VStatus = V_FORWARD | V_NORMAL;

	return(NULL);
}

/* Command : play reverse */

int FileReverse()
{
	int value;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Initialize notifer itimer signal handler */

	value = (1000000 / JPHeader.fps);
	timeval.it_value.tv_usec    = value;
	timeval.it_value.tv_sec     = 0;
	timeval.it_interval.tv_usec = value;
	timeval.it_interval.tv_sec  = 0;
	notify_set_itimer_func(VideoWin,service,ITIMER_REAL,&timeval,NULL);

	VStatus  = V_REVERSE | V_NORMAL;
	EndPoint = -1;

	return(NULL);
}

/* Command : Continue an interrupted segment */

int FileContinue()
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	return(NULL);
}

/* Command : pause during playback */

int FileStill()
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	if ((VStatus & V_STILL) == NULL)
		notify_set_itimer_func(VideoWin,NOTIFY_FUNC_NULL,
							   ITIMER_REAL,NULL,NULL);
	VStatus  = V_STILL;
	EndPoint = -1;
	return(NULL);
}

/* Command : Stop playback */

int FileStop()
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	if ((VStatus & V_STILL) == NULL)
		notify_set_itimer_func(VideoWin,NOTIFY_FUNC_NULL,
							   ITIMER_REAL,NULL,NULL);
	VStatus      = V_STILL;
	CurrentFrame = 0;
	EndPoint     = -1;
	return(NULL);
}

/* Command : move 1 frame forward */

int FileStepForward()
{
	int offset,size;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	if (CurrentFrame == (JPHeader.frames - 2)) {
		FileStill();
		return(NULL);
	}
	if (CurrentFrame == EndPoint) {
		FileStill();
		return(NULL);
	}
	CurrentFrame++;
	offset = indexbuf[CurrentFrame];
	if (lseek(PlayerId,offset,SEEK_SET) < 0) {
		SystemError(errno,"SEEK into movie file failed for frame index",NULL);
		return(-1);
	}
	size = indexbuf[CurrentFrame+1] - offset;
	if ((offset = read(PlayerId,Buffer,size)) < 0) {
		SystemError(errno,"Load of jpeg movie information failed");
		return(-1);
	}
	if (offset != size) {
		ShowAlert("Read didn`t return requested number of bytes",NULL);
		return(-1);
	}
	bufp   = Buffer;
	bufend = Buffer + offset;

	DecodeFrame(OurDisplay,PlayerId,audiofd,VideoWin,0);

	return(NULL);
}

/* Command : move 1 frame backward */

int FileStepReverse()
{
	int offset,size;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	if (CurrentFrame == 0) {
		FileStill();
		return(NULL);
	}
	CurrentFrame--;
	offset = indexbuf[CurrentFrame];
	if (lseek(PlayerId,offset,SEEK_SET) < 0) {
		SystemError(errno,"SEEK into movie file failed for frame index",NULL);
		return(-1);
	}
	size = indexbuf[CurrentFrame+1] - offset;
	if ((offset = read(PlayerId,Buffer,size)) < 0) {
		SystemError(errno,"Load of jpeg movie information failed");
		return(-1);
	}
	if (offset != size) {
		ShowAlert("Read didn`t return requested number of bytes",NULL);
		return(-1);
	}
	bufp   = Buffer;
	bufend = Buffer + offset;

	DecodeFrame(OurDisplay,PlayerId,audiofd,VideoWin,0);

	return(NULL);
}

/* Command : move slow forward */

int FileSlowForward()
{
	int value;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	FileStill();

/* Initialize notifer itimer signal handler */

	value = (1000000 / JPHeader.fps) * 3;
	timeval.it_value.tv_usec    = value;
	timeval.it_value.tv_sec     = 0;
	timeval.it_interval.tv_usec = value;
	timeval.it_interval.tv_sec  = 0;
	notify_set_itimer_func(VideoWin,service,ITIMER_REAL,&timeval,NULL);

	VStatus  = V_FORWARD | V_SLOW;
	EndPoint = -1;

	return(NULL);
}

/* Command : move slow reverse */

int FileSlowReverse()
{
	int value;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	FileStill();

/* Initialize notifer itimer signal handler */

	value = (1000000 / JPHeader.fps) * 3;
	timeval.it_value.tv_usec    = value;
	timeval.it_value.tv_sec     = 0;
	timeval.it_interval.tv_usec = value;
	timeval.it_interval.tv_sec  = 0;
	notify_set_itimer_func(VideoWin,service,ITIMER_REAL,&timeval,NULL);

	VStatus  = V_REVERSE | V_SLOW;
	EndPoint = -1;

	return(NULL);
}

/* Command : move fast forward */

int FileFastForward()
{
	int value;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

	FileStill();

/* Initialize notifer itimer signal handler */

	value = (1000000 / JPHeader.fps);
	timeval.it_value.tv_usec    = value;
	timeval.it_value.tv_sec     = 0;
	timeval.it_interval.tv_usec = value;
	timeval.it_interval.tv_sec  = 0;
	notify_set_itimer_func(VideoWin,service,ITIMER_REAL,&timeval,NULL);

	VStatus  = V_FORWARD | V_FAST;
	EndPoint = -1;

	return(NULL);
}

/* Command : move fast reverse */

int FileFastReverse()
{
	int value;

	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

	FileStill();

/* Initialize notifer itimer signal handler */

	value = (1000000 / JPHeader.fps);
	timeval.it_value.tv_usec    = value;
	timeval.it_value.tv_sec     = 0;
	timeval.it_interval.tv_usec = value;
	timeval.it_interval.tv_sec  = 0;
	notify_set_itimer_func(VideoWin,service,ITIMER_REAL,&timeval,NULL);

	VStatus  = V_REVERSE | V_FAST;
	EndPoint = -1;

	return(NULL);
}

/* Command : display visual frame location */

int FileIndex(status)
int status;
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}
	return(NULL);
}

/* Ejects the platter from the player */

int FileEject()
{
	if (PlayerId < 0) {
		ShowAlert("Virtual VideoDisc Player not initialized",NULL);
		return(-1);
	}

/* Close the open descriptors */

	close(PlayerId); PlayerId = 0;
	close(audiofd);  audiofd  = 0;

/* Free up any hared memory */

	if (UsingSharedMemory) xv_set(VideoWin,VIDEO_USE_SHM,FALSE,NULL);

/* Release the XPlxCImage structure */

	DestroyJPEGImage(OurDisplay,VideoImage);

/* Restart any current video */

	xv_set(VideoWin,VIDEO_LIVE,TRUE,NULL);

	return(NULL);
}

/* end of file.c */
