
/*
 *
 *	jpeg.c -- JPEG API library
 *
 *	Written by John Malleo-Roach
 *
 */

#define ASYNCWRITE
/* #define HYPERCMEDIA */

/* INCLUDE FILES */

#include	<stdio.h>
#include	<math.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/times.h>
#include	<sys/param.h>
#include	<sys/time.h>
#include	<sys/asynch.h>
#include	<sys/ipc.h>
#include	<sys/shm.h>

#include	<unistd.h>

#include    <xview/xview.h>
#include    <xview/canvas.h>
#include    <X11/Xlib.h>
#include	<X11/extensions/XShm.h>

#ifdef AUDIO
#include    <multimedia/libaudio.h>
#include    <multimedia/audio_device.h>
#endif

#include	"Video.h"
#include	"movie.h"
#include	"alert.h"

/* CONSTANTS */

#define JBUFSIZE	8192 * 8
#define BUFFERSIZE	8192 * 8
#define ABUFSIZE	8000
#define MAXFRAMES	16384

/* STATIC VARIABLES */

static u_char *bufp   = NULL;
static u_char *bufend = NULL;

static unsigned char Abuffer[BUFFERSIZE];	/* Disk buffer A for reads/writes		*/
static unsigned char Bbuffer[BUFFERSIZE];	/* Disk buffer B for reads/writes		*/
static unsigned int  BufSync=0;				/* Which buffer is active				*/
static aio_result_t  AioResult;				/* Use for asynchronous reads/writes	*/
static unsigned int  BufOffset=0;			/* Offset into file being read/wrote	*/

static unsigned int Counter  = 0;			/* Used to sync with the interrupts		*/

static int		NewJPEGMovie = 0;			/* True when jpeg movie is being built	*/
static int		FrameCount   = 0;			/* Number of frames written out			*/
static int		MovieFd      = 0;			/* Movie file descriptor				*/
static jpheader JPHeader;					/* JPEG header info						*/

static struct timeval TimePointer;			/* Starting time of day					*/

static unsigned char audiobuf[ABUFSIZE];	/* Buffer for loading audio data		*/
static unsigned char  jpegbuf[JBUFSIZE];	/* Buffer for loading jpeg data 	    */
static unsigned int  indexbuf[MAXFRAMES];	/* Buffer for storing frame offsets		*/

static JPEGImage *VideoImage;				/* The XPlxCImage structure				*/
static XShmSegmentInfo shminfo;				/* Info for any shared memory			*/
static BOOLEAN UsingSharedMemory=FALSE;

/* FUNCTIONS */

/* This is the no-op SIGALRM signal handler.  It just keeps track of the numer of	*/
/* times it has been called for sync purposes.										*/

static void handler()
{
	Counter = 1;
}

/* Compute some interesting playback numbers for the user */

static void FindRates(fd,frames)
int fd;
int frames;
{
	struct stat info;
	struct timeval TotalTime;
	float d,e;
	int i,j;

	gettimeofday(&TotalTime,NULL);
	i = TotalTime.tv_sec * 1000000 + TotalTime.tv_usec;
	j = TimePointer.tv_sec * 1000000 + TimePointer.tv_usec;
	i = i - j;
	d = ((float) i) / 1000000;
	printf("Elapsed: %2.4f seconds, ",d);
	e = 1000000.0 / (((float) i) / ((float) frames));
	printf("%2.2f frames/sec, ",e);
	if (fstat(fd,&info) < 0) return;
	e = (float) (info.st_size) / (d * 1000.0);
	printf("%3.0fK bytes per second.\n",e);
}

/* Create a shared memory version of JPEGImage structure */

static JPEGImage *CreateJPEGImage(display, width, height)
Display			*display;
int				 width;
int				 height;
{
	int			majorv;
	int			minorv;
	int			i;
	JPEGImage  *image;
	Bool		sharedPixmaps;

	if(!XShmQueryVersion(display,&majorv,&minorv,&sharedPixmaps)) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	shminfo.shmid = shmget(IPC_PRIVATE, 128 * 1024, IPC_CREAT|0777);
	if (shminfo.shmid < 0) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	shminfo.shmaddr = (char *)shmat(shminfo.shmid, 0, 0);
	if (shminfo.shmaddr == ((char *) -1)) {
		UsingSharedMemory = FALSE;
		image = (JPEGImage *) XPlxCreateCImage(display,jpegbuf,0,width,height);
		return(image);
	}
	image = (JPEGImage *) XPlxShmCreateCImage(display,shminfo.shmaddr,&shminfo,0,width,height);
	shminfo.readOnly  = False;
	XShmAttach(display, &shminfo);
	UsingSharedMemory = TRUE;
	return(image);
}

/* Free up a XVideo jpeg image data structure */

int jpeg_free_image(dpy,i)
Display   *dpy;
JPEGImage *i;
{
	XPlxDestroyCImage(i);
	return(NULL);
}

/* Destroy the X11 shared memory JPEGImage structure */

static int DestroyJPEGImage(display,image)
Display *display;
JPEGImage *image;
{
	int rc;

	if (UsingSharedMemory) {
		UsingSharedMemory = FALSE;
		XShmDetach(display, image->info);
		XSync(display,False);
		rc = shmdt(image->info->shmaddr);
#ifdef HYPERCMEDIA
		if (rc < 0) hCm_SystemError(errno,"shmdt failed");
		rc = shmctl(image->info->shmid, IPC_RMID, 0);
		if (rc < 0) hCm_SystemError(errno,"shmctl failed");
#else
		if (rc < 0) SystemError(errno,"shmdt failed");
		rc = shmctl(image->info->shmid, IPC_RMID, 0);
		if (rc < 0) SystemError(errno,"shmctl failed");
#endif
	}
	else jpeg_free_image(display,image);
}

static int LoadAudio(fd,audio,play)
int fd;
int audio;
int play;
{
	int i,j;
	u_char *f;

	i = JPHeader.audioslice;

	if ((bufp + i) < bufend) { bcopy(bufp,audiobuf,i); bufp += i; }
	else {
		f = audiobuf; j = bufend - bufp; bcopy(bufp,f,j); f += j;
		aiowait(NULL);
		if (BufSync) {
			bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		else {
			bufp = Abuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		i -= j; if (i) bcopy(bufp,f,i); bufp += i;
	}

/* Load up the audio associated with this frame */

	if (play && audio) write(audio,audiobuf,JPHeader.audioslice);
}

/* Decode a frame into a equivalend area of memory */

static int LoadFrame(dpy,fd,video,frame)
Display *dpy;
int     fd;
VideoIn video;
int     frame;
{
	int i,j;
	int size;
	u_char *f;

	i = sizeof(int);

	if ((bufp + i) < bufend) { bcopy(bufp,&size,i); bufp += i; }
	else {
		f = (u_char *) &size; j = bufend - bufp; bcopy(bufp,f,j); f += j;
		aiowait(NULL);
		if (BufSync) {
			bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		else {
			bufp = Abuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		i -= j; if (i) bcopy(bufp,f,i); bufp += i;
	}
	i = size;
	if ((bufp + i) < bufend) { bcopy(bufp,VideoImage->data,i); bufp += i; }
	else {
		f = (u_char *) VideoImage->data; j = bufend - bufp; bcopy(bufp,f,j); f += j;
		aiowait(NULL);
		if (BufSync) {
			bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		else {
			bufp = Abuffer; bufend = bufp + AioResult.aio_return;
			BufOffset += AioResult.aio_return; BufSync ^= 1;
			aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		}
		i -= j; if (i) bcopy(bufp,f,i); bufp += i;
	}

/* Set the size field of the JPEGImage structure */

	VideoImage->size = size;

/* Pull in the decoded frame now

	if (frame && UsingSharedMemory) while (xv_get(video,VIDEO_SHM_DONE) == FALSE);
*/
	xv_set(video,VIDEO_JPEG_IMAGE,VideoImage,NULL);

/* Flush the X server */

	XSync(dpy,False);
}

static int DecodeFrame(dpy,fd,audio,video,frame,track)
Display *dpy;
int     fd;
int		audio;
VideoIn video;
int     frame;
int     track;
{
	unsigned char c;

/* FIXUP: This assumes that the first read returned BUFFERSIZE */

	if (frame == 0) {
		AioResult.aio_return = AIO_INPROGRESS;
		AioResult.aio_errno  = NULL;
		aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
		BufSync  ^= 1;
	}

	while (1) {

/* Get next byte stream command until we get a frame loaded */

		c = *bufp++;
		if (bufp == bufend) {
			aiowait(NULL);
			if (BufSync) {
				bufp = Bbuffer; bufend = bufp + AioResult.aio_return;
				BufOffset += AioResult.aio_return; BufSync ^= 1;
				aioread(fd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			}
			else {
				bufp = Abuffer; bufend = bufp + AioResult.aio_return;
				BufOffset += AioResult.aio_return; BufSync ^= 1;
				aioread(fd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			}
		}
		switch (c) {
		case LOAD_JPEG:
			LoadFrame(dpy,fd,video,frame);
			break;
		case END_FRAME:
			return;
			break;
		case LOAD_AUDIO_0:
			LoadAudio(fd,audio,(track == 0));
			break;
		case LOAD_AUDIO_1:
			LoadAudio(fd,audio,(track == 1));
			break;
		case LOAD_AUDIO_2:
			LoadAudio(fd,audio,(track == 2));
			break;
		case LOAD_AUDIO_3:
			LoadAudio(fd,audio,(track == 3));
			break;
		default:
			printf("Undefined j_movie byte stream command: %d\n",c);
			printf("Current frame number is %d\n",frame);
			return(-1);
		}
	}
}

jpeg_decode_file(dpy,fd,audio,video)
Display *dpy;
int      fd;
int		 audio;
VideoIn  video;
{
	int j,frames;

	frames    = 0;
	BufOffset = 0;
	BufSync   = 0;
	Counter   = 0;

/* Initialize the XPlxCImage structure */

	VideoImage = (JPEGImage *) CreateJPEGImage(dpy,JPHeader.width,JPHeader.height);

	if (UsingSharedMemory) xv_set(video,VIDEO_USE_SHM,TRUE,NULL);

/* Set the interrupt scheduler to hit us at the requested frames per second */

	signal(SIGALRM,handler);
	j = 1000000 / JPHeader.fps;
	ualarm(j,j);
		
/* Get the clock start time */
	
	gettimeofday(&TimePointer,NULL);

	while (frames < JPHeader.frames) {

/* Show the next available frame */

		DecodeFrame(dpy,fd,audio,video,frames,0);

/* Handle any application events */

		if (QLength(dpy)) {
			xv_input_pending(dpy,XConnectionNumber(dpy));
		}
		if (Counter == 0) pause(); Counter = 0;
		frames++;
	}
	ualarm(0,0);

/*	FindRates(fd,frames); */

	if (UsingSharedMemory) xv_set(video,VIDEO_USE_SHM,FALSE,NULL);

/* Release the XPlxCImage structure */

	DestroyJPEGImage(dpy,VideoImage);
}

/* Save one frame of video as a still jpeg movie file */

int jpeg_save_still(dpy,image,file)
Display   *dpy;
JPEGImage *image;
char      *file;
{
	FILE *fp;
	u_char    c;
	int       i;
	int      fd;
	jpheader  h;

	if ((fd = open(file,O_CREAT|O_WRONLY|O_TRUNC,0644)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jpeg file for output");
#else
		SystemError(errno,"Couldn't open jpeg file for output");
#endif
		return(-1);
	}

	strcpy(h.magic,"j_movie");
    
	h.version    = JPEG_VERSION_2;
	h.fps        = 0;
	h.frames     = 1;
	h.width      = image->width;
	h.height     = image->height;
	h.bandwidth  = image->size;
	h.qfactor	 = 0;
	h.mapsize	 = 256 * 256 * 256;
	h.tracks     = 0;
	h.volbase    = 0;
	h.audioslice = 0;
	
	for (i=0; i<FILLER; i++) h.filler[i] = 0;
	
/* Put out the header info */

	if (write(fd,h,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header write failed!");
#else
		SystemError(errno,"JPEG file header write failed!");
#endif
		close(fd);
		unlink(file);
		return(-1);
	}

/* Write the load jpeg frame instruction */

	c = LOAD_JPEG;
	write(fd,&c,1);

/* Save the compressed image data */

	if (write(fd,image->data,image->size) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file image data write failed!");
#else
		SystemError(errno,"JPEG file image data write failed!");
#endif
		close(fd);
		unlink(file);
		return(-1);
	}
	close(fd);
	return(NULL);
}

/* Load one frame of video from a still jpeg movie file */

JPEGImage *jpeg_load_still(dpy,file)
Display *dpy;
char    *file;
{
	u_char    *data;
	int          fd;
	JPEGImage    *i;
	jpheader      h;

	if ((fd = open(file,O_RDONLY)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jpeg file for input");
#else
		SystemError(errno,"Couldn't open jpeg file for input");
#endif
		return((JPEGImage *) NULL);
	}

/* Load the header info */

	if (read(fd,&h,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header read failed!");
#else
		SystemError(errno,"JPEG file header read failed!");
#endif
		close(fd);
		return((JPEGImage *) NULL);
	}

/* Verify that the file is kosher */

	if (strcmp(h.magic,"j_movie")) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("%s is not a valid JPEG compressed still image",file,NULL);
#else
		ShowAlert("File is not a valid JPEG compressed still image","",file,NULL);
#endif
		close(fd);
		return((JPEGImage *) NULL);
	}

/* Check the load single frame sequence */

	data = (u_char *) malloc(h.bandwidth);
	read(fd,data,1);
	if (*data != LOAD_JPEG) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("JPEG movie contains invalid movie sequence",NULL,NULL);
#else
		ShowAlert("JPEG movie contains invalid movie sequence",NULL);
#endif
		free(data);
		close(fd);
		return((JPEGImage *) NULL);
	}

/* Get the compressed frame out */

	if (read(fd,data,h.bandwidth) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file image data read failed!");
#else
		SystemError(errno,"JPEG file image data read failed!");
#endif
		free(data);
		close(fd);
		return((JPEGImage *) NULL);
	}
	close(fd);

	i = (JPEGImage *) XPlxCreateCImage(dpy,data,h.bandwidth,h.width,h.height);

	if (i == NULL) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("XPlxCreateCImage failed: Unknown errror",NULL,NULL);
#else
		ShowAlert("XPlxCreateCImage failed: Unknown errror",NULL);
#endif
		free(data);
		return((JPEGImage *) NULL);
	}
	return(i);
}

int jpeg_open_movie(dpy,filename,fps,audio)
Display   *dpy;
char      *filename;
int        fps;
Audio_hdr *audio;
{
	u_char    c;
	int       i;

	if (NewJPEGMovie) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Please finish current movie before starting new one!",NULL,NULL);
#else
		ShowAlert("Please finish current jpeg movie before starting new one!",NULL);
#endif
		return(-1);
	}
	else {
		NewJPEGMovie = TRUE;
		FrameCount   = 0;
	}
	if ((MovieFd = open(filename,O_CREAT|O_WRONLY|O_TRUNC,0644)) == -1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open jpeg movie file for output");
#else
		SystemError(errno,"Couldn't open jpeg movie file for output");
#endif
		return(-1);
	}

	strcpy(JPHeader.magic,"j_movie");
    
	JPHeader.version    = JPEG_VERSION_2;
	JPHeader.fps        = fps;
	JPHeader.frames     = 0;
	JPHeader.width      = 0;
	JPHeader.height     = 0;
	JPHeader.bandwidth  = 0;
	JPHeader.qfactor	= 0;
	JPHeader.mapsize    = 256 * 256 * 256;
	JPHeader.tracks     = 1;
	JPHeader.volbase    = 0;
	JPHeader.audioslice = (8000 / fps);
	if (8000 % fps) JPHeader.audioslice++;
	
	if (audio) bcopy(audio,&(JPHeader.audio),sizeof(Audio_hdr));
	else bzero(&(JPHeader.audio),sizeof(Audio_hdr));

	for (i=0; i<FILLER; i++) JPHeader.filler[i] = 0;
	
/* Put out the header info */

	if (write(MovieFd,JPHeader,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header write failed!");
#else
		SystemError(errno,"JPEG file header write failed!");
#endif
		close(MovieFd);
		unlink(filename);
		return(-1);
	}

/* Record offset to first frame */

	indexbuf[FrameCount] = sizeof(jpheader);

/* Set up data structures for asyncio writes */
#ifdef ASYNCWRITE
	bufp      = Bbuffer;
	bufend    = Bbuffer + BUFFERSIZE;
	BufOffset = 0;
	BufSync   = 0;
#endif
	return(0);
}

int jpeg_add_still(dpy,image)
Display *dpy;
JPEGImage *image;
{
	int     i,j;
	u_char *p;
	u_char  c;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Please open a JPEG movie before writing stills out!",NULL,NULL);
#else
		ShowAlert("Please open a JPEG movie before writing stills out!",NULL);
#endif
		return(-1);
	}
	if (FrameCount == 0) {
		JPHeader.width  = image->width;
		JPHeader.height = image->height;
	}
	else {
		if ((JPHeader.width != image->width) || (JPHeader.height != image->height)) {
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Frame width and/or height differs from previous!",NULL,NULL);
#else
			ShowAlert("Frame width and/or height differs from previous!",NULL);
#endif
			return(-1);
		}
	}
	JPHeader.bandwidth += image->size;

/* Write the load jpeg still instruction */

#ifdef ASYNCWRITE
	if (bufp != bufend) *bufp++ = LOAD_JPEG;
	else {
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		*bufp++ = LOAD_JPEG;
	}
#else
	c = LOAD_JPEG;
	if (write(MovieFd,&c,1) != 1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to JPEG movie file failed");
#else
		SystemError(errno,"Write to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif

/* Store the size of the compressed data */

#ifdef ASYNCWRITE
	if ((bufp + sizeof(int)) < bufend) {
		bcopy(&(image->size),bufp,sizeof(int));
		bufp += sizeof(int);
	}
	else {
		i = sizeof(int); p = (u_char *) &(image->size);
		j = bufend - bufp; bcopy(p,bufp,j); p += j;
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		i -= j; if (i) bcopy(p,bufp,i); bufp += i;
	}
#else
	if (write(MovieFd,&(image->size),sizeof(int)) != sizeof(int)) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to JPEG movie file failed");
#else
		SystemError(errno,"Write to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif

/* Store the frame data */

#ifdef ASYNCWRITE
	if ((bufp + image->size) < bufend) {
		bcopy(image->data,bufp,image->size);
		bufp += image->size;
	}
	else {
		i = image->size; p = (u_char *) image->data;
		j = bufend - bufp; bcopy(p,bufp,j); p += j;
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		i -= j; if (i) bcopy(p,bufp,i); bufp += i;
	}
#else
	if (write(MovieFd,image->data,image->size) != image->size) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to JPEG movie file failed");
#else
		SystemError(errno,"Write to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif
	return(0);
}

int jpeg_get_slice(dpy)
Display *dpy;
{
	return(JPHeader.audioslice);
}

int jpeg_add_audio(dpy,audio,track)
Display *dpy;
u_char  *audio;
int      track;
{
	int     i,j;
	u_char  c;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Please open a JPEG movie before writing audio out!",NULL,NULL);
#else
		ShowAlert("Please open a JPEG movie before writing audio out!",NULL);
#endif
		return(-1);
	}
	if ((track < 0) || (track > 3)) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Invalid track number requested!",NULL,NULL);
#else
		ShowAlert("Invalid track number requested!",NULL);
#endif
		return(-1);
	}

/* Write the load audio track instruction */

	c = LOAD_AUDIO_0 - track;
#ifdef ASYNCWRITE
	if (bufp != bufend) *bufp++ = c;
	else {
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		*bufp++ = c;
	}
#else
	if (write(MovieFd,&c,1) != 1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write to LOAD_AUDIO to JPEG movie file failed");
#else
		SystemError(errno,"Write to LOAD_AUDIO to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif

/* Store the audio data */

#ifdef ASYNCWRITE
	if ((bufp + JPHeader.audioslice) < bufend) {
		bcopy(audio,bufp,JPHeader.audioslice);
		bufp += JPHeader.audioslice;
	}
	else {
		i = JPHeader.audioslice; j = bufend - bufp;
		bcopy(audio,bufp,j); audio += j;
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		i -= j; if (i) bcopy(audio,bufp,i); bufp += i;
	}
#else
	if (write(MovieFd,audio,JPHeader.audioslice) != JPHeader.audioslice) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write of audio to JPEG movie file failed");
#else
		SystemError(errno,"Write of audio to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif
	return(0);
}

int jpeg_end_frame(dpy)
{
	u_char c;
	int length;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("No open JPEG movie file exists!",NULL,NULL);
#else
		ShowAlert("No open JPEG movie file exists!",NULL);
#endif
		return(-1);
	}

/* Write the end of movie frame instruction */

#ifdef ASYNCWRITE
	if (bufp != bufend) *bufp++ = END_FRAME;
	else {
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		*bufp++ = END_FRAME;
	}
#else
	c = END_FRAME;
	if (write(MovieFd,&c,1) != 1) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Write of END_FRAME to JPEG movie file failed");
#else
		SystemError(errno,"Write of END_FRAME to JPEG movie file failed");
#endif
		close(MovieFd);
		return(-1);
	}
#endif
	FrameCount++;

/* Record offset to next frame */
#ifdef ASYNCWRITE
	indexbuf[FrameCount] = sizeof(jpheader) + BufOffset + BUFFERSIZE - (bufend - bufp);
#else
	indexbuf[FrameCount] = lseek(MovieFd,0L,SEEK_CUR);
#endif
/* Terminate if reached max frame count */

	if (FrameCount == MAXFRAMES) {
		jpeg_stop_movie(NULL);
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Reached maximum frame count for movie!",NULL,NULL);
#else
		ShowAlert("Reached maximum frame count for movie!",
				  "Ending current movie file.",
				  NULL);
#endif
	}
	return(0);
}

int jpeg_stop_movie(dpy)
Display *dpy;
{
	int value;

	if (NewJPEGMovie == FALSE) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("No open JPEG movie file exists!",NULL,NULL);
#else
		ShowAlert("No open JPEG movie file exists!",NULL);
#endif
		return(-1);
	}

/* Dump the asyncio buffer to the file if required */

#ifdef ASYNCWRITE
	if ((bufend - bufp) != BUFFERSIZE) {
		if (FrameCount) aiowait(NULL);
		if (BufSync) {
			aiowrite(MovieFd,Abuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Bbuffer; bufend = Bbuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
		else {
			aiowrite(MovieFd,Bbuffer,BUFFERSIZE,BufOffset,SEEK_CUR,&AioResult);
			bufp = Abuffer; bufend = Abuffer + BUFFERSIZE;
			BufOffset += BUFFERSIZE; BufSync ^= 1;
		}
	}
#endif

/* Dump out the frame index buffer */
#ifdef ASYNCWRITE
	if (FrameCount) aiowait(NULL);
	JPHeader.indexbuf = BufOffset + sizeof(jpheader);
	value = sizeof(int) * (FrameCount + 1);
	aiowrite(MovieFd,indexbuf,value,BufOffset,SEEK_CUR,&AioResult);
	if (FrameCount) aiowait(NULL);
#else
	JPHeader.indexbuf = lseek(MovieFd,0L,SEEK_CUR);
	write(MovieFd,(char *) indexbuf,(sizeof(int) * FrameCount));
#endif	
/* Update the header info in the file */

	JPHeader.frames = FrameCount;
	if (FrameCount)
		JPHeader.bandwidth = ((JPHeader.bandwidth / FrameCount) * JPHeader.fps) / 1024;
	else JPHeader.bandwidth = 0;

	if (lseek(MovieFd,0L,SEEK_SET) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Reset of MovieFd failed");
#else
		SystemError(errno,"Reset of MovieFd failed");
#endif
		close(MovieFd);
		return(-1);
	}
	
/* Update the header info */

	if (write(MovieFd,JPHeader,sizeof(jpheader)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"JPEG file header rewrite failed!");
#else
		SystemError(errno,"JPEG file header rewrite failed!");
#endif
		close(MovieFd);
		return(-1);
	}
	close(MovieFd);
	FrameCount = 0;
	NewJPEGMovie  = 0;
}

int jpeg_play_movie(dpy,video,filename)
Display *dpy;
VideoIn video;
char *filename;
{
	int fd;
	int offset;
	int audio;
	double vol;

/* Open the compressed video file */

	if ((fd = open(filename,O_RDONLY)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Couldn't open j_movie file");
#else
		SystemError(errno,"Couldn't open j_movie file");
#endif
		return(-1);
	}

/* Load the Uniflix header information */

	if ((offset = read(fd,Abuffer,BUFFERSIZE)) < 0) {
#ifdef HYPERCMEDIA
		hCm_SystemError(errno,"Load of j_movie header information failed");
#else
		SystemError(errno,"Load of j_movie header information failed");
#endif
		close(fd);
		return(-1);
	}
	bcopy(Abuffer,&JPHeader,sizeof(jpheader));

	bufp   = Abuffer + sizeof(jpheader);
	bufend = Abuffer + offset;

/* Verify the file type and version */

	if (strcmp(JPHeader.magic,"j_movie")) {
#ifdef HYPERCMEDIA
		hCm_ShowAlert("Requested file (%s) is not a j_movie file!",filename,NULL);
#else
		ShowAlert("Requested file is not a j_movie file!","",filename,NULL);
#endif
		close(fd); return(-1);
	}
	
/* Open up port for communicating with the audio driver, if needed */

	if (JPHeader.audio.data_size != 0) {
		if ((audio = open("/dev/audio",O_WRONLY|O_NDELAY)) < 0) {
			if (errno == EBUSY) {
#ifdef HYPERCMEDIA
				hCm_ShowAlert("Audio device is currently busy, try again later...",NULL,NULL);
#else
				ShowAlert("Audio device is currently busy, try again later...",NULL);
#endif
				close(fd); return(-1);
			}
			else {
#ifdef HYPERCMEDIA
				hCm_SystemError(errno,"Audio device open failed");
#else
				SystemError(errno,"Audio device open failed");
#endif
				close(fd); return(-1);
			}
		}

/* Make sure audio device matches audio file requirements */

		if (audio_set_play_config(audio,&(JPHeader.audio)) != AUDIO_SUCCESS) {
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Couldn't reset audio device encoding",NULL,NULL);
#else
			ShowAlert("Couldn't reset audio device encoding",NULL);
#endif
			close(audio); close(fd);
			return(-1);
		}

/* Set playback volume (currently disabled)

		vol =  60.0 / 100.0;

		if (audio_set_play_gain(audio,&vol) != AUDIO_SUCCESS) {
#ifdef HYPERCMEDIA
			hCm_ShowAlert("Could not set audio gain!",NULL,NULL);
#else
			ShowAlert("Could not set audio gain!",NULL);
#endif
			close(audio); close(fd);
			return(-1);
		}
*/
	}
	else audio = 0;

/* Set the video object to still frames */

	xv_set(video,
		VIDEO_LIVE,		FALSE,
		NULL);

/* Decode the j_movie */

	jpeg_decode_file(dpy,fd,audio,video);

/* Set the video object to live frames */

	xv_set(video,
		VIDEO_LIVE,		TRUE,
		NULL);

/* Make sure audio buffer has emptied out */

	if (audio) {
		audio_flush_play(audio);
		close(audio);
	}
	close(fd);
}

/* end of jpeg.c */
