#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>

#include <XPlxExt.h>
#ifdef sun
#include <player.h>
#endif
#ifdef ultrix
#include <signal.h>
#include <sys/time.h>
#endif

#include <stdio.h>
#include <sys/param.h>

#include <vtrack.h>

#define ONE_HALF_SIZE    320
#define ONE_QUARTER_SIZE 160
#define ONE_EIGHT_SIZE    80
#define ONE_SIXTEEN_SIZE  40

#ifdef notdef
static Widget videotrack, vpane, buttonbox1, buttonbox2, quit, start, stop, step, play, size, pshell;
#endif
static Widget videotrack, vpane, buttonbox1, buttonbox2, quit, start, stop, fill, size, pshell;
static Widget size1 = NULL;
static Widget size2 = NULL;
static Widget size3 = NULL;
static Widget size4 = NULL;
static GC draw_gc, video_gc, copy_gc, xor_gc;
static Widget bitmap;
static CellDataPtr cell = NULL;
static CellData PlayData[2];
static Dimension pixmap_width_in_pixels, pixmap_height_in_pixels;
static int video_width, video_height, video_start;
static int cur_x, cur_y;
static int select_flag = 0;
static int start_flag = FALSE;
static int sleep_time;
static XtWorkProcId work_proc_id = NULL;
static XtAppContext app_context;

#ifdef ultrix
static int alrm;
#endif

#ifdef sun
static Player *VideoPlayer;
static int PlayerModel;
static int PlayerSpeed;
static char *PlayerDevice;
#endif

extern char *optarg;
extern int optind;

XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display);


typedef struct {
    Pixel copy_fg;
    Pixel copy_bg;
    int pixmap_width_in_cells;
    int pixmap_height_in_cells;
    int cell_size_in_pixels;
    Boolean debug;
} AppData, *AppDataPtr;

AppData app_data;

static XtResource resources[] = {
    {
	XtNforeground,
	XtCForeground,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(AppData, copy_fg),
	XtRString,
	XtDefaultForeground
    },
    {
	XtNbackground,
	XtCBackground,
	XtRPixel,
	sizeof(Pixel),
	XtOffsetOf(AppData, copy_fg),
	XtRString,
	XtDefaultBackground
    },
    {
	XtNpixmapWidthInCells,
	XtCPixmapWidthInCells,
	XtRInt,
	sizeof(int),
	XtOffsetOf(AppData, pixmap_width_in_cells),
	XtRImmediate,
	(XtPointer) 6,
    },
    {
	XtNpixmapHeightInCells,
	XtCPixmapHeightInCells,
	XtRInt,
	sizeof(int),
	XtOffsetOf(AppData, pixmap_height_in_cells),
	XtRImmediate,
	(XtPointer) 4,
    },
    {
	XtNcellSizeInPixels,
	XtCCellSizeInPixels,
	XtRInt,
	sizeof(int),
	XtOffsetOf(AppData, cell_size_in_pixels),
	XtRImmediate,
	(XtPointer) 80,
    },
    {
	XtNdebug,
	XtCDebug,
	XtRBoolean,
	sizeof(Boolean),
	XtOffsetOf(AppData, debug),
	XtRImmediate,
	(XtPointer) FALSE,
    },
};

static XrmOptionDescRec options[] = {
    {"-pw",		"*pixmapWidthInCells",	XrmoptionSepArg, NULL},
    {"-pixmapwidth",	"*pixmapWidthInCells",	XrmoptionSepArg, NULL},
    {"-ph",		"*pixmapHeightInCells",	XrmoptionSepArg, NULL},
    {"-pixmapheight",	"*pixmapHeightInCells",	XrmoptionSepArg, NULL},
    {"-cellsize",	"*cellSizeInPixels",	XrmoptionSepArg, NULL},
    {"-fg",		"*foreground",		XrmoptionSepArg, NULL},
    {"-foreground",	"*foreground",		XrmoptionSepArg, NULL},
    {"-debug",		"*debug",		XrmoptionNoArg,  "True"},
};

#ifdef ultrix
static void
ALHandler ()
{
    alrm = 1;
    return;
}

usleep(time)
int time;
{
    struct itimerval value, ovalue;

    value.it_interval.tv_sec = 0;
    value.it_interval.tv_usec = 0;
    value.it_value.tv_sec = 0;
    value.it_value.tv_usec = time;
    alrm = 0;
    setitimer(ITIMER_REAL, &value, &ovalue);
      while (alrm == 0);
}
#endif
#ifdef SVR4
static void MyUSleep(unsigned int);
#else
static void MyUSleep();
#endif

void QuitProc(Widget	unusedWidget,
	      XtPointer	unusedClientData,
	      XtPointer	unusedCallData)
{
  exit(0);
} /* end function QuitProc */



static Boolean
PutVideoCell(w)
Widget w;
{
    int i, n;
    CellDataPtr cellp;

#ifdef notdef
    if (!start_flag)
	return(FALSE);
#endif
    cellp = cell;
    n = app_data.pixmap_height_in_cells * app_data.pixmap_width_in_cells;
    XPlxVideoTag(XtDisplay(w), XtWindow(bitmap), video_gc, PLX_VIDEO);
    for (i = 0, cellp = cell; i < n; ++i, ++cellp)
    {
	XPlxVideoSqueezeStill(XtDisplay(w), XtWindow(bitmap), video_gc,
				0, video_start, video_width, video_height,
				cellp->x + 1, cellp->y + 1,
				(unsigned int) app_data.cell_size_in_pixels - 4,
				(unsigned int) app_data.cell_size_in_pixels * 3 / 4 - 4);
	XFlush(XtDisplay(w));
	MyUSleep(sleep_time);
    }
    XPlxVideoTag(XtDisplay(w), XtWindow(bitmap), video_gc, PLX_VIDEO_OVR);
    return (FALSE);
}

/*
 * Draw the outline of the cells.
 */
DrawVideoCellOutline()
{
    XSegment segment[MAXLINES];
    int x, y, n_horiz_segments, n_vert_segments;


    XClearWindow(XtDisplay(bitmap), XtWindow(bitmap));

    n_vert_segments = app_data.pixmap_width_in_cells + 1;
    n_horiz_segments = app_data.pixmap_height_in_cells + 1;

    for (x = 0; x < n_horiz_segments; ++x) {
	segment[x].x1 = 0;
	segment[x].x2 = pixmap_width_in_pixels;
	segment[x].y1 = app_data.cell_size_in_pixels * x * 3 / 4;
	segment[x].y2 = app_data.cell_size_in_pixels * x * 3 / 4;
    }

    XDrawSegments(XtDisplay(bitmap), XtWindow(bitmap), draw_gc, segment,
			n_horiz_segments);

    for (y = 0; y < n_vert_segments; ++y) {
	segment[y].x1 = app_data.cell_size_in_pixels * y;
	segment[y].x2 = app_data.cell_size_in_pixels * y;
	segment[y].y1 = 0;
	segment[y].y2 = pixmap_height_in_pixels;
    }

    XDrawSegments(XtDisplay(bitmap), XtWindow(bitmap), draw_gc, segment,
			n_vert_segments);
}

/*
 * Set up the dimension of each cell.
 */
SetUpVideoCell()
{
    CellDataPtr cellp;
    int i, j, x, y;

    cellp = (CellDataPtr) XtRealloc((char*)cell, app_data.pixmap_width_in_cells *
		    app_data.pixmap_height_in_cells * sizeof(CellData));
    cell = cellp;
    x = app_data.cell_size_in_pixels;
    for (i = 0; i < app_data.pixmap_height_in_cells; ++i) {
	y = app_data.cell_size_in_pixels * i * 3 / 4 + 1;
	for (j = 0; j < app_data.pixmap_width_in_cells; ++j) {
	    cellp->x = x * j + 1;
	    cellp->y = y;
	    cellp->frame = i * app_data.pixmap_width_in_cells + j;
	    cellp++;
	}
    }
}

static void
Start(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{

    if (start_flag)
	return;

    if (w == start)
    {
	if (work_proc_id)
	    XtRemoveWorkProc(work_proc_id);

#ifdef OPENLOOK
	work_proc_id = XtAddWorkProc(PutVideoCell, start);
#else
	work_proc_id = XtAppAddWorkProc(app_context, PutVideoCell, start);
#endif

	/*
	 * Ignore further button press after the first one.
	 * This flag will be reset by the Stop procedure.
	 */
	start_flag = TRUE;
    }

    PutVideoCell(w);
}

static void
Stop(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    start_flag = FALSE;
    if (work_proc_id)
    {
	XtRemoveWorkProc(work_proc_id);
	work_proc_id = NULL;
    }
}

#ifdef notdef
static void
Step(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    printf("Step Video Track\n");
}

static void
Play(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    printf("Play Video Track\n");
}
#endif

static void
Change(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    static void Config();

    if (w == size1)
	app_data.cell_size_in_pixels = ONE_HALF_SIZE;
    else if (w == size2)
	app_data.cell_size_in_pixels = ONE_QUARTER_SIZE;
    else if (w == size3)
	app_data.cell_size_in_pixels = ONE_EIGHT_SIZE;
    else if (w == size4)
	app_data.cell_size_in_pixels = ONE_SIXTEEN_SIZE;

    Config(bitmap, NULL, NULL);
}

static void
Done(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    XtPopdown(pshell);
}

static void
Size(w, event, params, num_params)
Widget w;
XEvent *event;
String *params;
Cardinal *num_params;
{
    static Widget done = NULL;

    if (!size1)
    {
	size1 = XtCreateManagedWidget("1/2", commandWidgetClass, buttonbox2, NULL, 0);
	XtAddCallback(size1, XtNcallback, Change, size1);
    }
    if (!size2)
    {
	size2 = XtCreateManagedWidget("1/4", commandWidgetClass, buttonbox2, NULL, 0);
	XtAddCallback(size2, XtNcallback, Change, size2);
    }
    if (!size3)
    {
	size3 = XtCreateManagedWidget("1/8", commandWidgetClass, buttonbox2, NULL, 0);
	XtAddCallback(size3, XtNcallback, Change, size3);
    }
    if (!size4)
    {
	size4 = XtCreateManagedWidget("1/16", commandWidgetClass, buttonbox2, NULL, 0);
	XtAddCallback(size4, XtNcallback, Change, size4);
    }
    if (!done)
    {
	done = XtCreateManagedWidget("done", commandWidgetClass, buttonbox2, NULL, 0);
	XtAddCallback(done, XtNcallback, Done, NULL);
    }

    XtPopup(pshell, XtGrabNonexclusive);
}

static void
Config(w, data, event)
Widget w;
caddr_t data;
XEvent *event;
{
    XWindowAttributes attrib;
    int i, j, new_width_in_cells, new_height_in_cells;

#ifndef HACK
/*
 * Kludge fix before bitmap is mapped.
 */
if (!draw_gc) return;
#endif

    XGetWindowAttributes(XtDisplay(w), XtWindow(w), &attrib);
    new_width_in_cells = attrib.width / app_data.cell_size_in_pixels;
    /*
     * Set the minimual size to be 1.
     */
    if (!new_width_in_cells)
	new_width_in_cells = 1;
    new_height_in_cells = attrib.height / (app_data.cell_size_in_pixels * 3 / 4);
    /*
     * Set the minimual size to be 1.
     */
    if (!new_height_in_cells)
	new_height_in_cells = 1;

    pixmap_width_in_pixels = new_width_in_cells * app_data.cell_size_in_pixels;
    pixmap_height_in_pixels = new_height_in_cells * (app_data.cell_size_in_pixels * 3 / 4);
    app_data.pixmap_width_in_cells = new_width_in_cells;
    app_data.pixmap_height_in_cells = new_height_in_cells;

    XtResizeWidget(w, pixmap_width_in_pixels, pixmap_height_in_pixels, 0);
    XGetWindowAttributes(XtDisplay(buttonbox1), XtWindow(buttonbox1), &attrib);
    XtResizeWidget(videotrack, pixmap_width_in_pixels,
		   pixmap_height_in_pixels + attrib.height + 1, 0);

    DrawVideoCellOutline();
    SetUpVideoCell();
}

static void Syntax(argc, argv)
int argc;
char *argv[];
{
    int i;
    static int errs = False;

    for (i = 1; i < argc; i++) {
	if (!errs++)
	    fprintf(stderr, "xbitmap4: command line option not understood:\n");
	fprintf(stderr, "option: %s\n", argv[i]);
    }

    fprintf(stderr, "Additional options are as follows:\n");
    fprintf(stderr, "Option		Valid Range\n");
    fprintf(stderr, "-pw		MINBITMAPWIDTH to MAXBITMAPWIDTH\n");
    fprintf(stderr, "-pixmapwidth	MINBITMAPWIDTH to MAXBITMAPWIDTH\n");
    fprintf(stderr, "-ph		MINBITMAPHEIGHT to MAXBITMAPHEIGHT\n");
    fprintf(stderr, "-pixmapheight	MINBITMAPHEIGHT to MAXBITMAPHEIGHT\n");
    fprintf(stderr, "-cellsize		MINCELLSIZE to MAXCELLSIZE\n");
    fprintf(stderr, "-fg		color name\n");
    fprintf(stderr, "-foreground	color name\n");
    fprintf(stderr, "-debug		no value necessary\n");
}

set_up_things(w)
Widget w;
{
    XGCValues values;

#define GC_MASK GCForeground | GCBackground | GCDashOffset | GCDashList | \
		GCLineStyle

#ifdef sun
    VideoPlayer = (Player *) InitializeVideo(PlayerDevice, PlayerModel,
					     PlayerSpeed);

    if (VideoPlayer == NULL)
    {
	fprintf(stderr,"InitializeVideo Failed\n");
	exit(1);
    }

    PlayerPlay(VideoPlayer);
#endif
#ifdef ultrix
    signal(SIGALRM, ALHandler);
#endif

    pixmap_width_in_pixels = app_data.pixmap_width_in_cells *
			     app_data.cell_size_in_pixels;
    pixmap_height_in_pixels = app_data.pixmap_height_in_cells *
			      app_data.cell_size_in_pixels * 3 / 4;

    values.foreground = app_data.copy_fg;
    values.background = app_data.copy_bg;
    copy_gc = XCreateGC(XtDisplay(w), RootWindowOfScreen(XtScreen(w)),
			GCForeground | GCBackground, &values);
    SetUpVideoCell();
}

static void
RedrawPicture(w, event, params, num_params)
Widget w;
XExposeEvent *event;
String *params;
Cardinal *num_params;
{
    register int x, y;
    unsigned int width, height;
    static int first_time1 = TRUE;

    if (event) {
        x = event->x;
        y = event->y;
        width = event->width;
        height = event->height;
    } else {
        x = 0;
        y = 0;
        width = 10000;
        height = 10000;
    }

    if (first_time1)
    {
	plx_IO *plxio;
	plx_IO_list *iolist;
	plx_signal_list *siglist;
	plx_signal *sig;
	Window video_win = XtWindow(bitmap);
	XGCValues values;
	int tmp, badsync;

	first_time1 = FALSE;

	values.foreground = 1;
	values.background = 0;
	values.dashes = 1;
	values.dash_offset = 0;
	values.line_style = LineOnOffDash;
	draw_gc = XCreateGC(XtDisplay(w), XtWindow(bitmap), GC_MASK, &values);

	values.foreground = 1;
	values.background = 0;
	values.function = GXxor;
	xor_gc = XCreateGC(XtDisplay(w), XtWindow(bitmap),
			   GCForeground | GCBackground | GCFunction,
			   &values);

	video_gc = XCreateGC(XtDisplay(w), XtWindow(bitmap), 0, 0);

	plxio = (plx_IO *) XPlxQueryConfig(XtDisplay(w), video_win, video_gc);
	if (plxio->hardware != PLX_XVIDEO_24)
	{
	    exit(1);
	}

	for (iolist = plxio->inputs; iolist && (iolist->channel != PLX_INPUT_0);
		iolist = iolist->next_IO);

	badsync = 1;

	for (siglist = iolist->signal_list; siglist;
		siglist = siglist->next_signal)
	{
	    XPlxVideoInputSelect(XtDisplay(w), video_win, video_gc, PLX_INPUT_0,
				 siglist->signal->standard, PLX_COMP, PLX_RGB24);

	    sig = (plx_signal *) XPlxQueryVideo(XtDisplay(w), video_win, video_gc);
	    if (sig->sync_ok)
	    {
		badsync = 0;
		break;
	    }
	}

	if (badsync)
	{
	    fprintf(stderr, "no video connected to input 0\n");
	    exit(1);
	}

	video_width = sig->w;
	tmp = (sig->w * 3) / 4;
	video_height = tmp;
	tmp = sig->h - tmp;
	video_start = tmp + 1;

        DrawVideoCellOutline();

#ifdef notdef
	sleep_time = 70000;
#endif
	if (app_data.cell_size_in_pixels >= 160)
	    sleep_time = 100000;
	else if (app_data.cell_size_in_pixels >= 80)
	    sleep_time = 60000;
	else if (app_data.cell_size_in_pixels >= 40)
	    sleep_time = 40000;
    }
}

static void
HighlightSelectedCell(w, event, params, num_params)
Widget w;
XButtonEvent *event;
String *params;
Cardinal *num_params;
{
    int newx, newy;
    static int first_time = TRUE;
    CellDataPtr cellp;

    newx = (cur_x + event->x) / app_data.cell_size_in_pixels;
    newy = (cur_y + event->y) / (app_data.cell_size_in_pixels * 3 / 4);

    /*
     * need to check for start_flag, if video track is started and stopped
     * allow user to select a beginning and ending clip, also
     * before the user finish the ending clip does disable start and stop
     * button ???
     */
    if (!select_flag && !first_time)
    {
	/*
	 * Undraw the previous selected frame.
	 */
	XDrawRectangle(XtDisplay(w), XtWindow(bitmap), xor_gc, PlayData[0].x,
			PlayData[0].y,
			(unsigned int) app_data.cell_size_in_pixels - 2,
			(unsigned int) app_data.cell_size_in_pixels * 3 / 4 - 2);

	XDrawRectangle(XtDisplay(w), XtWindow(bitmap), xor_gc, PlayData[1].x,
			PlayData[1].y,
			(unsigned int) app_data.cell_size_in_pixels - 2,
			(unsigned int) app_data.cell_size_in_pixels * 3 / 4 - 2);
    }

    if (first_time)
	first_time = FALSE;

    /*
     * Draw the newly selected frame.
     */
    XDrawRectangle(XtDisplay(w), XtWindow(bitmap), xor_gc,
		   app_data.cell_size_in_pixels*newx + 1,
		   app_data.cell_size_in_pixels*newy*3/4 + 1,
		   (unsigned int) app_data.cell_size_in_pixels - 2,
		   (unsigned int) app_data.cell_size_in_pixels * 3 / 4 - 2);

    if (!select_flag)
	cellp = &PlayData[0];
    else
	cellp = &PlayData[1];

    cellp->x = app_data.cell_size_in_pixels*newx + 1;
    cellp->y = app_data.cell_size_in_pixels*newy*3/4 + 1;
    cellp->frame = cell[newx + newy * app_data.pixmap_width_in_cells].frame;
    select_flag = !select_flag;
}

#ifdef notdef
static void
DrawCell(w, event, params, num_params)
Widget w;
XButtonEvent *event;
String *params;
Cardinal *num_params;
{
    DrawPixmaps(draw_gc, DRAW, w, event);
}
#endif

main (argc, argv)
int argc;
char *argv[];
{
    int i;
    char *tmps;
    Arg arglist[10];
    Display *display;
    int screen;
    Colormap colormap;
    XVisualInfo * visualInfo;
    int copy_of_argc;
    char ** copy_of_argv;
    int argv_size;
    int n;
    Arg args[10];

    extern exit();


    String trans =
	"<Expose>:		RedrawPicture()		\n\
	 <Btn1Down>:		HighlightSelectedCell()";

    static XtActionsRec window_actions[] = {
	{"RedrawPicture",		RedrawPicture},
	{"HighlightSelectedCell",	HighlightSelectedCell},
    };

    static String fallbackResources[] =
    {
      "videotrack*Background:  white",
      "videotrack*Foreground:  black"
    };

    PlayerModel = LIVE_VIDEO;
    PlayerSpeed = NULL;
    PlayerDevice = "/dev/null";

    /* Get options */
    while ((i = getopt(argc, argv, "SPNO1249p?d:")) != EOF) {
	switch (i) {
	    case 'p':
#ifdef notdef
		savepid = TRUE;
#endif
		break;
	    case 'S':
		PlayerModel = SONY_IP;
		break;
	    case 'P':
		PlayerModel = PIONEER;
		break;
	    case 'O':
		PlayerModel = PANASONIC;
		break;
	    case 'N':
		PlayerModel = NEC;
		break;
	    case '1':
		PlayerSpeed = 1;
		break;
	    case '2':
		PlayerSpeed = 2;
		break;
	    case '4':
		PlayerSpeed = 3;
		break;
	    case '9':
		PlayerSpeed = 4;
		break;
	    case 'd':
		PlayerDevice = optarg;
		break;
	    case '?':
		fprintf(stderr,"usage: %s [ -speed ] [-Sony -Pioneer -NEC -POD] [-d device]\n", argv[0]);
		exit(1);
		break;

	}
    }

    if ((argc - optind) != 0) {
	fprintf(stderr,"usage: %s [ -speed ] [-Sony -Pioneer -NEC -POD] [-d device]\n", argv[0]);
	exit(1);

    }

#ifdef OPENLOOK
    videotrack = OlInitialize("XBitmap4", "XBitmap4",
				 options, XtNumber(options),
				 &argc, argv);
#else
/*    videotrack = XtVaAppInitialize(&app_context, "Video Track",
				 options, XtNumber(options),
				 &argc, argv,
				 NULL, NULL); */

    XtToolkitInitialize();
    app_context = XtCreateApplicationContext();
    
    copy_of_argc = argc;
    argv_size = argc * (sizeof(char*));
    copy_of_argv = (char**) XtMalloc(argv_size);
    memcpy(copy_of_argv, argv, argv_size);

    XtAppSetFallbackResources(app_context,
                              fallbackResources);
    
    display = XtOpenDisplay(app_context,
			    NULL,
			    NULL,
			    "Video Track",
			    options, XtNumber(options),
			    &argc, argv);
    if((Display *) NULL == display)
      {
	printf("Unable to open local display check permisions\n");
	exit(1);
      }
    
    screen = DefaultScreen(display);
    /*
     * Set up the TrueColor visual:
     */
    
    visualInfo = GetTrueColorVisual(screen, display);
    if(visualInfo == (XVisualInfo*)NULL)
      {
	XtWarning("Could not find the TrueColor visual being requested.");
	exit(-1);
      }
    
    colormap = XCreateColormap(display,
			       XRootWindow(display, screen),
			       visualInfo->visual,
			       AllocNone);
    
    n = 0;
    XtSetArg(args[n], XtNvisual, visualInfo->visual); n++;
    XtSetArg(args[n], XtNdepth, visualInfo->depth); n++;
    XtSetArg(args[n], XtNcolormap, colormap); n++;
    XtSetArg(args[n], XtNargc, copy_of_argc); n++;
    XtSetArg(args[n], XtNargv, copy_of_argv); n++;

    videotrack = XtAppCreateShell("videotrack",
				"Video Track",
				applicationShellWidgetClass,
				display,
				args, n);

#endif
    if (argc > 1)
	Syntax(argc, argv);

    XtGetApplicationResources(videotrack, &app_data,
			      resources, XtNumber(resources),
			      NULL, 0);

    if ((app_data.pixmap_width_in_cells > MAXBITMAPWIDTH) ||
	(app_data.pixmap_width_in_cells < MINBITMAPWIDTH) ||
	(app_data.pixmap_height_in_cells > MAXBITMAPHEIGHT) ||
	(app_data.pixmap_height_in_cells < MINBITMAPHEIGHT)) {
	fprintf(stderr, "xbitmap4: error in resource settings:\
		dimension must be between %d and %d cells\n",
		MINBITMAPWIDTH, MAXBITMAPWIDTH);
	exit(1);
    }

    if ((app_data.cell_size_in_pixels < MINCELLSIZE) ||
	(app_data.cell_size_in_pixels > MAXCELLSIZE)) {
	fprintf(stderr, "xbitmap4: error in resource settings:\
		cell size must be between %d and %d pixels\n",
		MINCELLSIZE, MAXCELLSIZE);
    }

#ifdef notdef
    cell = (CellDataPtr) XtCalloc(app_data.pixmap_width_in_cells *
		    app_data.pixmap_height_in_cells, sizeof(CellData));
#endif

    set_up_things(videotrack);

    if (app_data.debug)
	fprintf(stderr, "xbitmap4: pixmap dimensions are %d by %d\n",
		app_data.pixmap_width_in_cells,
		app_data.pixmap_height_in_cells);

    vpane = XtVaCreateManagedWidget("vpane", panedWidgetClass, videotrack,
				   XtNwidth, pixmap_width_in_pixels, NULL);

    buttonbox1 = XtCreateManagedWidget("buttonbox1", boxWidgetClass, vpane,
				     NULL, 0);

    start = XtCreateManagedWidget("start", commandWidgetClass, buttonbox1,
				NULL, 0);

    XtAddCallback(start, XtNcallback, Start, start);

    stop = XtCreateManagedWidget("stop", commandWidgetClass, buttonbox1,
				NULL, 0);

    XtAddCallback(stop, XtNcallback, Stop, NULL);

#ifdef notdef
    step = XtCreateManagedWidget("step", commandWidgetClass, buttonbox1,
				NULL, 0);

    XtAddCallback(step, XtNcallback, Step, NULL);

    play = XtCreateManagedWidget("play", commandWidgetClass, buttonbox1,
				  NULL, 0);

    XtAddCallback(play, XtNcallback, Play, NULL);
#endif
    fill = XtCreateManagedWidget("fill", commandWidgetClass, buttonbox1,
				  NULL, 0);

    XtAddCallback(fill, XtNcallback, Start, fill);

    size = XtCreateManagedWidget("size", commandWidgetClass, buttonbox1, NULL, 0);

    XtAddCallback(size, XtNcallback, Size, NULL);

#ifdef notdef
    pshell = XtCreatePopupShell("pshell", transientShellWidgetClass, videotrack, NULL, 0);
    printf("notdef -- transientShellWidgetClass\n");
#endif
/*    pshell = XtCreatePopupShell("pshell", shellWidgetClass, videotrack, NULL, 0); */
    n = 0;
    XtSetArg(args[n], XtNvisual, visualInfo->visual); n++;
    XtSetArg(args[n], XtNdepth, visualInfo->depth); n++;
    XtSetArg(args[n], XtNcolormap, colormap); n++;

    pshell = XtCreatePopupShell("pshell", transientShellWidgetClass, videotrack, args, n);


    XtSetArg(arglist[0], XtNtitle,(XtArgVal) "SCROLLINGLIST");
    buttonbox2 = XtCreateManagedWidget("buttonbox2", boxWidgetClass, pshell,
				     arglist, 1);

    quit = XtCreateManagedWidget("quit", commandWidgetClass, buttonbox1,
				NULL, 0);

    XtAddCallback(quit, XtNcallback, QuitProc, NULL);

    bitmap = XtVaCreateManagedWidget("bitmap", widgetClass, vpane,
				    XtNtranslations,
				    XtParseTranslationTable(trans),
				    XtNwidth, pixmap_width_in_pixels,
				    XtNheight, pixmap_height_in_pixels, NULL);
    XtAddEventHandler(bitmap, StructureNotifyMask, False, Config, bitmap);
#ifdef OPENLOOK
    XtAddActions(window_actions, XtNumber(window_actions));
#else
    XtAppAddActions(app_context, window_actions, XtNumber(window_actions));
#endif
    XtRealizeWidget(videotrack);
#ifdef OPENLOOK
/*
    XtAddWorkProc(PutVideoCell, start);
*/
    XtMainLoop();
#else
/*
    XtAppAddWorkProc(app_context, PutVideoCell, start);
*/
    XtAppMainLoop(app_context);
#endif
}


static void MyUSleep(microseconds)
     unsigned int microseconds;
     
{
#ifdef SVR4
  int			descriptorTableSize = 0;
  int			result;
  struct timeval	waitTime;				    /* Used to tell select() how long to wait until it returns */
  
  descriptorTableSize = sysconf(_SC_OPEN_MAX);			    /* Get size of descriptor table for later use by select() */
  waitTime.tv_sec = 0;						    /* Set up a timeval to wait for the time passed in as... */
  waitTime.tv_usec = microseconds;				    /* ...argument. */
  result = select(descriptorTableSize,				    /* Use select to sleep by not asking to be informed of... */
		      (fd_set*)0, (fd_set*)0,(fd_set*)0,	    /* ...input on any file descriptors, and filling in the... */
		      &waitTime);				    /* ...timeval for the wait period. */
  return;
#else
  usleep(microseconds);
  return;
#endif  
} /* end function MyUSleep */


XVisualInfo* GetTrueColorVisual(int 		screenNumber,
				Display*	display)
{
  int			i;
  long			visualInfoMask;
  int			visualsMatched;
  XVisualInfo		visualHints;
  static XVisualInfo*	visualList = (XVisualInfo*)NULL;
  XVisualInfo*		theVisual;
  
  if (visualList != (XVisualInfo*)NULL)				    /* Was data saved by a previous call to this function? */
    XFree((void*)visualList);					    /* Yes, release the data since it is no longer needed */
  visualHints.class = TrueColor;				    /* Set up hints to determine the type of visual needed... */
  visualHints.screen = screenNumber;				    /* ...to create a window that can display video */
#ifdef __hpux
  visualInfoMask = (VisualScreenMask);				    /* For the HP product, don't actually need a TrueColor class */
#else
  visualInfoMask = (VisualClassMask | VisualScreenMask);	    /* For Sun, need to get real TrueColor class */
#endif
  visualList = XGetVisualInfo(display,				    /* Try to find a visual that matches what is needed for... */
			      visualInfoMask,			    /* ... a window that will display video or TrueColor... */
			      &visualHints,			    /* ...graphics. */
			      &visualsMatched);
  if (visualsMatched == 0)					    /* Did the search for a TrueColor visual fail? */
    return((XVisualInfo*)NULL);					    /* Yes, return a NULL structure to represent failure */
  for (theVisual = visualList, i = 0;				    /* Look at all the visuals returned by XGetVisualInfo... */
       i < visualsMatched;					    /* ...to find the best match.  The idea match would be... */
       i++, theVisual++)					    /* ...a 24-bit depth Visual. */
  {
    if (theVisual->depth == 24)					    /* Was an ideal match found? */
      return(theVisual);					    /* Yes, return this Visual */
  }
  for (theVisual = visualList, i = 0;				    /* If this statement is reached, no 24-bit TrueColor... */
       i < visualsMatched;					    /* ...Visual was found; in this case, fallback and try... */
       i++, theVisual++)					    /* ...to find the next best thing: any TrueColor visual... */
  {								    /* ...of depth >= 24 bits (probably a 32-bit is what will... */
    if (theVisual->depth >= 24)					    /* ...be found). */
      return(theVisual);					    /* Was a good fallback Visual found?  If so, return it. */
  }
  return(visualList);						    /* If this statement was reached, no good fallback was... */
								    /* ...found.  In this case, simply return the first... */
								    /* ...TrueColor visual found. */
}								    /* end function GetTrueColorVisual */
